<?php
/**
 * This file is part of the ForkBB <https://github.com/forkbb>.
 *
 * @copyright (c) Visman <mio.visman@yandex.ru, https://github.com/MioVisman>
 * @license   The MIT License (MIT)
 */

declare(strict_types=1);

/**
 * Массив подмены символов
 * Используется в методе app\Core\Func\friendly() в случае:
 * 1. если конфиг FRIENDLY_URL['translit'] содержит строку правил - тогда подмена символов производится до Transliterator::transliterate()
 * 2. если конфиг FRIENDLY_URL['translit'] равен true - тогда подмена символов выполняется без вызова Transliterator::transliterate()
 */
return [
    "'" => '',
    '’' => '',
    '@' => 'at',

    // дополнение к латинице — 1
    'À' => 'A', // #c0
    'Á' => 'A', // #c1
    'Â' => 'A', // #c2
    'Ã' => 'A', // #c3
    'Ä' => 'A', // #c4
    'Å' => 'A', // #c5
    'Æ' => 'AE', // #c6
    'Ç' => 'C', // #c7
    'È' => 'E', // #c8
    'É' => 'E', // #c9
    'Ê' => 'E', // #ca
    'Ë' => 'E', // #cb
    'Ì' => 'I', // #cc
    'Í' => 'I', // #cd
    'Î' => 'I', // #ce
    'Ï' => 'I', // #cf
    'Ð' => 'D', // #d0
    'Ñ' => 'N', // #d1
    'Ò' => 'O', // #d2
    'Ó' => 'O', // #d3
    'Ô' => 'O', // #d4
    'Õ' => 'O', // #d5
    'Ö' => 'O', // #d6
    '×' => 'x', // #d7
    'Ø' => 'O', // #d8
    'Ù' => 'U', // #d9
    'Ú' => 'U', // #da
    'Û' => 'U', // #db
    'Ü' => 'U', // #dc
    'Ý' => 'Y', // #dd
    'Þ' => 'TH', // #de
    'ß' => 'ss', // #df
    'à' => 'a', // #e0
    'á' => 'a', // #e1
    'â' => 'a', // #e2
    'ã' => 'a', // #e3
    'ä' => 'a', // #e4
    'å' => 'a', // #e5
    'æ' => 'ae', // #e6
    'ç' => 'c', // #e7
    'è' => 'e', // #e8
    'é' => 'e', // #e9
    'ê' => 'e', // #ea
    'ë' => 'e', // #eb
    'ì' => 'i', // #ec
    'í' => 'i', // #ed
    'î' => 'i', // #ee
    'ï' => 'i', // #ef
    'ð' => 'd', // #f0
    'ñ' => 'n', // #f1
    'ò' => 'o', // #f2
    'ó' => 'o', // #f3
    'ô' => 'o', // #f4
    'õ' => 'o', // #f5
    'ö' => 'o', // #f6
    'ø' => 'o', // #f8
    'ù' => 'u', // #f9
    'ú' => 'u', // #fa
    'û' => 'u', // #fb
    'ü' => 'u', // #fc
    'ý' => 'y', // #fd
    'þ' => 'th', // #fe
    'ÿ' => 'y', // #ff
    // расширенная латиница — A
    'Ā' => 'A', // #100
    'ā' => 'a', // #101
    'Ă' => 'A', // #102
    'ă' => 'a', // #103
    'Ą' => 'A', // #104
    'ą' => 'a', // #105
    'Ć' => 'C', // #106
    'ć' => 'c', // #107
    'Ĉ' => 'C', // #108
    'ĉ' => 'c', // #109
    'Ċ' => 'C', // #10a
    'ċ' => 'c', // #10b
    'Č' => 'C', // #10c
    'č' => 'c', // #10d
    'Ď' => 'D', // #10e
    'ď' => 'd', // #10f
    'Đ' => 'D', // #110
    'đ' => 'd', // #111
    'Ē' => 'E', // #112
    'ē' => 'e', // #113
    'Ĕ' => 'E', // #114
    'ĕ' => 'e', // #115
    'Ė' => 'E', // #116
    'ė' => 'e', // #117
    'Ę' => 'E', // #118
    'ę' => 'e', // #119
    'Ě' => 'E', // #11a
    'ě' => 'e', // #11b
    'Ĝ' => 'G', // #11c
    'ĝ' => 'g', // #11d
    'Ğ' => 'G', // #11e
    'ğ' => 'g', // #11f
    'Ġ' => 'G', // #120
    'ġ' => 'g', // #121
    'Ģ' => 'G', // #122
    'ģ' => 'g', // #123
    'Ĥ' => 'H', // #124
    'ĥ' => 'h', // #125
    'Ħ' => 'H', // #126
    'ħ' => 'h', // #127
    'Ĩ' => 'I', // #128
    'ĩ' => 'i', // #129
    'Ī' => 'I', // #12a
    'ī' => 'i', // #12b
    'Ĭ' => 'I', // #12c
    'ĭ' => 'i', // #12d
    'Į' => 'I', // #12e
    'į' => 'i', // #12f
    'İ' => 'I', // #130
    'ı' => 'i', // #131
    'Ĳ' => 'IJ', // #132
    'ĳ' => 'ij', // #133
    'Ĵ' => 'J', // #134
    'ĵ' => 'j', // #135
    'Ķ' => 'K', // #136
    'ķ' => 'k', // #137
    'ĸ' => 'q', // #138
    'Ĺ' => 'L', // #139
    'ĺ' => 'l', // #13a
    'Ļ' => 'L', // #13b
    'ļ' => 'l', // #13c
    'Ľ' => 'L', // #13d
    'ľ' => 'l', // #13e
    'Ŀ' => 'L', // #13f
    'ŀ' => 'l', // #140
    'Ł' => 'L', // #141
    'ł' => 'l', // #142
    'Ń' => 'N', // #143
    'ń' => 'n', // #144
    'Ņ' => 'N', // #145
    'ņ' => 'n', // #146
    'Ň' => 'N', // #147
    'ň' => 'n', // #148
    'ŉ' => 'n', // #149
    'Ŋ' => 'N', // #14a
    'ŋ' => 'n', // #14b
    'Ō' => 'O', // #14c
    'ō' => 'o', // #14d
    'Ŏ' => 'O', // #14e
    'ŏ' => 'o', // #14f
    'Ő' => 'O', // #150
    'ő' => 'o', // #151
    'Œ' => 'OE', // #152
    'œ' => 'oe', // #153
    'Ŕ' => 'R', // #154
    'ŕ' => 'r', // #155
    'Ŗ' => 'R', // #156
    'ŗ' => 'r', // #157
    'Ř' => 'R', // #158
    'ř' => 'r', // #159
    'Ś' => 'S', // #15a
    'ś' => 's', // #15b
    'Ŝ' => 'S', // #15c
    'ŝ' => 's', // #15d
    'Ş' => 'S', // #15e
    'ş' => 's', // #15f
    'Š' => 'S', // #160
    'š' => 's', // #161
    'Ţ' => 'T', // #162
    'ţ' => 't', // #163
    'Ť' => 'T', // #164
    'ť' => 't', // #165
    'Ŧ' => 'T', // #166
    'ŧ' => 't', // #167
    'Ũ' => 'U', // #168
    'ũ' => 'u', // #169
    'Ū' => 'U', // #16a
    'ū' => 'u', // #16b
    'Ŭ' => 'U', // #16c
    'ŭ' => 'u', // #16d
    'Ů' => 'U', // #16e
    'ů' => 'u', // #16f
    'Ű' => 'U', // #170
    'ű' => 'u', // #171
    'Ų' => 'U', // #172
    'ų' => 'u', // #173
    'Ŵ' => 'W', // #174
    'ŵ' => 'w', // #175
    'Ŷ' => 'Y', // #176
    'ŷ' => 'y', // #177
    'Ÿ' => 'Y', // #178
    'Ź' => 'Z', // #179
    'ź' => 'z', // #17a
    'Ż' => 'Z', // #17b
    'ż' => 'z', // #17c
    'Ž' => 'Z', // #17d
    'ž' => 'z', // #17e
    'ſ' => 's', // #17f
    // расширенная латиница — B
    'ƀ' => 'b', // #180
    'Ɓ' => 'B', // #181
    'Ƃ' => 'B', // #182
    'ƃ' => 'b', // #183
    'Ƈ' => 'C', // #187
    'ƈ' => 'c', // #188
    'Ɖ' => 'D', // #189
    'Ɗ' => 'D', // #18a
    'Ƌ' => 'D', // #18b
    'ƌ' => 'd', // #18c
    'Ɛ' => 'E', // #190
    'Ƒ' => 'F', // #191
    'ƒ' => 'f', // #192
    'Ɠ' => 'G', // #193
    'ƕ' => 'hv', // #195
    'Ɩ' => 'I', // #196
    'Ɨ' => 'I', // #197
    'Ƙ' => 'K', // #198
    'ƙ' => 'k', // #199
    'ƚ' => 'l', // #19a
    'Ɲ' => 'N', // #19d
    'ƞ' => 'n', // #19e
    'Ơ' => 'O', // #1a0
    'ơ' => 'o', // #1a1
    'Ƣ' => 'OI', // #1a2
    'ƣ' => 'oi', // #1a3
    'Ƥ' => 'P', // #1a4
    'ƥ' => 'p', // #1a5
    'ƫ' => 't', // #1ab
    'Ƭ' => 'T', // #1ac
    'ƭ' => 't', // #1ad
    'Ʈ' => 'T', // #1ae
    'Ư' => 'U', // #1af
    'ư' => 'u', // #1b0
    'Ʋ' => 'V', // #1b2
    'Ƴ' => 'Y', // #1b3
    'ƴ' => 'y', // #1b4
    'Ƶ' => 'Z', // #1b5
    'ƶ' => 'z', // #1b6
    'Ǆ' => 'DZ', // #1c4
    'ǅ' => 'Dz', // #1c5
    'ǆ' => 'dz', // #1c6
    'Ǉ' => 'LJ', // #1c7
    'ǈ' => 'Lj', // #1c8
    'ǉ' => 'lj', // #1c9
    'Ǌ' => 'NJ', // #1ca
    'ǋ' => 'Nj', // #1cb
    'ǌ' => 'nj', // #1cc
    'Ǎ' => 'A', // #1cd
    'ǎ' => 'a', // #1ce
    'Ǐ' => 'I', // #1cf
    'ǐ' => 'i', // #1d0
    'Ǒ' => 'O', // #1d1
    'ǒ' => 'o', // #1d2
    'Ǔ' => 'U', // #1d3
    'ǔ' => 'u', // #1d4
    'Ǖ' => 'U', // #1d5
    'ǖ' => 'u', // #1d6
    'Ǘ' => 'U', // #1d7
    'ǘ' => 'u', // #1d8
    'Ǚ' => 'U', // #1d9
    'ǚ' => 'u', // #1da
    'Ǜ' => 'U', // #1db
    'ǜ' => 'u', // #1dc
    'Ǟ' => 'A', // #1de
    'ǟ' => 'a', // #1df
    'Ǡ' => 'A', // #1e0
    'ǡ' => 'a', // #1e1
    'Ǣ' => 'AE', // #1e2
    'ǣ' => 'ae', // #1e3
    'Ǥ' => 'G', // #1e4
    'ǥ' => 'g', // #1e5
    'Ǧ' => 'G', // #1e6
    'ǧ' => 'g', // #1e7
    'Ǩ' => 'K', // #1e8
    'ǩ' => 'k', // #1e9
    'Ǫ' => 'O', // #1ea
    'ǫ' => 'o', // #1eb
    'Ǭ' => 'O', // #1ec
    'ǭ' => 'o', // #1ed
    'Ǯ' => 'Dj', // #1ee
    'ǯ' => 'dj', // #1ef
    'ǰ' => 'j', // #1f0
    'Ǳ' => 'DZ', // #1f1
    'ǲ' => 'Dz', // #1f2
    'ǳ' => 'dz', // #1f3
    'Ǵ' => 'G', // #1f4
    'ǵ' => 'g', // #1f5
    'Ǹ' => 'N', // #1f8
    'ǹ' => 'n', // #1f9
    'Ǻ' => 'A', // #1fa
    'ǻ' => 'a', // #1fb
    'Ǽ' => 'AE', // #1fc
    'ǽ' => 'ae', // #1fd
    'Ǿ' => 'O', // #1fe
    'ǿ' => 'o', // #1ff
    'Ȁ' => 'A', // #200
    'ȁ' => 'a', // #201
    'Ȃ' => 'A', // #202
    'ȃ' => 'a', // #203
    'Ȅ' => 'E', // #204
    'ȅ' => 'e', // #205
    'Ȇ' => 'E', // #206
    'ȇ' => 'e', // #207
    'Ȉ' => 'I', // #208
    'ȉ' => 'i', // #209
    'Ȋ' => 'I', // #20a
    'ȋ' => 'i', // #20b
    'Ȍ' => 'O', // #20c
    'ȍ' => 'o', // #20d
    'Ȏ' => 'O', // #20e
    'ȏ' => 'o', // #20f
    'Ȑ' => 'R', // #210
    'ȑ' => 'r', // #211
    'Ȓ' => 'R', // #212
    'ȓ' => 'r', // #213
    'Ȕ' => 'U', // #214
    'ȕ' => 'u', // #215
    'Ȗ' => 'U', // #216
    'ȗ' => 'u', // #217
    'Ș' => 'S', // #218
    'ș' => 's', // #219
    'Ț' => 'T', // #21a
    'ț' => 't', // #21b
    'Ȟ' => 'H', // #21e
    'ȟ' => 'h', // #21f
    'ȡ' => 'd', // #221
    'Ȥ' => 'Z', // #224
    'ȥ' => 'z', // #225
    'Ȧ' => 'A', // #226
    'ȧ' => 'a', // #227
    'Ȩ' => 'E', // #228
    'ȩ' => 'e', // #229
    'Ȫ' => 'O', // #22a
    'ȫ' => 'o', // #22b
    'Ȭ' => 'O', // #22c
    'ȭ' => 'o', // #22d
    'Ȯ' => 'O', // #22e
    'ȯ' => 'o', // #22f
    'Ȱ' => 'O', // #230
    'ȱ' => 'o', // #231
    'Ȳ' => 'Y', // #232
    'ȳ' => 'y', // #233
    'ȴ' => 'l', // #234
    'ȵ' => 'n', // #235
    'ȶ' => 't', // #236
    'ȷ' => 'j', // #237
    'ȸ' => 'db', // #238
    'ȹ' => 'qp', // #239
    'Ⱥ' => 'A', // #23a
    'Ȼ' => 'C', // #23b
    'ȼ' => 'c', // #23c
    'Ƚ' => 'L', // #23d
    'Ⱦ' => 'T', // #23e
    'ȿ' => 's', // #23f
    'ɀ' => 'z', // #240
    'Ƀ' => 'B', // #243
    'Ʉ' => 'U', // #244
    'Ɇ' => 'E', // #246
    'ɇ' => 'e', // #247
    'Ɉ' => 'J', // #248
    'ɉ' => 'j', // #249
    'Ɍ' => 'R', // #24c
    'ɍ' => 'r', // #24d
    'Ɏ' => 'Y', // #24e
    'ɏ' => 'y', // #24f
    // расширения МФА
    'ɓ' => 'b', // #253
    'ɕ' => 'c', // #255
    'ɖ' => 'd', // #256
    'ɗ' => 'd', // #257
    'ɛ' => 'e', // #25b
    'ɟ' => 'j', // #25f
    'ɠ' => 'g', // #260
    'ɡ' => 'g', // #261
    'ɢ' => 'G', // #262
    'ɦ' => 'h', // #266
    'ɧ' => 'h', // #267
    'ɨ' => 'i', // #268
    'ɪ' => 'I', // #26a
    'ɫ' => 'l', // #26b
    'ɬ' => 'l', // #26c
    'ɭ' => 'l', // #26d
    'ɱ' => 'm', // #271
    'ɲ' => 'n', // #272
    'ɳ' => 'n', // #273
    'ɴ' => 'N', // #274
    'ɶ' => 'OE', // #276
    'ɼ' => 'r', // #27c
    'ɽ' => 'r', // #27d
    'ɾ' => 'r', // #27e
    'ʀ' => 'R', // #280
    'ʂ' => 's', // #282
    'ʈ' => 't', // #288
    'ʉ' => 'u', // #289
    'ʋ' => 'v', // #28b
    'ʏ' => 'Y', // #28f
    'ʐ' => 'z', // #290
    'ʑ' => 'z', // #291
    'ʙ' => 'B', // #299
    'ʛ' => 'G', // #29b
    'ʜ' => 'H', // #29c
    'ʝ' => 'j', // #29d
    'ʟ' => 'L', // #29f
    'ʠ' => 'q', // #2a0
    'ʣ' => 'dz', // #2a3
    'ʥ' => 'dz', // #2a5
    'ʦ' => 'ts', // #2a6
    'ʪ' => 'ls', // #2aa
    'ʫ' => 'lz', // #2ab
    // фонетические расширения
    'ᴀ' => 'A', // #1d00
    'ᴁ' => 'AE', // #1d01
    'ᴃ' => 'B', // #1d03
    'ᴄ' => 'C', // #1d04
    'ᴅ' => 'D', // #1d05
    'ᴆ' => 'D', // #1d06
    'ᴇ' => 'E', // #1d07
    'ᴊ' => 'J', // #1d0a
    'ᴋ' => 'K', // #1d0b
    'ᴌ' => 'L', // #1d0c
    'ᴍ' => 'M', // #1d0d
    'ᴏ' => 'O', // #1d0f
    'ᴘ' => 'P', // #1d18
    'ᴛ' => 'T', // #1d1b
    'ᴜ' => 'U', // #1d1c
    'ᴠ' => 'V', // #1d20
    'ᴡ' => 'W', // #1d21
    'ᴢ' => 'Z', // #1d22
    'ᵫ' => 'ue', // #1d6b
    'ᵬ' => 'b', // #1d6c
    'ᵭ' => 'd', // #1d6d
    'ᵮ' => 'f', // #1d6e
    'ᵯ' => 'm', // #1d6f
    'ᵰ' => 'n', // #1d70
    'ᵱ' => 'p', // #1d71
    'ᵲ' => 'r', // #1d72
    'ᵳ' => 'r', // #1d73
    'ᵴ' => 's', // #1d74
    'ᵵ' => 't', // #1d75
    'ᵶ' => 'z', // #1d76
    'ᵺ' => 'th', // #1d7a
    'ᵻ' => 'I', // #1d7b
    'ᵽ' => 'p', // #1d7d
    'ᵾ' => 'U', // #1d7e
    // дополнения к фонетическим расширениям
    'ᶀ' => 'b', // #1d80
    'ᶁ' => 'd', // #1d81
    'ᶂ' => 'f', // #1d82
    'ᶃ' => 'g', // #1d83
    'ᶄ' => 'k', // #1d84
    'ᶅ' => 'l', // #1d85
    'ᶆ' => 'm', // #1d86
    'ᶇ' => 'n', // #1d87
    'ᶈ' => 'p', // #1d88
    'ᶉ' => 'r', // #1d89
    'ᶊ' => 's', // #1d8a
    'ᶌ' => 'v', // #1d8c
    'ᶍ' => 'x', // #1d8d
    'ᶎ' => 'z', // #1d8e
    'ᶏ' => 'a', // #1d8f
    'ᶑ' => 'd', // #1d91
    'ᶒ' => 'e', // #1d92
    'ᶓ' => 'e', // #1d93
    'ᶖ' => 'i', // #1d96
    'ᶙ' => 'u', // #1d99
    // дополнительная расширенная латиница
    'Ḁ' => 'A', // #1e00
    'ḁ' => 'a', // #1e01
    'Ḃ' => 'B', // #1e02
    'ḃ' => 'b', // #1e03
    'Ḅ' => 'B', // #1e04
    'ḅ' => 'b', // #1e05
    'Ḇ' => 'B', // #1e06
    'ḇ' => 'b', // #1e07
    'Ḉ' => 'C', // #1e08
    'ḉ' => 'c', // #1e09
    'Ḋ' => 'D', // #1e0a
    'ḋ' => 'd', // #1e0b
    'Ḍ' => 'D', // #1e0c
    'ḍ' => 'd', // #1e0d
    'Ḏ' => 'D', // #1e0e
    'ḏ' => 'd', // #1e0f
    'Ḑ' => 'D', // #1e10
    'ḑ' => 'd', // #1e11
    'Ḓ' => 'D', // #1e12
    'ḓ' => 'd', // #1e13
    'Ḕ' => 'E', // #1e14
    'ḕ' => 'e', // #1e15
    'Ḗ' => 'E', // #1e16
    'ḗ' => 'e', // #1e17
    'Ḙ' => 'E', // #1e18
    'ḙ' => 'e', // #1e19
    'Ḛ' => 'E', // #1e1a
    'ḛ' => 'e', // #1e1b
    'Ḝ' => 'E', // #1e1c
    'ḝ' => 'e', // #1e1d
    'Ḟ' => 'F', // #1e1e
    'ḟ' => 'f', // #1e1f
    'Ḡ' => 'G', // #1e20
    'ḡ' => 'g', // #1e21
    'Ḣ' => 'H', // #1e22
    'ḣ' => 'h', // #1e23
    'Ḥ' => 'H', // #1e24
    'ḥ' => 'h', // #1e25
    'Ḧ' => 'H', // #1e26
    'ḧ' => 'h', // #1e27
    'Ḩ' => 'H', // #1e28
    'ḩ' => 'h', // #1e29
    'Ḫ' => 'H', // #1e2a
    'ḫ' => 'h', // #1e2b
    'Ḭ' => 'I', // #1e2c
    'ḭ' => 'i', // #1e2d
    'Ḯ' => 'I', // #1e2e
    'ḯ' => 'i', // #1e2f
    'Ḱ' => 'K', // #1e30
    'ḱ' => 'k', // #1e31
    'Ḳ' => 'K', // #1e32
    'ḳ' => 'k', // #1e33
    'Ḵ' => 'K', // #1e34
    'ḵ' => 'k', // #1e35
    'Ḷ' => 'L', // #1e36
    'ḷ' => 'l', // #1e37
    'Ḹ' => 'L', // #1e38
    'ḹ' => 'l', // #1e39
    'Ḻ' => 'L', // #1e3a
    'ḻ' => 'l', // #1e3b
    'Ḽ' => 'L', // #1e3c
    'ḽ' => 'l', // #1e3d
    'Ḿ' => 'M', // #1e3e
    'ḿ' => 'm', // #1e3f
    'Ṁ' => 'M', // #1e40
    'ṁ' => 'm', // #1e41
    'Ṃ' => 'M', // #1e42
    'ṃ' => 'm', // #1e43
    'Ṅ' => 'N', // #1e44
    'ṅ' => 'n', // #1e45
    'Ṇ' => 'N', // #1e46
    'ṇ' => 'n', // #1e47
    'Ṉ' => 'N', // #1e48
    'ṉ' => 'n', // #1e49
    'Ṋ' => 'N', // #1e4a
    'ṋ' => 'n', // #1e4b
    'Ṍ' => 'O', // #1e4c
    'ṍ' => 'o', // #1e4d
    'Ṏ' => 'O', // #1e4e
    'ṏ' => 'o', // #1e4f
    'Ṑ' => 'O', // #1e50
    'ṑ' => 'o', // #1e51
    'Ṓ' => 'O', // #1e52
    'ṓ' => 'o', // #1e53
    'Ṕ' => 'P', // #1e54
    'ṕ' => 'p', // #1e55
    'Ṗ' => 'P', // #1e56
    'ṗ' => 'p', // #1e57
    'Ṙ' => 'R', // #1e58
    'ṙ' => 'r', // #1e59
    'Ṛ' => 'R', // #1e5a
    'ṛ' => 'r', // #1e5b
    'Ṝ' => 'R', // #1e5c
    'ṝ' => 'r', // #1e5d
    'Ṟ' => 'R', // #1e5e
    'ṟ' => 'r', // #1e5f
    'Ṡ' => 'S', // #1e60
    'ṡ' => 's', // #1e61
    'Ṣ' => 'S', // #1e62
    'ṣ' => 's', // #1e63
    'Ṥ' => 'S', // #1e64
    'ṥ' => 's', // #1e65
    'Ṧ' => 'S', // #1e66
    'ṧ' => 's', // #1e67
    'Ṩ' => 'S', // #1e68
    'ṩ' => 's', // #1e69
    'Ṫ' => 'T', // #1e6a
    'ṫ' => 't', // #1e6b
    'Ṭ' => 'T', // #1e6c
    'ṭ' => 't', // #1e6d
    'Ṯ' => 'T', // #1e6e
    'ṯ' => 't', // #1e6f
    'Ṱ' => 'T', // #1e70
    'ṱ' => 't', // #1e71
    'Ṳ' => 'U', // #1e72
    'ṳ' => 'u', // #1e73
    'Ṵ' => 'U', // #1e74
    'ṵ' => 'u', // #1e75
    'Ṷ' => 'U', // #1e76
    'ṷ' => 'u', // #1e77
    'Ṹ' => 'U', // #1e78
    'ṹ' => 'u', // #1e79
    'Ṻ' => 'U', // #1e7a
    'ṻ' => 'u', // #1e7b
    'Ṽ' => 'V', // #1e7c
    'ṽ' => 'v', // #1e7d
    'Ṿ' => 'V', // #1e7e
    'ṿ' => 'v', // #1e7f
    'Ẁ' => 'W', // #1e80
    'ẁ' => 'w', // #1e81
    'Ẃ' => 'W', // #1e82
    'ẃ' => 'w', // #1e83
    'Ẅ' => 'W', // #1e84
    'ẅ' => 'w', // #1e85
    'Ẇ' => 'W', // #1e86
    'ẇ' => 'w', // #1e87
    'Ẉ' => 'W', // #1e88
    'ẉ' => 'w', // #1e89
    'Ẋ' => 'X', // #1e8a
    'ẋ' => 'x', // #1e8b
    'Ẍ' => 'X', // #1e8c
    'ẍ' => 'x', // #1e8d
    'Ẏ' => 'Y', // #1e8e
    'ẏ' => 'y', // #1e8f
    'Ẑ' => 'Z', // #1e90
    'ẑ' => 'z', // #1e91
    'Ẓ' => 'Z', // #1e92
    'ẓ' => 'z', // #1e93
    'Ẕ' => 'Z', // #1e94
    'ẕ' => 'z', // #1e95
    'ẖ' => 'h', // #1e96
    'ẗ' => 't', // #1e97
    'ẘ' => 'w', // #1e98
    'ẙ' => 'y', // #1e99
    'ẚ' => 'a', // #1e9a
    'ẛ' => 's', // #1e9b
    'ẜ' => 's', // #1e9c
    'ẝ' => 's', // #1e9d
    'ẞ' => 'SS', // #1e9e
    'Ạ' => 'A', // #1ea0
    'ạ' => 'a', // #1ea1
    'Ả' => 'A', // #1ea2
    'ả' => 'a', // #1ea3
    'Ấ' => 'A', // #1ea4
    'ấ' => 'a', // #1ea5
    'Ầ' => 'A', // #1ea6
    'ầ' => 'a', // #1ea7
    'Ẩ' => 'A', // #1ea8
    'ẩ' => 'a', // #1ea9
    'Ẫ' => 'A', // #1eaa
    'ẫ' => 'a', // #1eab
    'Ậ' => 'A', // #1eac
    'ậ' => 'a', // #1ead
    'Ắ' => 'A', // #1eae
    'ắ' => 'a', // #1eaf
    'Ằ' => 'A', // #1eb0
    'ằ' => 'a', // #1eb1
    'Ẳ' => 'A', // #1eb2
    'ẳ' => 'a', // #1eb3
    'Ẵ' => 'A', // #1eb4
    'ẵ' => 'a', // #1eb5
    'Ặ' => 'A', // #1eb6
    'ặ' => 'a', // #1eb7
    'Ẹ' => 'E', // #1eb8
    'ẹ' => 'e', // #1eb9
    'Ẻ' => 'E', // #1eba
    'ẻ' => 'e', // #1ebb
    'Ẽ' => 'E', // #1ebc
    'ẽ' => 'e', // #1ebd
    'Ế' => 'E', // #1ebe
    'ế' => 'e', // #1ebf
    'Ề' => 'E', // #1ec0
    'ề' => 'e', // #1ec1
    'Ể' => 'E', // #1ec2
    'ể' => 'e', // #1ec3
    'Ễ' => 'E', // #1ec4
    'ễ' => 'e', // #1ec5
    'Ệ' => 'E', // #1ec6
    'ệ' => 'e', // #1ec7
    'Ỉ' => 'I', // #1ec8
    'ỉ' => 'i', // #1ec9
    'Ị' => 'I', // #1eca
    'ị' => 'i', // #1ecb
    'Ọ' => 'O', // #1ecc
    'ọ' => 'o', // #1ecd
    'Ỏ' => 'O', // #1ece
    'ỏ' => 'o', // #1ecf
    'Ố' => 'O', // #1ed0
    'ố' => 'o', // #1ed1
    'Ồ' => 'O', // #1ed2
    'ồ' => 'o', // #1ed3
    'Ổ' => 'O', // #1ed4
    'ổ' => 'o', // #1ed5
    'Ỗ' => 'O', // #1ed6
    'ỗ' => 'o', // #1ed7
    'Ộ' => 'O', // #1ed8
    'ộ' => 'o', // #1ed9
    'Ớ' => 'O', // #1eda
    'ớ' => 'o', // #1edb
    'Ờ' => 'O', // #1edc
    'ờ' => 'o', // #1edd
    'Ở' => 'O', // #1ede
    'ở' => 'o', // #1edf
    'Ỡ' => 'O', // #1ee0
    'ỡ' => 'o', // #1ee1
    'Ợ' => 'O', // #1ee2
    'ợ' => 'o', // #1ee3
    'Ụ' => 'U', // #1ee4
    'ụ' => 'u', // #1ee5
    'Ủ' => 'U', // #1ee6
    'ủ' => 'u', // #1ee7
    'Ứ' => 'U', // #1ee8
    'ứ' => 'u', // #1ee9
    'Ừ' => 'U', // #1eea
    'ừ' => 'u', // #1eeb
    'Ử' => 'U', // #1eec
    'ử' => 'u', // #1eed
    'Ữ' => 'U', // #1eee
    'ữ' => 'u', // #1eef
    'Ự' => 'U', // #1ef0
    'ự' => 'u', // #1ef1
    'Ỳ' => 'Y', // #1ef2
    'ỳ' => 'y', // #1ef3
    'Ỵ' => 'Y', // #1ef4
    'ỵ' => 'y', // #1ef5
    'Ỷ' => 'Y', // #1ef6
    'ỷ' => 'y', // #1ef7
    'Ỹ' => 'Y', // #1ef8
    'ỹ' => 'y', // #1ef9
    'Ỻ' => 'LL', // #1efa
    'ỻ' => 'll', // #1efb
    'Ỽ' => 'V', // #1efc
    'ỽ' => 'v', // #1efd
    'Ỿ' => 'Y', // #1efe
    'ỿ' => 'y', // #1eff
    // буквоподобные символы
    '℀' => 'a-c', // #2100
    '℁' => 'a-s', // #2101
    'ℂ' => 'C', // #2102
    '℅' => 'c-o', // #2105
    '℆' => 'c-u', // #2106
    'ℊ' => 'g', // #210a
    'ℋ' => 'H', // #210b
    'ℌ' => 'x', // #210c
    'ℍ' => 'H', // #210d
    'ℎ' => 'h', // #210e
    'ℐ' => 'I', // #2110
    'ℑ' => 'I', // #2111
    'ℒ' => 'L', // #2112
    'ℓ' => 'l', // #2113
    'ℕ' => 'N', // #2115
    '№' => 'No', // #2116
    '℗' => '(P)', // #2117
    '℘' => 'P', // #2118
    'ℙ' => 'P', // #2119
    'ℚ' => 'Q', // #211a
    'ℛ' => 'R', // #211b
    'ℜ' => 'R', // #211c
    'ℝ' => 'R', // #211d
    '℞' => 'Rx', // #211e
    '℡' => 'TEL', // #2121
    'ℤ' => 'Z', // #2124
    'ℨ' => 'Z', // #2128
    'K' => 'K', // #212a
    'Å' => 'A', // #212b
    'ℬ' => 'B', // #212c
    'ℭ' => 'C', // #212d
    'ℯ' => 'e', // #212f
    'ℰ' => 'E', // #2130
    'ℱ' => 'F', // #2131
    'ℳ' => 'M', // #2133
    'ℴ' => 'o', // #2134
    'ℹ' => 'i', // #2139
    '℻' => 'FAX', // #213b
    'ⅅ' => 'D', // #2145
    'ⅆ' => 'd', // #2146
    'ⅇ' => 'e', // #2147
    'ⅈ' => 'i', // #2148
    'ⅉ' => 'j', // #2149
    //числовые формы
    'Ⅰ' => 'I', // #2160
    'Ⅱ' => 'II', // #2161
    'Ⅲ' => 'III', // #2162
    'Ⅳ' => 'IV', // #2163
    'Ⅴ' => 'V', // #2164
    'Ⅵ' => 'VI', // #2165
    'Ⅶ' => 'VII', // #2166
    'Ⅷ' => 'VIII', // #2167
    'Ⅸ' => 'IX', // #2168
    'Ⅹ' => 'X', // #2169
    'Ⅺ' => 'XI', // #216a
    'Ⅻ' => 'XII', // #216b
    'Ⅼ' => 'L', // #216c
    'Ⅽ' => 'C', // #216d
    'Ⅾ' => 'D', // #216e
    'Ⅿ' => 'M', // #216f
    'ⅰ' => 'i', // #2170
    'ⅱ' => 'ii', // #2171
    'ⅲ' => 'iii', // #2172
    'ⅳ' => 'iv', // #2173
    'ⅴ' => 'v', // #2174
    'ⅵ' => 'vi', // #2175
    'ⅶ' => 'vii', // #2176
    'ⅷ' => 'viii', // #2177
    'ⅸ' => 'ix', // #2178
    'ⅹ' => 'x', // #2179
    'ⅺ' => 'xi', // #217a
    'ⅻ' => 'xii', // #217b
    'ⅼ' => 'l', // #217c
    'ⅽ' => 'c', // #217d
    'ⅾ' => 'd', // #217e
    'ⅿ' => 'm', // #217f
    // обрамлённые буквы и цифры
    '⑴' => '(1)', // #2474
    '⑵' => '(2)', // #2475
    '⑶' => '(3)', // #2476
    '⑷' => '(4)', // #2477
    '⑸' => '(5)', // #2478
    '⑹' => '(6)', // #2479
    '⑺' => '(7)', // #247a
    '⑻' => '(8)', // #247b
    '⑼' => '(9)', // #247c
    '⑽' => '(10)', // #247d
    '⑾' => '(11)', // #247e
    '⑿' => '(12)', // #247f
    '⒀' => '(13)', // #2480
    '⒁' => '(14)', // #2481
    '⒂' => '(15)', // #2482
    '⒃' => '(16)', // #2483
    '⒄' => '(17)', // #2484
    '⒅' => '(18)', // #2485
    '⒆' => '(19)', // #2486
    '⒇' => '(20)', // #2487
    '⒈' => '1.', // #2488
    '⒉' => '2.', // #2489
    '⒊' => '3.', // #248a
    '⒋' => '4.', // #248b
    '⒌' => '5.', // #248c
    '⒍' => '6.', // #248d
    '⒎' => '7.', // #248e
    '⒏' => '8.', // #248f
    '⒐' => '9.', // #2490
    '⒑' => '10.', // #2491
    '⒒' => '11.', // #2492
    '⒓' => '12.', // #2493
    '⒔' => '13.', // #2494
    '⒕' => '14.', // #2495
    '⒖' => '15.', // #2496
    '⒗' => '16.', // #2497
    '⒘' => '17.', // #2498
    '⒙' => '18.', // #2499
    '⒚' => '19.', // #249a
    '⒛' => '20.', // #249b
    '⒜' => '(a)', // #249c
    '⒝' => '(b)', // #249d
    '⒞' => '(c)', // #249e
    '⒟' => '(d)', // #249f
    '⒠' => '(e)', // #24a0
    '⒡' => '(f)', // #24a1
    '⒢' => '(g)', // #24a2
    '⒣' => '(h)', // #24a3
    '⒤' => '(i)', // #24a4
    '⒥' => '(j)', // #24a5
    '⒦' => '(k)', // #24a6
    '⒧' => '(l)', // #24a7
    '⒨' => '(m)', // #24a8
    '⒩' => '(n)', // #24a9
    '⒪' => '(o)', // #24aa
    '⒫' => '(p)', // #24ab
    '⒬' => '(q)', // #24ac
    '⒭' => '(r)', // #24ad
    '⒮' => '(s)', // #24ae
    '⒯' => '(t)', // #24af
    '⒰' => '(u)', // #24b0
    '⒱' => '(v)', // #24b1
    '⒲' => '(w)', // #24b2
    '⒳' => '(x)', // #24b3
    '⒴' => '(y)', // #24b4
    '⒵' => '(z)', // #24b5
    // расширенная латиница — C
    'Ⱡ' => 'L', // #2c60
    'ⱡ' => 'l', // #2c61
    'Ɫ' => 'L', // #2c62
    'Ᵽ' => 'P', // #2c63
    'Ɽ' => 'R', // #2c64
    'ⱥ' => 'a', // #2c65
    'ⱦ' => 't', // #2c66
    'Ⱨ' => 'H', // #2c67
    'ⱨ' => 'h', // #2c68
    'Ⱪ' => 'K', // #2c69
    'ⱪ' => 'k', // #2c6a
    'Ⱬ' => 'Z', // #2c6b
    'ⱬ' => 'z', // #2c6c
    'Ɱ' => 'M', // #2c6e
    'ⱱ' => 'v', // #2c71
    'Ⱳ' => 'W', // #2c72
    'ⱳ' => 'w', // #2c73
    'ⱴ' => 'v', // #2c74
    'ⱸ' => 'e', // #2c78
    'ⱺ' => 'o', // #2c7a
    'Ȿ' => 'S', // #2c7e
    'Ɀ' => 'Z', // #2c7f
    // расширенная латиница — D
    'ꜰ' => 'F', // #a730
    'ꜱ' => 'S', // #a731
    'Ꜳ' => 'AA', // #a732
    'ꜳ' => 'aa', // #a733
    'Ꜵ' => 'AO', // #a734
    'ꜵ' => 'ao', // #a735
    'Ꜷ' => 'AU', // #a736
    'ꜷ' => 'au', // #a737
    'Ꜹ' => 'AV', // #a738
    'ꜹ' => 'av', // #a739
    'Ꜻ' => 'AV', // #a73a
    'ꜻ' => 'av', // #a73b
    'Ꜽ' => 'AY', // #a73c
    'ꜽ' => 'ay', // #a73d
    'Ꝁ' => 'K', // #a740
    'ꝁ' => 'k', // #a741
    'Ꝃ' => 'K', // #a742
    'ꝃ' => 'k', // #a743
    'Ꝅ' => 'K', // #a744
    'ꝅ' => 'k', // #a745
    'Ꝇ' => 'L', // #a746
    'ꝇ' => 'l', // #a747
    'Ꝉ' => 'L', // #a748
    'ꝉ' => 'l', // #a749
    'Ꝋ' => 'O', // #a74a
    'ꝋ' => 'o', // #a74b
    'Ꝍ' => 'O', // #a74c
    'ꝍ' => 'o', // #a74d
    'Ꝏ' => 'OO', // #a74e
    'ꝏ' => 'oo', // #a74f
    'Ꝑ' => 'P', // #a750
    'ꝑ' => 'p', // #a751
    'Ꝓ' => 'P', // #a752
    'ꝓ' => 'p', // #a753
    'Ꝕ' => 'P', // #a754
    'ꝕ' => 'p', // #a755
    'Ꝗ' => 'Q', // #a756
    'ꝗ' => 'q', // #a757
    'Ꝙ' => 'Q', // #a758
    'ꝙ' => 'q', // #a759
    'Ꝟ' => 'V', // #a75e
    'ꝟ' => 'v', // #a75f
    'Ꝡ' => 'VY', // #a760
    'ꝡ' => 'vy', // #a761
    'Ꝥ' => 'TH', // #a764
    'ꝥ' => 'th', // #a765
    'Ꝧ' => 'TH', // #a766
    'ꝧ' => 'th', // #a767
    'ꝱ' => 'd', // #a771
    'ꝲ' => 'l', // #a772
    'ꝳ' => 'm', // #a773
    'ꝴ' => 'n', // #a774
    'ꝵ' => 'r', // #a775
    'ꝶ' => 'R', // #a776
    'ꝷ' => 't', // #a777
    'Ꝺ' => 'D', // #a779
    'ꝺ' => 'd', // #a77a
    'Ꝼ' => 'F', // #a77b
    'ꝼ' => 'f', // #a77c
    'Ꞇ' => 'T', // #a786
    'ꞇ' => 't', // #a787
    'Ꞑ' => 'N', // #a790
    'ꞑ' => 'n', // #a791
    'Ꞓ' => 'C', // #a792
    'ꞓ' => 'c', // #a793
    'Ꞡ' => 'G', // #a7a0
    'ꞡ' => 'g', // #a7a1
    'Ꞣ' => 'K', // #a7a2
    'ꞣ' => 'k', // #a7a3
    'Ꞥ' => 'N', // #a7a4
    'ꞥ' => 'n', // #a7a5
    'Ꞧ' => 'R', // #a7a6
    'ꞧ' => 'r', // #a7a7
    'Ꞩ' => 'S', // #a7a8
    'ꞩ' => 's', // #a7a9
    'Ɦ' => 'H', // #a7aa
    // алфавитные формы представления
    'ﬀ' => 'ff', // #fb00
    'ﬁ' => 'fi', // #fb01
    'ﬂ' => 'fl', // #fb02
    'ﬃ' => 'ffi', // #fb03
    'ﬄ' => 'ffl', // #fb04
    'ﬅ' => 'st', // #fb05
    'ﬆ' => 'st', // #fb06
    // полуширинные и полноширинные формы
    '０' => '0', // #ff10
    '１' => '1', // #ff11
    '２' => '2', // #ff12
    '３' => '3', // #ff13
    '４' => '4', // #ff14
    '５' => '5', // #ff15
    '６' => '6', // #ff16
    '７' => '7', // #ff17
    '８' => '8', // #ff18
    '９' => '9', // #ff19
    'Ａ' => 'A', // #ff21
    'Ｂ' => 'B', // #ff22
    'Ｃ' => 'C', // #ff23
    'Ｄ' => 'D', // #ff24
    'Ｅ' => 'E', // #ff25
    'Ｆ' => 'F', // #ff26
    'Ｇ' => 'G', // #ff27
    'Ｈ' => 'H', // #ff28
    'Ｉ' => 'I', // #ff29
    'Ｊ' => 'J', // #ff2a
    'Ｋ' => 'K', // #ff2b
    'Ｌ' => 'L', // #ff2c
    'Ｍ' => 'M', // #ff2d
    'Ｎ' => 'N', // #ff2e
    'Ｏ' => 'O', // #ff2f
    'Ｐ' => 'P', // #ff30
    'Ｑ' => 'Q', // #ff31
    'Ｒ' => 'R', // #ff32
    'Ｓ' => 'S', // #ff33
    'Ｔ' => 'T', // #ff34
    'Ｕ' => 'U', // #ff35
    'Ｖ' => 'V', // #ff36
    'Ｗ' => 'W', // #ff37
    'Ｘ' => 'X', // #ff38
    'Ｙ' => 'Y', // #ff39
    'Ｚ' => 'Z', // #ff3a
    'ａ' => 'a', // #ff41
    'ｂ' => 'b', // #ff42
    'ｃ' => 'c', // #ff43
    'ｄ' => 'd', // #ff44
    'ｅ' => 'e', // #ff45
    'ｆ' => 'f', // #ff46
    'ｇ' => 'g', // #ff47
    'ｈ' => 'h', // #ff48
    'ｉ' => 'i', // #ff49
    'ｊ' => 'j', // #ff4a
    'ｋ' => 'k', // #ff4b
    'ｌ' => 'l', // #ff4c
    'ｍ' => 'm', // #ff4d
    'ｎ' => 'n', // #ff4e
    'ｏ' => 'o', // #ff4f
    'ｐ' => 'p', // #ff50
    'ｑ' => 'q', // #ff51
    'ｒ' => 'r', // #ff52
    'ｓ' => 's', // #ff53
    'ｔ' => 't', // #ff54
    'ｕ' => 'u', // #ff55
    'ｖ' => 'v', // #ff56
    'ｗ' => 'w', // #ff57
    'ｘ' => 'x', // #ff58
    'ｙ' => 'y', // #ff59
    'ｚ' => 'z', // #ff5a

    // русский
    'Ё' => 'Yo', // #401
    'А' => 'A', // #410
    'Б' => 'B', // #411
    'В' => 'V', // #412
    'Г' => 'G', // #413
    'Д' => 'D', // #414
    'Е' => 'E', // #415
    'Ж' => 'Zh', // #416
    'З' => 'Z', // #417
    'И' => 'I', // #418
    'Й' => 'J', // #419
    'К' => 'K', // #41a
    'Л' => 'L', // #41b
    'М' => 'M', // #41c
    'Н' => 'N', // #41d
    'О' => 'O', // #41e
    'П' => 'P', // #41f
    'Р' => 'R', // #420
    'С' => 'S', // #421
    'Т' => 'T', // #422
    'У' => 'U', // #423
    'Ф' => 'F', // #424
    'Х' => 'Kh', // #425
    'Ц' => 'Ts', // #426
    'Ч' => 'Ch', // #427
    'Ш' => 'Sh', // #428
    'Щ' => 'Shch', // #429
    'Ъ' => '', // #42a
    'Ы' => 'Y', // #42b
    'Ь' => '', // #42c
    'Э' => 'E', // #42d
    'Ю' => 'Yu', // #42e
    'Я' => 'Ya', // #42f
    'а' => 'a', // #430
    'б' => 'b', // #431
    'в' => 'v', // #432
    'г' => 'g', // #433
    'д' => 'd', // #434
    'е' => 'e', // #435
    'ж' => 'zh', // #436
    'з' => 'z', // #437
    'и' => 'i', // #438
    'й' => 'j', // #439
    'к' => 'k', // #43a
    'л' => 'l', // #43b
    'м' => 'm', // #43c
    'н' => 'n', // #43d
    'о' => 'o', // #43e
    'п' => 'p', // #43f
    'р' => 'r', // #440
    'с' => 's', // #441
    'т' => 't', // #442
    'у' => 'u', // #443
    'ф' => 'f', // #444
    'х' => 'kh', // #445
    'ц' => 'ts', // #446
    'ч' => 'ch', // #447
    'ш' => 'sh', // #448
    'щ' => 'shch', // #449
    'ъ' => '', // #44a
    'ы' => 'y', // #44b
    'ь' => '', // #44c
    'э' => 'e', // #44d
    'ю' => 'yu', // #44e
    'я' => 'ya', // #44f
    'ё' => 'yo', // #451
];
