<?php
/**
 * This file is part of the ForkBB <https://github.com/forkbb>.
 *
 * @copyright (c) Visman <mio.visman@yandex.ru, https://github.com/MioVisman>
 * @license   The MIT License (MIT)
 */

declare(strict_types=1);

namespace ForkBB\Models\User;

use ForkBB\Models\Action;

class NormUsername extends Action
{
    /**
     * https://www.unicode.org/Public/security/latest/confusables.txt
     */
    protected array $confusables = [
        "\u{05AD}" => "\u{0596}",
        "\u{05AE}" => "\u{0598}",
        "\u{05A8}" => "\u{0599}",
        "\u{05A4}" => "\u{059A}",
        "\u{1AB4}" => "\u{06DB}",
        "\u{20DB}" => "\u{06DB}",
        "\u{0619}" => "\u{0313}",
        "\u{08F3}" => "\u{0313}",
        "\u{0343}" => "\u{0313}",
        "\u{0315}" => "\u{0313}",
        "\u{064F}" => "\u{0313}",
        "\u{065D}" => "\u{0314}",
        "\u{059C}" => "\u{0301}",
        "\u{059D}" => "\u{0301}",
        "\u{0618}" => "\u{0301}",
        "\u{0747}" => "\u{0301}",
        "\u{0341}" => "\u{0301}",
        "\u{0954}" => "\u{0301}",
        "\u{064E}" => "\u{0301}",
        "\u{0340}" => "\u{0300}",
        "\u{0953}" => "\u{0300}",
        "\u{030C}" => "\u{0306}",
        "\u{A67C}" => "\u{0306}",
        "\u{0658}" => "\u{0306}",
        "\u{065A}" => "\u{0306}",
        "\u{036E}" => "\u{0306}",
        "\u{06E8}" => "\u{0306}\u{0307}",
        "\u{0310}" => "\u{0306}\u{0307}",
        "\u{0901}" => "\u{0306}\u{0307}",
        "\u{0981}" => "\u{0306}\u{0307}",
        "\u{0A81}" => "\u{0306}\u{0307}",
        "\u{0B01}" => "\u{0306}\u{0307}",
        "\u{0C00}" => "\u{0306}\u{0307}",
        "\u{0C81}" => "\u{0306}\u{0307}",
        "\u{0D01}" => "\u{0306}\u{0307}",
        "\u{114BF}" => "\u{0306}\u{0307}",
        "\u{1CD0}" => "\u{0302}",
        "\u{0311}" => "\u{0302}",
        "\u{065B}" => "\u{0302}",
        "\u{07EE}" => "\u{0302}",
        "\u{A6F0}" => "\u{0302}",
        "\u{05AF}" => "\u{030A}",
        "\u{06DF}" => "\u{030A}",
        "\u{17D3}" => "\u{030A}",
        "\u{309A}" => "\u{030A}",
        "\u{0652}" => "\u{030A}",
        "\u{0B82}" => "\u{030A}",
        "\u{1036}" => "\u{030A}",
        "\u{17C6}" => "\u{030A}",
        "\u{11300}" => "\u{030A}",
        "\u{0E4D}" => "\u{030A}",
        "\u{0ECD}" => "\u{030A}",
        "\u{0366}" => "\u{030A}",
        "\u{2DEA}" => "\u{030A}",
        "\u{08EB}" => "\u{0308}",
        "\u{07F3}" => "\u{0308}",
        "\u{064B}" => "\u{030B}",
        "\u{08F0}" => "\u{030B}",
        "\u{0342}" => "\u{0303}",
        "\u{0653}" => "\u{0303}",
        "\u{05C4}" => "\u{0307}",
        "\u{06EC}" => "\u{0307}",
        "\u{0740}" => "\u{0307}",
        "\u{08EA}" => "\u{0307}",
        "\u{0741}" => "\u{0307}",
        "\u{0358}" => "\u{0307}",
        "\u{05B9}" => "\u{0307}",
        "\u{05BA}" => "\u{0307}",
        "\u{05C2}" => "\u{0307}",
        "\u{05C1}" => "\u{0307}",
        "\u{07ED}" => "\u{0307}",
        "\u{0902}" => "\u{0307}",
        "\u{0A02}" => "\u{0307}",
        "\u{0A82}" => "\u{0307}",
        "\u{0BCD}" => "\u{0307}",
        "\u{0337}" => "\u{0338}",
        "\u{1AB7}" => "\u{0328}",
        "\u{0322}" => "\u{0328}",
        "\u{0345}" => "\u{0328}",
        "\u{1CD2}" => "\u{0304}",
        "\u{0305}" => "\u{0304}",
        "\u{0659}" => "\u{0304}",
        "\u{07EB}" => "\u{0304}",
        "\u{A6F1}" => "\u{0304}",
        "\u{1CDA}" => "\u{030E}",
        "\u{0657}" => "\u{0312}",
        "\u{0357}" => "\u{0350}",
        "\u{08FF}" => "\u{0350}",
        "\u{08F8}" => "\u{0350}",
        "\u{0900}" => "\u{0352}",
        "\u{1CED}" => "\u{0316}",
        "\u{1CDC}" => "\u{0329}",
        "\u{0656}" => "\u{0329}",
        "\u{1CD5}" => "\u{032B}",
        "\u{0347}" => "\u{0333}",
        "\u{08F9}" => "\u{0354}",
        "\u{08FA}" => "\u{0355}",
        "\u{309B}" => "\u{FF9E}",
        "\u{309C}" => "\u{FF9F}",
        "\u{0336}" => "\u{0335}",
        "\u{302C}" => "\u{0309}",
        "\u{05C5}" => "\u{0323}",
        "\u{08ED}" => "\u{0323}",
        "\u{1CDD}" => "\u{0323}",
        "\u{05B4}" => "\u{0323}",
        "\u{065C}" => "\u{0323}",
        "\u{093C}" => "\u{0323}",
        "\u{09BC}" => "\u{0323}",
        "\u{0A3C}" => "\u{0323}",
        "\u{0ABC}" => "\u{0323}",
        "\u{0B3C}" => "\u{0323}",
        "\u{111CA}" => "\u{0323}",
        "\u{114C3}" => "\u{0323}",
        "\u{10A3A}" => "\u{0323}",
        "\u{08EE}" => "\u{0324}",
        "\u{1CDE}" => "\u{0324}",
        "\u{0F37}" => "\u{0325}",
        "\u{302D}" => "\u{0325}",
        "\u{0327}" => "\u{0326}",
        "\u{0321}" => "\u{0326}",
        "\u{0339}" => "\u{0326}",
        "\u{1CD9}" => "\u{032D}",
        "\u{1CD8}" => "\u{032E}",
        "\u{0952}" => "\u{0331}",
        "\u{0320}" => "\u{0331}",
        "\u{08F1}" => "\u{064C}",
        "\u{08E8}" => "\u{064C}",
        "\u{08E5}" => "\u{064C}",
        "\u{FC5E}" => "\u{FE72}\u{0651}",
        "\u{08F2}" => "\u{064D}",
        "\u{FC5F}" => "\u{FE74}\u{0651}",
        "\u{FCF2}" => "\u{FE77}\u{0651}",
        "\u{FC60}" => "\u{FE76}\u{0651}",
        "\u{FCF3}" => "\u{FE79}\u{0651}",
        "\u{FC61}" => "\u{FE78}\u{0651}",
        "\u{061A}" => "\u{0650}",
        "\u{0317}" => "\u{0650}",
        "\u{FCF4}" => "\u{FE7B}\u{0651}",
        "\u{FC62}" => "\u{FE7A}\u{0651}",
        "\u{FC63}" => "\u{FE7C}\u{0670}",
        "\u{065F}" => "\u{0655}",
        "\u{030D}" => "\u{0670}",
        "\u{0742}" => "\u{073C}",
        "\u{0A03}" => "\u{0983}",
        "\u{0C03}" => "\u{0983}",
        "\u{0C83}" => "\u{0983}",
        "\u{0D03}" => "\u{0983}",
        "\u{0D83}" => "\u{0983}",
        "\u{1038}" => "\u{0983}",
        "\u{114C1}" => "\u{0983}",
        "\u{17CB}" => "\u{0E48}",
        "\u{0EC8}" => "\u{0E48}",
        "\u{0EC9}" => "\u{0E49}",
        "\u{0ECA}" => "\u{0E4A}",
        "\u{0ECB}" => "\u{0E4B}",
        "\u{A66F}" => "\u{20E9}",
        "\u{2028}" => "\u{0020}",
        "\u{2029}" => "\u{0020}",
        "\u{1680}" => "\u{0020}",
        "\u{2000}" => "\u{0020}",
        "\u{2001}" => "\u{0020}",
        "\u{2002}" => "\u{0020}",
        "\u{2003}" => "\u{0020}",
        "\u{2004}" => "\u{0020}",
        "\u{2005}" => "\u{0020}",
        "\u{2006}" => "\u{0020}",
        "\u{2008}" => "\u{0020}",
        "\u{2009}" => "\u{0020}",
        "\u{200A}" => "\u{0020}",
        "\u{205F}" => "\u{0020}",
        "\u{00A0}" => "\u{0020}",
        "\u{2007}" => "\u{0020}",
        "\u{202F}" => "\u{0020}",
        "\u{07FA}" => "\u{005F}",
        "\u{FE4D}" => "\u{005F}",
        "\u{FE4E}" => "\u{005F}",
        "\u{FE4F}" => "\u{005F}",
        "\u{2010}" => "\u{002D}",
        "\u{2011}" => "\u{002D}",
        "\u{2012}" => "\u{002D}",
        "\u{2013}" => "\u{002D}",
        "\u{FE58}" => "\u{002D}",
        "\u{06D4}" => "\u{002D}",
        "\u{2043}" => "\u{002D}",
        "\u{02D7}" => "\u{002D}",
        "\u{2212}" => "\u{002D}",
        "\u{2796}" => "\u{002D}",
        "\u{2CBA}" => "\u{002D}",
        "\u{2A29}" => "\u{002D}\u{0313}",
        "\u{2E1A}" => "\u{002D}\u{0308}",
        "\u{FB29}" => "\u{002D}\u{0307}",
        "\u{2238}" => "\u{002D}\u{0307}",
        "\u{2A2A}" => "\u{002D}\u{0323}",
        "\u{A4FE}" => "\u{002D}\u{002E}",
        "\u{FF5E}" => "\u{301C}",
        "\u{060D}" => "\u{002C}",
        "\u{066B}" => "\u{002C}",
        "\u{201A}" => "\u{002C}",
        "\u{00B8}" => "\u{002C}",
        "\u{A4F9}" => "\u{002C}",
        "\u{2E32}" => "\u{060C}",
        "\u{066C}" => "\u{060C}",
        "\u{037E}" => "\u{003B}",
        "\u{2E35}" => "\u{061B}",
        "\u{0903}" => "\u{003A}",
        "\u{0A83}" => "\u{003A}",
        "\u{FF1A}" => "\u{003A}",
        "\u{0589}" => "\u{003A}",
        "\u{0703}" => "\u{003A}",
        "\u{0704}" => "\u{003A}",
        "\u{16EC}" => "\u{003A}",
        "\u{FE30}" => "\u{003A}",
        "\u{1803}" => "\u{003A}",
        "\u{1809}" => "\u{003A}",
        "\u{205A}" => "\u{003A}",
        "\u{05C3}" => "\u{003A}",
        "\u{02F8}" => "\u{003A}",
        "\u{A789}" => "\u{003A}",
        "\u{2236}" => "\u{003A}",
        "\u{02D0}" => "\u{003A}",
        "\u{A4FD}" => "\u{003A}",
        "\u{2A74}" => "\u{003A}\u{003A}\u{003D}",
        "\u{29F4}" => "\u{003A}\u{2192}",
        "\u{FF01}" => "\u{0021}",
        "\u{01C3}" => "\u{0021}",
        "\u{2D51}" => "\u{0021}",
        "\u{203C}" => "\u{0021}\u{0021}",
        "\u{2049}" => "\u{0021}\u{003F}",
        "\u{0294}" => "\u{003F}",
        "\u{0241}" => "\u{003F}",
        "\u{097D}" => "\u{003F}",
        "\u{13AE}" => "\u{003F}",
        "\u{A6EB}" => "\u{003F}",
        "\u{2048}" => "\u{003F}\u{0021}",
        "\u{2047}" => "\u{003F}\u{003F}",
        "\u{2E2E}" => "\u{061F}",
        "\u{1D16D}" => "\u{002E}",
        "\u{2024}" => "\u{002E}",
        "\u{0701}" => "\u{002E}",
        "\u{0702}" => "\u{002E}",
        "\u{A60E}" => "\u{002E}",
        "\u{10A50}" => "\u{002E}",
        "\u{0660}" => "\u{002E}",
        "\u{06F0}" => "\u{002E}",
        "\u{A4F8}" => "\u{002E}",
        "\u{A4FB}" => "\u{002E}\u{002C}",
        "\u{2025}" => "\u{002E}\u{002E}",
        "\u{A4FA}" => "\u{002E}\u{002E}",
        "\u{2026}" => "\u{002E}\u{002E}\u{002E}",
        "\u{A6F4}" => "\u{A6F3}\u{A6F3}",
        "\u{30FB}" => "\u{00B7}",
        "\u{FF65}" => "\u{00B7}",
        "\u{16EB}" => "\u{00B7}",
        "\u{0387}" => "\u{00B7}",
        "\u{2E31}" => "\u{00B7}",
        "\u{10101}" => "\u{00B7}",
        "\u{2022}" => "\u{00B7}",
        "\u{2027}" => "\u{00B7}",
        "\u{2219}" => "\u{00B7}",
        "\u{22C5}" => "\u{00B7}",
        "\u{A78F}" => "\u{00B7}",
        "\u{1427}" => "\u{00B7}",
        "\u{22EF}" => "\u{00B7}\u{00B7}\u{00B7}",
        "\u{2D48}" => "\u{00B7}\u{00B7}\u{00B7}",
        "\u{1444}" => "\u{00B7}\u{003C}",
        "\u{22D7}" => "\u{00B7}\u{003E}",
        "\u{1437}" => "\u{00B7}\u{003E}",
        "\u{1440}" => "\u{00B7}\u{003E}",
        "\u{152F}" => "\u{00B7}\u{0034}",
        "\u{147E}" => "\u{00B7}\u{0062}",
        "\u{1480}" => "\u{00B7}\u{0062}\u{0307}",
        "\u{147A}" => "\u{00B7}\u{0064}",
        "\u{1498}" => "\u{00B7}\u{004A}",
        "\u{14B6}" => "\u{00B7}\u{004C}",
        "\u{1476}" => "\u{00B7}\u{0050}",
        "\u{1457}" => "\u{00B7}\u{0055}",
        "\u{143A}" => "\u{00B7}\u{0056}",
        "\u{143C}" => "\u{00B7}\u{0245}",
        "\u{14AE}" => "\u{00B7}\u{0393}",
        "\u{140E}" => "\u{00B7}\u{0394}",
        "\u{1459}" => "\u{00B7}\u{0548}",
        "\u{140C}" => "\u{00B7}\u{1401}",
        "\u{1410}" => "\u{00B7}\u{1404}",
        "\u{1412}" => "\u{00B7}\u{1405}",
        "\u{1414}" => "\u{00B7}\u{1406}",
        "\u{1417}" => "\u{00B7}\u{140A}",
        "\u{1419}" => "\u{00B7}\u{140B}",
        "\u{143E}" => "\u{00B7}\u{1432}",
        "\u{1442}" => "\u{00B7}\u{1434}",
        "\u{1446}" => "\u{00B7}\u{1439}",
        "\u{145B}" => "\u{00B7}\u{144F}",
        "\u{1454}" => "\u{00B7}\u{1450}",
        "\u{145D}" => "\u{00B7}\u{1450}",
        "\u{145F}" => "\u{00B7}\u{1451}",
        "\u{1461}" => "\u{00B7}\u{1455}",
        "\u{1463}" => "\u{00B7}\u{1456}",
        "\u{1474}" => "\u{00B7}\u{146B}",
        "\u{1478}" => "\u{00B7}\u{146E}",
        "\u{147C}" => "\u{00B7}\u{1470}",
        "\u{1492}" => "\u{00B7}\u{1489}",
        "\u{1494}" => "\u{00B7}\u{148B}",
        "\u{1496}" => "\u{00B7}\u{148C}",
        "\u{149A}" => "\u{00B7}\u{148E}",
        "\u{149C}" => "\u{00B7}\u{1490}",
        "\u{149E}" => "\u{00B7}\u{1491}",
        "\u{14AC}" => "\u{00B7}\u{14A3}",
        "\u{14B0}" => "\u{00B7}\u{14A6}",
        "\u{14B2}" => "\u{00B7}\u{14A7}",
        "\u{14B4}" => "\u{00B7}\u{14A8}",
        "\u{14B8}" => "\u{00B7}\u{14AB}",
        "\u{14C9}" => "\u{00B7}\u{14C0}",
        "\u{18C6}" => "\u{00B7}\u{14C2}",
        "\u{18C8}" => "\u{00B7}\u{14C3}",
        "\u{18CA}" => "\u{00B7}\u{14C4}",
        "\u{18CC}" => "\u{00B7}\u{14C5}",
        "\u{14CB}" => "\u{00B7}\u{14C7}",
        "\u{14CD}" => "\u{00B7}\u{14C8}",
        "\u{14DC}" => "\u{00B7}\u{14D3}",
        "\u{14DE}" => "\u{00B7}\u{14D5}",
        "\u{14E0}" => "\u{00B7}\u{14D6}",
        "\u{14E2}" => "\u{00B7}\u{14D7}",
        "\u{14E4}" => "\u{00B7}\u{14D8}",
        "\u{14E6}" => "\u{00B7}\u{14DA}",
        "\u{14E8}" => "\u{00B7}\u{14DB}",
        "\u{14F6}" => "\u{00B7}\u{14ED}",
        "\u{14F8}" => "\u{00B7}\u{14EF}",
        "\u{14FA}" => "\u{00B7}\u{14F0}",
        "\u{14FC}" => "\u{00B7}\u{14F1}",
        "\u{14FE}" => "\u{00B7}\u{14F2}",
        "\u{1500}" => "\u{00B7}\u{14F4}",
        "\u{1502}" => "\u{00B7}\u{14F5}",
        "\u{1517}" => "\u{00B7}\u{1510}",
        "\u{1519}" => "\u{00B7}\u{1511}",
        "\u{151B}" => "\u{00B7}\u{1512}",
        "\u{151D}" => "\u{00B7}\u{1513}",
        "\u{151F}" => "\u{00B7}\u{1514}",
        "\u{1521}" => "\u{00B7}\u{1515}",
        "\u{1523}" => "\u{00B7}\u{1516}",
        "\u{1531}" => "\u{00B7}\u{1528}",
        "\u{1533}" => "\u{00B7}\u{1529}",
        "\u{1535}" => "\u{00B7}\u{152A}",
        "\u{1537}" => "\u{00B7}\u{152B}",
        "\u{1539}" => "\u{00B7}\u{152D}",
        "\u{153B}" => "\u{00B7}\u{152E}",
        "\u{18CE}" => "\u{00B7}\u{1543}",
        "\u{18CF}" => "\u{00B7}\u{1546}",
        "\u{18D0}" => "\u{00B7}\u{1547}",
        "\u{18D1}" => "\u{00B7}\u{1548}",
        "\u{18D2}" => "\u{00B7}\u{1549}",
        "\u{18D3}" => "\u{00B7}\u{154B}",
        "\u{154E}" => "\u{00B7}\u{154C}",
        "\u{155B}" => "\u{00B7}\u{155A}",
        "\u{1568}" => "\u{00B7}\u{1567}",
        "\u{18B3}" => "\u{00B7}\u{18B1}",
        "\u{18B6}" => "\u{00B7}\u{18B4}",
        "\u{18B9}" => "\u{00B7}\u{18B8}",
        "\u{18C2}" => "\u{00B7}\u{18C0}",
        "\u{A830}" => "\u{0964}",
        "\u{0965}" => "\u{0964}\u{0964}",
        "\u{1C3C}" => "\u{1C3B}\u{1C3B}",
        "\u{104B}" => "\u{104A}\u{104A}",
        "\u{1AA9}" => "\u{1AA8}\u{1AA8}",
        "\u{1AAB}" => "\u{1AAA}\u{1AA8}",
        "\u{1B5F}" => "\u{1B5E}\u{1B5E}",
        "\u{10A57}" => "\u{10A56}\u{10A56}",
        "\u{1144C}" => "\u{1144B}\u{1144B}",
        "\u{11642}" => "\u{11641}\u{11641}",
        "\u{11C42}" => "\u{11C41}\u{11C41}",
        "\u{1C7F}" => "\u{1C7E}\u{1C7E}",
        "\u{055D}" => "\u{0027}",
        "\u{FF07}" => "\u{0027}",
        "\u{2018}" => "\u{0027}",
        "\u{2019}" => "\u{0027}",
        "\u{201B}" => "\u{0027}",
        "\u{2032}" => "\u{0027}",
        "\u{2035}" => "\u{0027}",
        "\u{055A}" => "\u{0027}",
        "\u{05F3}" => "\u{0027}",
        "\u{0060}" => "\u{0027}",
        "\u{1FEF}" => "\u{0027}",
        "\u{FF40}" => "\u{0027}",
        "\u{00B4}" => "\u{0027}",
        "\u{0384}" => "\u{0027}",
        "\u{1FFD}" => "\u{0027}",
        "\u{1FBD}" => "\u{0027}",
        "\u{1FBF}" => "\u{0027}",
        "\u{1FFE}" => "\u{0027}",
        "\u{02B9}" => "\u{0027}",
        "\u{0374}" => "\u{0027}",
        "\u{02C8}" => "\u{0027}",
        "\u{02CA}" => "\u{0027}",
        "\u{02CB}" => "\u{0027}",
        "\u{02F4}" => "\u{0027}",
        "\u{02BB}" => "\u{0027}",
        "\u{02BD}" => "\u{0027}",
        "\u{02BC}" => "\u{0027}",
        "\u{02BE}" => "\u{0027}",
        "\u{A78C}" => "\u{0027}",
        "\u{05D9}" => "\u{0027}",
        "\u{07F4}" => "\u{0027}",
        "\u{07F5}" => "\u{0027}",
        "\u{144A}" => "\u{0027}",
        "\u{16CC}" => "\u{0027}",
        "\u{16F51}" => "\u{0027}",
        "\u{16F52}" => "\u{0027}",
        "\u{1CD3}" => "\u{0027}\u{0027}",
        "\u{0022}" => "\u{0027}\u{0027}",
        "\u{FF02}" => "\u{0027}\u{0027}",
        "\u{201C}" => "\u{0027}\u{0027}",
        "\u{201D}" => "\u{0027}\u{0027}",
        "\u{201F}" => "\u{0027}\u{0027}",
        "\u{2033}" => "\u{0027}\u{0027}",
        "\u{2036}" => "\u{0027}\u{0027}",
        "\u{3003}" => "\u{0027}\u{0027}",
        "\u{05F4}" => "\u{0027}\u{0027}",
        "\u{02DD}" => "\u{0027}\u{0027}",
        "\u{02BA}" => "\u{0027}\u{0027}",
        "\u{02F6}" => "\u{0027}\u{0027}",
        "\u{02EE}" => "\u{0027}\u{0027}",
        "\u{05F2}" => "\u{0027}\u{0027}",
        "\u{2034}" => "\u{0027}\u{0027}\u{0027}",
        "\u{2037}" => "\u{0027}\u{0027}\u{0027}",
        "\u{2057}" => "\u{0027}\u{0027}\u{0027}\u{0027}",
        "\u{0181}" => "\u{0027}\u{0042}",
        "\u{018A}" => "\u{0027}\u{0044}",
        "\u{0149}" => "\u{0027}\u{006E}",
        "\u{01A4}" => "\u{0027}\u{0050}",
        "\u{01AC}" => "\u{0027}\u{0054}",
        "\u{01B3}" => "\u{0027}\u{0059}",
        "\u{FF3B}" => "\u{0028}",
        "\u{2768}" => "\u{0028}",
        "\u{2772}" => "\u{0028}",
        "\u{3014}" => "\u{0028}",
        "\u{FD3E}" => "\u{0028}",
        "\u{2E28}" => "\u{0028}\u{0028}",
        "\u{3220}" => "\u{0028}\u{30FC}\u{0029}",
        "\u{2475}" => "\u{0028}\u{0032}\u{0029}",
        "\u{2487}" => "\u{0028}\u{0032}\u{004F}\u{0029}",
        "\u{2476}" => "\u{0028}\u{0033}\u{0029}",
        "\u{2477}" => "\u{0028}\u{0034}\u{0029}",
        "\u{2478}" => "\u{0028}\u{0035}\u{0029}",
        "\u{2479}" => "\u{0028}\u{0036}\u{0029}",
        "\u{247A}" => "\u{0028}\u{0037}\u{0029}",
        "\u{247B}" => "\u{0028}\u{0038}\u{0029}",
        "\u{247C}" => "\u{0028}\u{0039}\u{0029}",
        "\u{249C}" => "\u{0028}\u{0061}\u{0029}",
        "\u{1F110}" => "\u{0028}\u{0041}\u{0029}",
        "\u{249D}" => "\u{0028}\u{0062}\u{0029}",
        "\u{1F111}" => "\u{0028}\u{0042}\u{0029}",
        "\u{249E}" => "\u{0028}\u{0063}\u{0029}",
        "\u{1F112}" => "\u{0028}\u{0043}\u{0029}",
        "\u{249F}" => "\u{0028}\u{0064}\u{0029}",
        "\u{1F113}" => "\u{0028}\u{0044}\u{0029}",
        "\u{24A0}" => "\u{0028}\u{0065}\u{0029}",
        "\u{1F114}" => "\u{0028}\u{0045}\u{0029}",
        "\u{24A1}" => "\u{0028}\u{0066}\u{0029}",
        "\u{1F115}" => "\u{0028}\u{0046}\u{0029}",
        "\u{24A2}" => "\u{0028}\u{0067}\u{0029}",
        "\u{1F116}" => "\u{0028}\u{0047}\u{0029}",
        "\u{24A3}" => "\u{0028}\u{0068}\u{0029}",
        "\u{1F117}" => "\u{0028}\u{0048}\u{0029}",
        "\u{24A4}" => "\u{0028}\u{0069}\u{0029}",
        "\u{24A5}" => "\u{0028}\u{006A}\u{0029}",
        "\u{1F119}" => "\u{0028}\u{004A}\u{0029}",
        "\u{24A6}" => "\u{0028}\u{006B}\u{0029}",
        "\u{1F11A}" => "\u{0028}\u{004B}\u{0029}",
        "\u{2474}" => "\u{0028}\u{006C}\u{0029}",
        "\u{1F118}" => "\u{0028}\u{006C}\u{0029}",
        "\u{24A7}" => "\u{0028}\u{006C}\u{0029}",
        "\u{1F11B}" => "\u{0028}\u{004C}\u{0029}",
        "\u{247F}" => "\u{0028}\u{006C}\u{0032}\u{0029}",
        "\u{2480}" => "\u{0028}\u{006C}\u{0033}\u{0029}",
        "\u{2481}" => "\u{0028}\u{006C}\u{0034}\u{0029}",
        "\u{2482}" => "\u{0028}\u{006C}\u{0035}\u{0029}",
        "\u{2483}" => "\u{0028}\u{006C}\u{0036}\u{0029}",
        "\u{2484}" => "\u{0028}\u{006C}\u{0037}\u{0029}",
        "\u{2485}" => "\u{0028}\u{006C}\u{0038}\u{0029}",
        "\u{2486}" => "\u{0028}\u{006C}\u{0039}\u{0029}",
        "\u{247E}" => "\u{0028}\u{006C}\u{006C}\u{0029}",
        "\u{247D}" => "\u{0028}\u{006C}\u{004F}\u{0029}",
        "\u{1F11C}" => "\u{0028}\u{004D}\u{0029}",
        "\u{24A9}" => "\u{0028}\u{006E}\u{0029}",
        "\u{1F11D}" => "\u{0028}\u{004E}\u{0029}",
        "\u{24AA}" => "\u{0028}\u{006F}\u{0029}",
        "\u{1F11E}" => "\u{0028}\u{004F}\u{0029}",
        "\u{24AB}" => "\u{0028}\u{0070}\u{0029}",
        "\u{1F11F}" => "\u{0028}\u{0050}\u{0029}",
        "\u{24AC}" => "\u{0028}\u{0071}\u{0029}",
        "\u{1F120}" => "\u{0028}\u{0051}\u{0029}",
        "\u{24AD}" => "\u{0028}\u{0072}\u{0029}",
        "\u{1F121}" => "\u{0028}\u{0052}\u{0029}",
        "\u{24A8}" => "\u{0028}\u{0072}\u{006E}\u{0029}",
        "\u{24AE}" => "\u{0028}\u{0073}\u{0029}",
        "\u{1F122}" => "\u{0028}\u{0053}\u{0029}",
        "\u{1F12A}" => "\u{0028}\u{0053}\u{0029}",
        "\u{24AF}" => "\u{0028}\u{0074}\u{0029}",
        "\u{1F123}" => "\u{0028}\u{0054}\u{0029}",
        "\u{24B0}" => "\u{0028}\u{0075}\u{0029}",
        "\u{1F124}" => "\u{0028}\u{0055}\u{0029}",
        "\u{24B1}" => "\u{0028}\u{0076}\u{0029}",
        "\u{1F125}" => "\u{0028}\u{0056}\u{0029}",
        "\u{24B2}" => "\u{0028}\u{0077}\u{0029}",
        "\u{1F126}" => "\u{0028}\u{0057}\u{0029}",
        "\u{24B3}" => "\u{0028}\u{0078}\u{0029}",
        "\u{1F127}" => "\u{0028}\u{0058}\u{0029}",
        "\u{24B4}" => "\u{0028}\u{0079}\u{0029}",
        "\u{1F128}" => "\u{0028}\u{0059}\u{0029}",
        "\u{24B5}" => "\u{0028}\u{007A}\u{0029}",
        "\u{1F129}" => "\u{0028}\u{005A}\u{0029}",
        "\u{3200}" => "\u{0028}\u{1100}\u{0029}",
        "\u{320E}" => "\u{0028}\u{AC00}\u{0029}",
        "\u{3201}" => "\u{0028}\u{1102}\u{0029}",
        "\u{320F}" => "\u{0028}\u{B098}\u{0029}",
        "\u{3202}" => "\u{0028}\u{1103}\u{0029}",
        "\u{3210}" => "\u{0028}\u{B2E4}\u{0029}",
        "\u{3203}" => "\u{0028}\u{1105}\u{0029}",
        "\u{3211}" => "\u{0028}\u{B77C}\u{0029}",
        "\u{3204}" => "\u{0028}\u{1106}\u{0029}",
        "\u{3212}" => "\u{0028}\u{B9C8}\u{0029}",
        "\u{3205}" => "\u{0028}\u{1107}\u{0029}",
        "\u{3213}" => "\u{0028}\u{BC14}\u{0029}",
        "\u{3206}" => "\u{0028}\u{1109}\u{0029}",
        "\u{3214}" => "\u{0028}\u{C0AC}\u{0029}",
        "\u{3207}" => "\u{0028}\u{110B}\u{0029}",
        "\u{3215}" => "\u{0028}\u{C544}\u{0029}",
        "\u{321D}" => "\u{0028}\u{C624}\u{C804}\u{0029}",
        "\u{321E}" => "\u{0028}\u{C624}\u{D6C4}\u{0029}",
        "\u{3208}" => "\u{0028}\u{110C}\u{0029}",
        "\u{3216}" => "\u{0028}\u{C790}\u{0029}",
        "\u{321C}" => "\u{0028}\u{C8FC}\u{0029}",
        "\u{3209}" => "\u{0028}\u{110E}\u{0029}",
        "\u{3217}" => "\u{0028}\u{CC28}\u{0029}",
        "\u{320A}" => "\u{0028}\u{110F}\u{0029}",
        "\u{3218}" => "\u{0028}\u{CE74}\u{0029}",
        "\u{320B}" => "\u{0028}\u{1110}\u{0029}",
        "\u{3219}" => "\u{0028}\u{D0C0}\u{0029}",
        "\u{320C}" => "\u{0028}\u{1111}\u{0029}",
        "\u{321A}" => "\u{0028}\u{D30C}\u{0029}",
        "\u{320D}" => "\u{0028}\u{1112}\u{0029}",
        "\u{321B}" => "\u{0028}\u{D558}\u{0029}",
        "\u{3226}" => "\u{0028}\u{4E03}\u{0029}",
        "\u{3222}" => "\u{0028}\u{4E09}\u{0029}",
        "\u{1F241}" => "\u{0028}\u{4E09}\u{0029}",
        "\u{3228}" => "\u{0028}\u{4E5D}\u{0029}",
        "\u{3221}" => "\u{0028}\u{4E8C}\u{0029}",
        "\u{1F242}" => "\u{0028}\u{4E8C}\u{0029}",
        "\u{3224}" => "\u{0028}\u{4E94}\u{0029}",
        "\u{3239}" => "\u{0028}\u{4EE3}\u{0029}",
        "\u{323D}" => "\u{0028}\u{4F01}\u{0029}",
        "\u{3241}" => "\u{0028}\u{4F11}\u{0029}",
        "\u{3227}" => "\u{0028}\u{516B}\u{0029}",
        "\u{3225}" => "\u{0028}\u{516D}\u{0029}",
        "\u{3238}" => "\u{0028}\u{52B4}\u{0029}",
        "\u{1F247}" => "\u{0028}\u{52DD}\u{0029}",
        "\u{3229}" => "\u{0028}\u{5341}\u{0029}",
        "\u{323F}" => "\u{0028}\u{5354}\u{0029}",
        "\u{3234}" => "\u{0028}\u{540D}\u{0029}",
        "\u{323A}" => "\u{0028}\u{547C}\u{0029}",
        "\u{3223}" => "\u{0028}\u{56DB}\u{0029}",
        "\u{322F}" => "\u{0028}\u{571F}\u{0029}",
        "\u{323B}" => "\u{0028}\u{5B66}\u{0029}",
        "\u{1F243}" => "\u{0028}\u{5B89}\u{0029}",
        "\u{1F245}" => "\u{0028}\u{6253}\u{0029}",
        "\u{1F248}" => "\u{0028}\u{6557}\u{0029}",
        "\u{3230}" => "\u{0028}\u{65E5}\u{0029}",
        "\u{322A}" => "\u{0028}\u{6708}\u{0029}",
        "\u{3232}" => "\u{0028}\u{6709}\u{0029}",
        "\u{322D}" => "\u{0028}\u{6728}\u{0029}",
        "\u{1F240}" => "\u{0028}\u{672C}\u{0029}",
        "\u{3231}" => "\u{0028}\u{682A}\u{0029}",
        "\u{322C}" => "\u{0028}\u{6C34}\u{0029}",
        "\u{322B}" => "\u{0028}\u{706B}\u{0029}",
        "\u{1F244}" => "\u{0028}\u{70B9}\u{0029}",
        "\u{3235}" => "\u{0028}\u{7279}\u{0029}",
        "\u{1F246}" => "\u{0028}\u{76D7}\u{0029}",
        "\u{323C}" => "\u{0028}\u{76E3}\u{0029}",
        "\u{3233}" => "\u{0028}\u{793E}\u{0029}",
        "\u{3237}" => "\u{0028}\u{795D}\u{0029}",
        "\u{3240}" => "\u{0028}\u{796D}\u{0029}",
        "\u{3242}" => "\u{0028}\u{81EA}\u{0029}",
        "\u{3243}" => "\u{0028}\u{81F3}\u{0029}",
        "\u{3236}" => "\u{0028}\u{8CA1}\u{0029}",
        "\u{323E}" => "\u{0028}\u{8CC7}\u{0029}",
        "\u{322E}" => "\u{0028}\u{91D1}\u{0029}",
        "\u{FF3D}" => "\u{0029}",
        "\u{2769}" => "\u{0029}",
        "\u{2773}" => "\u{0029}",
        "\u{3015}" => "\u{0029}",
        "\u{FD3F}" => "\u{0029}",
        "\u{2E29}" => "\u{0029}\u{0029}",
        "\u{2774}" => "\u{007B}",
        "\u{1D114}" => "\u{007B}",
        "\u{2775}" => "\u{007D}",
        "\u{301A}" => "\u{27E6}",
        "\u{301B}" => "\u{27E7}",
        "\u{27E8}" => "\u{276C}",
        "\u{2329}" => "\u{276C}",
        "\u{3008}" => "\u{276C}",
        "\u{31DB}" => "\u{276C}",
        "\u{304F}" => "\u{276C}",
        "\u{21FE8}" => "\u{276C}",
        "\u{27E9}" => "\u{276D}",
        "\u{232A}" => "\u{276D}",
        "\u{3009}" => "\u{276D}",
        "\u{FF3E}" => "\u{FE3F}",
        "\u{2E3F}" => "\u{00B6}",
        "\u{204E}" => "\u{002A}",
        "\u{066D}" => "\u{002A}",
        "\u{2217}" => "\u{002A}",
        "\u{1031F}" => "\u{002A}",
        "\u{1735}" => "\u{002F}",
        "\u{2041}" => "\u{002F}",
        "\u{2215}" => "\u{002F}",
        "\u{2044}" => "\u{002F}",
        "\u{2571}" => "\u{002F}",
        "\u{27CB}" => "\u{002F}",
        "\u{29F8}" => "\u{002F}",
        "\u{1D23A}" => "\u{002F}",
        "\u{31D3}" => "\u{002F}",
        "\u{3033}" => "\u{002F}",
        "\u{2CC6}" => "\u{002F}",
        "\u{30CE}" => "\u{002F}",
        "\u{4E3F}" => "\u{002F}",
        "\u{2F03}" => "\u{002F}",
        "\u{29F6}" => "\u{002F}\u{0304}",
        "\u{2AFD}" => "\u{002F}\u{002F}",
        "\u{2AFB}" => "\u{002F}\u{002F}\u{002F}",
        "\u{FF3C}" => "\u{005C}",
        "\u{FE68}" => "\u{005C}",
        "\u{2216}" => "\u{005C}",
        "\u{27CD}" => "\u{005C}",
        "\u{29F5}" => "\u{005C}",
        "\u{29F9}" => "\u{005C}",
        "\u{1D20F}" => "\u{005C}",
        "\u{1D23B}" => "\u{005C}",
        "\u{31D4}" => "\u{005C}",
        "\u{4E36}" => "\u{005C}",
        "\u{2F02}" => "\u{005C}",
        "\u{2CF9}" => "\u{005C}\u{005C}",
        "\u{244A}" => "\u{005C}\u{005C}",
        "\u{27C8}" => "\u{005C}\u{1455}",
        "\u{A778}" => "\u{0026}",
        "\u{0AF0}" => "\u{0970}",
        "\u{110BB}" => "\u{0970}",
        "\u{111C7}" => "\u{0970}",
        "\u{26AC}" => "\u{0970}",
        "\u{111DB}" => "\u{A8FC}",
        "\u{17D9}" => "\u{0E4F}",
        "\u{17D5}" => "\u{0E5A}",
        "\u{17DA}" => "\u{0E5B}",
        "\u{0F0C}" => "\u{0F0B}",
        "\u{0F0E}" => "\u{0F0D}\u{0F0D}",
        "\u{02C4}" => "\u{005E}",
        "\u{02C6}" => "\u{005E}",
        "\u{A67E}" => "\u{02C7}",
        "\u{02D8}" => "\u{02C7}",
        "\u{203E}" => "\u{02C9}",
        "\u{FE49}" => "\u{02C9}",
        "\u{FE4A}" => "\u{02C9}",
        "\u{FE4B}" => "\u{02C9}",
        "\u{FE4C}" => "\u{02C9}",
        "\u{00AF}" => "\u{02C9}",
        "\u{FFE3}" => "\u{02C9}",
        "\u{2594}" => "\u{02C9}",
        "\u{044A}" => "\u{02C9}\u{0062}",
        "\u{A651}" => "\u{02C9}\u{0062}\u{0069}",
        "\u{0375}" => "\u{02CF}",
        "\u{02FB}" => "\u{02EA}",
        "\u{A716}" => "\u{02EA}",
        "\u{A714}" => "\u{02EB}",
        "\u{3002}" => "\u{02F3}",
        "\u{2E30}" => "\u{00B0}",
        "\u{02DA}" => "\u{00B0}",
        "\u{2218}" => "\u{00B0}",
        "\u{25CB}" => "\u{00B0}",
        "\u{25E6}" => "\u{00B0}",
        "\u{235C}" => "\u{00B0}\u{0332}",
        "\u{2364}" => "\u{00B0}\u{0308}",
        "\u{2103}" => "\u{00B0}\u{0043}",
        "\u{2109}" => "\u{00B0}\u{0046}",
        "\u{0BF5}" => "\u{0BF3}",
        "\u{0F1B}" => "\u{0F1A}\u{0F1A}",
        "\u{0F1F}" => "\u{0F1A}\u{0F1D}",
        "\u{0FCE}" => "\u{0F1D}\u{0F1A}",
        "\u{0F1E}" => "\u{0F1D}\u{0F1D}",
        "\u{24B8}" => "\u{00A9}",
        "\u{24C7}" => "\u{00AE}",
        "\u{24C5}" => "\u{2117}",
        "\u{1D21B}" => "\u{2144}",
        "\u{2BEC}" => "\u{219E}",
        "\u{2BED}" => "\u{219F}",
        "\u{2BEE}" => "\u{21A0}",
        "\u{2BEF}" => "\u{21A1}",
        "\u{21B5}" => "\u{21B2}",
        "\u{2965}" => "\u{21C3}\u{21C2}",
        "\u{296F}" => "\u{21C3}\u{16DA}",
        "\u{1D6DB}" => "\u{2202}",
        "\u{1D715}" => "\u{2202}",
        "\u{1D74F}" => "\u{2202}",
        "\u{1D789}" => "\u{2202}",
        "\u{1D7C3}" => "\u{2202}",
        "\u{1E8CC}" => "\u{2202}",
        "\u{1E8CD}" => "\u{2202}\u{0335}",
        "\u{00F0}" => "\u{2202}\u{0335}",
        "\u{2300}" => "\u{2205}",
        "\u{1D6C1}" => "\u{2207}",
        "\u{1D6FB}" => "\u{2207}",
        "\u{1D735}" => "\u{2207}",
        "\u{1D76F}" => "\u{2207}",
        "\u{1D7A9}" => "\u{2207}",
        "\u{118A8}" => "\u{2207}",
        "\u{2362}" => "\u{2207}\u{0308}",
        "\u{236B}" => "\u{2207}\u{0334}",
        "\u{2588}" => "\u{220E}",
        "\u{25A0}" => "\u{220E}",
        "\u{2A3F}" => "\u{2210}",
        "\u{16ED}" => "\u{002B}",
        "\u{2795}" => "\u{002B}",
        "\u{1029B}" => "\u{002B}",
        "\u{2A23}" => "\u{002B}\u{0302}",
        "\u{2A22}" => "\u{002B}\u{030A}",
        "\u{2A24}" => "\u{002B}\u{0303}",
        "\u{2214}" => "\u{002B}\u{0307}",
        "\u{2A25}" => "\u{002B}\u{0323}",
        "\u{2A26}" => "\u{002B}\u{0330}",
        "\u{2A27}" => "\u{002B}\u{2082}",
        "\u{2797}" => "\u{00F7}",
        "\u{2039}" => "\u{003C}",
        "\u{276E}" => "\u{003C}",
        "\u{02C2}" => "\u{003C}",
        "\u{1D236}" => "\u{003C}",
        "\u{1438}" => "\u{003C}",
        "\u{16B2}" => "\u{003C}",
        "\u{22D6}" => "\u{003C}\u{00B7}",
        "\u{2CB4}" => "\u{003C}\u{00B7}",
        "\u{1445}" => "\u{003C}\u{00B7}",
        "\u{226A}" => "\u{003C}\u{003C}",
        "\u{22D8}" => "\u{003C}\u{003C}\u{003C}",
        "\u{1400}" => "\u{003D}",
        "\u{2E40}" => "\u{003D}",
        "\u{30A0}" => "\u{003D}",
        "\u{A4FF}" => "\u{003D}",
        "\u{225A}" => "\u{003D}\u{0306}",
        "\u{2259}" => "\u{003D}\u{0302}",
        "\u{2257}" => "\u{003D}\u{030A}",
        "\u{2250}" => "\u{003D}\u{0307}",
        "\u{2251}" => "\u{003D}\u{0307}\u{0323}",
        "\u{2A6E}" => "\u{003D}\u{20F0}",
        "\u{2A75}" => "\u{003D}\u{003D}",
        "\u{2A76}" => "\u{003D}\u{003D}\u{003D}",
        "\u{225E}" => "\u{003D}\u{036B}",
        "\u{203A}" => "\u{003E}",
        "\u{276F}" => "\u{003E}",
        "\u{02C3}" => "\u{003E}",
        "\u{1D237}" => "\u{003E}",
        "\u{1433}" => "\u{003E}",
        "\u{16F3F}" => "\u{003E}",
        "\u{1441}" => "\u{003E}\u{00B7}",
        "\u{2AA5}" => "\u{003E}\u{003C}",
        "\u{226B}" => "\u{003E}\u{003E}",
        "\u{2A20}" => "\u{003E}\u{003E}",
        "\u{22D9}" => "\u{003E}\u{003E}\u{003E}",
        "\u{2053}" => "\u{007E}",
        "\u{02DC}" => "\u{007E}",
        "\u{1FC0}" => "\u{007E}",
        "\u{223C}" => "\u{007E}",
        "\u{2368}" => "\u{007E}\u{0308}",
        "\u{2E1E}" => "\u{007E}\u{0307}",
        "\u{2A6A}" => "\u{007E}\u{0307}",
        "\u{2E1F}" => "\u{007E}\u{0323}",
        "\u{1E8C8}" => "\u{2220}",
        "\u{22C0}" => "\u{2227}",
        "\u{222F}" => "\u{222E}\u{222E}",
        "\u{2230}" => "\u{222E}\u{222E}\u{222E}",
        "\u{2E2B}" => "\u{2234}",
        "\u{2E2A}" => "\u{2235}",
        "\u{2E2C}" => "\u{2237}",
        "\u{111DE}" => "\u{2248}",
        "\u{264E}" => "\u{224F}",
        "\u{1F75E}" => "\u{224F}",
        "\u{2263}" => "\u{2261}",
        "\u{2A03}" => "\u{228D}",
        "\u{2A04}" => "\u{228E}",
        "\u{1D238}" => "\u{228F}",
        "\u{1D239}" => "\u{2290}",
        "\u{2A05}" => "\u{2293}",
        "\u{2A06}" => "\u{2294}",
        "\u{2A02}" => "\u{2297}",
        "\u{235F}" => "\u{229B}",
        "\u{1F771}" => "\u{22A0}",
        "\u{1F755}" => "\u{22A1}",
        "\u{25C1}" => "\u{22B2}",
        "\u{25B7}" => "\u{22B3}",
        "\u{2363}" => "\u{22C6}\u{0308}",
        "\u{FE34}" => "\u{2307}",
        "\u{25E0}" => "\u{2312}",
        "\u{2A3D}" => "\u{2319}",
        "\u{2325}" => "\u{2324}",
        "\u{29C7}" => "\u{233B}",
        "\u{25CE}" => "\u{233E}",
        "\u{29BE}" => "\u{233E}",
        "\u{29C5}" => "\u{2342}",
        "\u{29B0}" => "\u{2349}",
        "\u{23C3}" => "\u{234B}",
        "\u{23C2}" => "\u{234E}",
        "\u{23C1}" => "\u{2355}",
        "\u{23C6}" => "\u{236D}",
        "\u{2638}" => "\u{2388}",
        "\u{FE35}" => "\u{23DC}",
        "\u{FE36}" => "\u{23DD}",
        "\u{FE37}" => "\u{23DE}",
        "\u{FE38}" => "\u{23DF}",
        "\u{FE39}" => "\u{23E0}",
        "\u{FE3A}" => "\u{23E1}",
        "\u{25B1}" => "\u{23E5}",
        "\u{23FC}" => "\u{23FB}",
        "\u{FE31}" => "\u{2502}",
        "\u{FF5C}" => "\u{2502}",
        "\u{2503}" => "\u{2502}",
        "\u{250F}" => "\u{250C}",
        "\u{2523}" => "\u{251C}",
        "\u{2590}" => "\u{258C}",
        "\u{2597}" => "\u{2596}",
        "\u{259D}" => "\u{2598}",
        "\u{2610}" => "\u{25A1}",
        "\u{FFED}" => "\u{25AA}",
        "\u{25B8}" => "\u{25B6}",
        "\u{25BA}" => "\u{25B6}",
        "\u{2CE9}" => "\u{2627}",
        "\u{1F70A}" => "\u{2629}",
        "\u{1F312}" => "\u{263D}",
        "\u{1F319}" => "\u{263D}",
        "\u{23FE}" => "\u{263E}",
        "\u{1F318}" => "\u{263E}",
        "\u{29D9}" => "\u{299A}",
        "\u{1F73A}" => "\u{29DF}",
        "\u{2A3E}" => "\u{2A1F}",
        "\u{101A0}" => "\u{2CE8}",
        "\u{2669}" => "\u{1D158}\u{1D165}",
        "\u{266A}" => "\u{1D158}\u{1D165}\u{1D16E}",
        "\u{24EA}" => "\u{1F10D}",
        "\u{21BA}" => "\u{1F10E}",
        "\u{02D9}" => "\u{0971}",
        "\u{0D4E}" => "\u{0971}",
        "\u{FF0D}" => "\u{30FC}",
        "\u{2014}" => "\u{30FC}",
        "\u{2015}" => "\u{30FC}",
        "\u{2500}" => "\u{30FC}",
        "\u{2501}" => "\u{30FC}",
        "\u{31D0}" => "\u{30FC}",
        "\u{A7F7}" => "\u{30FC}",
        "\u{1173}" => "\u{30FC}",
        "\u{3161}" => "\u{30FC}",
        "\u{4E00}" => "\u{30FC}",
        "\u{2F00}" => "\u{30FC}",
        "\u{1196}" => "\u{30FC}\u{30FC}",
        "\u{D7B9}" => "\u{30FC}\u{1161}",
        "\u{D7BA}" => "\u{30FC}\u{1165}",
        "\u{D7BB}" => "\u{30FC}\u{1165}\u{4E28}",
        "\u{D7BC}" => "\u{30FC}\u{1169}",
        "\u{1195}" => "\u{30FC}\u{116E}",
        "\u{1174}" => "\u{30FC}\u{4E28}",
        "\u{3162}" => "\u{30FC}\u{4E28}",
        "\u{1197}" => "\u{30FC}\u{4E28}\u{116E}",
        "\u{1F10F}" => "\u{0024}\u{20E0}",
        "\u{20A4}" => "\u{00A3}",
        "\u{3012}" => "\u{20B8}",
        "\u{3036}" => "\u{20B8}",
        "\u{1B5C}" => "\u{1B50}",
        "\u{A9C6}" => "\u{A9D0}",
        "\u{114D1}" => "\u{09E7}",
        "\u{0CE7}" => "\u{0C67}",
        "\u{1065}" => "\u{1041}",
        "\u{2460}" => "\u{2780}",
        "\u{2469}" => "\u{2789}",
        "\u{23E8}" => "\u{2081}\u{2080}",
        "\u{1D7D0}" => "\u{0032}",
        "\u{1D7DA}" => "\u{0032}",
        "\u{1D7E4}" => "\u{0032}",
        "\u{1D7EE}" => "\u{0032}",
        "\u{1D7F8}" => "\u{0032}",
        "\u{1FBF2}" => "\u{0032}",
        "\u{A75A}" => "\u{0032}",
        "\u{01A7}" => "\u{0032}",
        "\u{03E8}" => "\u{0032}",
        "\u{A644}" => "\u{0032}",
        "\u{14BF}" => "\u{0032}",
        "\u{A6EF}" => "\u{0032}",
        "\u{A9CF}" => "\u{0662}",
        "\u{06F2}" => "\u{0662}",
        "\u{0AE8}" => "\u{0968}",
        "\u{114D2}" => "\u{09E8}",
        "\u{0CE8}" => "\u{0C68}",
        "\u{2461}" => "\u{2781}",
        "\u{01BB}" => "\u{0032}\u{0335}",
        "\u{1F103}" => "\u{0032}\u{002C}",
        "\u{2489}" => "\u{0032}\u{002E}",
        "\u{33F5}" => "\u{0032}\u{0032}\u{65E5}",
        "\u{336E}" => "\u{0032}\u{0032}\u{70B9}",
        "\u{33F6}" => "\u{0032}\u{0033}\u{65E5}",
        "\u{336F}" => "\u{0032}\u{0033}\u{70B9}",
        "\u{33F7}" => "\u{0032}\u{0034}\u{65E5}",
        "\u{3370}" => "\u{0032}\u{0034}\u{70B9}",
        "\u{33F8}" => "\u{0032}\u{0035}\u{65E5}",
        "\u{33F9}" => "\u{0032}\u{0036}\u{65E5}",
        "\u{33FA}" => "\u{0032}\u{0037}\u{65E5}",
        "\u{33FB}" => "\u{0032}\u{0038}\u{65E5}",
        "\u{33FC}" => "\u{0032}\u{0039}\u{65E5}",
        "\u{33F4}" => "\u{0032}\u{006C}\u{65E5}",
        "\u{336D}" => "\u{0032}\u{006C}\u{70B9}",
        "\u{249B}" => "\u{0032}\u{004F}\u{002E}",
        "\u{33F3}" => "\u{0032}\u{004F}\u{65E5}",
        "\u{336C}" => "\u{0032}\u{004F}\u{70B9}",
        "\u{0DE9}" => "\u{0DE8}\u{0DCF}",
        "\u{0DEF}" => "\u{0DE8}\u{0DD3}",
        "\u{33E1}" => "\u{0032}\u{65E5}",
        "\u{32C1}" => "\u{0032}\u{6708}",
        "\u{335A}" => "\u{0032}\u{70B9}",
        "\u{1D206}" => "\u{0033}",
        "\u{1D7D1}" => "\u{0033}",
        "\u{1D7DB}" => "\u{0033}",
        "\u{1D7E5}" => "\u{0033}",
        "\u{1D7EF}" => "\u{0033}",
        "\u{1D7F9}" => "\u{0033}",
        "\u{1FBF3}" => "\u{0033}",
        "\u{A7AB}" => "\u{0033}",
        "\u{021C}" => "\u{0033}",
        "\u{01B7}" => "\u{0033}",
        "\u{A76A}" => "\u{0033}",
        "\u{2CCC}" => "\u{0033}",
        "\u{0417}" => "\u{0033}",
        "\u{04E0}" => "\u{0033}",
        "\u{16F3B}" => "\u{0033}",
        "\u{118CA}" => "\u{0033}",
        "\u{06F3}" => "\u{0663}",
        "\u{1E8C9}" => "\u{0663}",
        "\u{0AE9}" => "\u{0969}",
        "\u{2462}" => "\u{2782}",
        "\u{0498}" => "\u{0033}\u{0326}",
        "\u{1F104}" => "\u{0033}\u{002C}",
        "\u{248A}" => "\u{0033}\u{002E}",
        "\u{33FE}" => "\u{0033}\u{006C}\u{65E5}",
        "\u{33FD}" => "\u{0033}\u{004F}\u{65E5}",
        "\u{33E2}" => "\u{0033}\u{65E5}",
        "\u{32C2}" => "\u{0033}\u{6708}",
        "\u{335B}" => "\u{0033}\u{70B9}",
        "\u{1D7D2}" => "\u{0034}",
        "\u{1D7DC}" => "\u{0034}",
        "\u{1D7E6}" => "\u{0034}",
        "\u{1D7F0}" => "\u{0034}",
        "\u{1D7FA}" => "\u{0034}",
        "\u{1FBF4}" => "\u{0034}",
        "\u{13CE}" => "\u{0034}",
        "\u{118AF}" => "\u{0034}",
        "\u{06F4}" => "\u{0664}",
        "\u{0AEA}" => "\u{096A}",
        "\u{2463}" => "\u{2783}",
        "\u{1F105}" => "\u{0034}\u{002C}",
        "\u{248B}" => "\u{0034}\u{002E}",
        "\u{1530}" => "\u{0034}\u{00B7}",
        "\u{33E3}" => "\u{0034}\u{65E5}",
        "\u{32C3}" => "\u{0034}\u{6708}",
        "\u{335C}" => "\u{0034}\u{70B9}",
        "\u{1D7D3}" => "\u{0035}",
        "\u{1D7DD}" => "\u{0035}",
        "\u{1D7E7}" => "\u{0035}",
        "\u{1D7F1}" => "\u{0035}",
        "\u{1D7FB}" => "\u{0035}",
        "\u{1FBF5}" => "\u{0035}",
        "\u{01BC}" => "\u{0035}",
        "\u{118BB}" => "\u{0035}",
        "\u{2464}" => "\u{2784}",
        "\u{1F106}" => "\u{0035}\u{002C}",
        "\u{248C}" => "\u{0035}\u{002E}",
        "\u{33E4}" => "\u{0035}\u{65E5}",
        "\u{32C4}" => "\u{0035}\u{6708}",
        "\u{335D}" => "\u{0035}\u{70B9}",
        "\u{1D7D4}" => "\u{0036}",
        "\u{1D7DE}" => "\u{0036}",
        "\u{1D7E8}" => "\u{0036}",
        "\u{1D7F2}" => "\u{0036}",
        "\u{1D7FC}" => "\u{0036}",
        "\u{1FBF6}" => "\u{0036}",
        "\u{2CD2}" => "\u{0036}",
        "\u{0431}" => "\u{0036}",
        "\u{13EE}" => "\u{0036}",
        "\u{118D5}" => "\u{0036}",
        "\u{06F6}" => "\u{0666}",
        "\u{114D6}" => "\u{09EC}",
        "\u{2465}" => "\u{2785}",
        "\u{1F107}" => "\u{0036}\u{002C}",
        "\u{248D}" => "\u{0036}\u{002E}",
        "\u{33E5}" => "\u{0036}\u{65E5}",
        "\u{32C5}" => "\u{0036}\u{6708}",
        "\u{335E}" => "\u{0036}\u{70B9}",
        "\u{1D212}" => "\u{0037}",
        "\u{1D7D5}" => "\u{0037}",
        "\u{1D7DF}" => "\u{0037}",
        "\u{1D7E9}" => "\u{0037}",
        "\u{1D7F3}" => "\u{0037}",
        "\u{1D7FD}" => "\u{0037}",
        "\u{1FBF7}" => "\u{0037}",
        "\u{104D2}" => "\u{0037}",
        "\u{118C6}" => "\u{0037}",
        "\u{2466}" => "\u{2786}",
        "\u{1F108}" => "\u{0037}\u{002C}",
        "\u{248E}" => "\u{0037}\u{002E}",
        "\u{33E6}" => "\u{0037}\u{65E5}",
        "\u{32C6}" => "\u{0037}\u{6708}",
        "\u{335F}" => "\u{0037}\u{70B9}",
        "\u{0B03}" => "\u{0038}",
        "\u{09EA}" => "\u{0038}",
        "\u{0A6A}" => "\u{0038}",
        "\u{1E8CB}" => "\u{0038}",
        "\u{1D7D6}" => "\u{0038}",
        "\u{1D7E0}" => "\u{0038}",
        "\u{1D7EA}" => "\u{0038}",
        "\u{1D7F4}" => "\u{0038}",
        "\u{1D7FE}" => "\u{0038}",
        "\u{1FBF8}" => "\u{0038}",
        "\u{0223}" => "\u{0038}",
        "\u{0222}" => "\u{0038}",
        "\u{1031A}" => "\u{0038}",
        "\u{0AEE}" => "\u{096E}",
        "\u{2467}" => "\u{2787}",
        "\u{1F109}" => "\u{0038}\u{002C}",
        "\u{248F}" => "\u{0038}\u{002E}",
        "\u{33E7}" => "\u{0038}\u{65E5}",
        "\u{32C7}" => "\u{0038}\u{6708}",
        "\u{3360}" => "\u{0038}\u{70B9}",
        "\u{0A67}" => "\u{0039}",
        "\u{0B68}" => "\u{0039}",
        "\u{09ED}" => "\u{0039}",
        "\u{0D6D}" => "\u{0039}",
        "\u{1D7D7}" => "\u{0039}",
        "\u{1D7E1}" => "\u{0039}",
        "\u{1D7EB}" => "\u{0039}",
        "\u{1D7F5}" => "\u{0039}",
        "\u{1D7FF}" => "\u{0039}",
        "\u{1FBF9}" => "\u{0039}",
        "\u{A76E}" => "\u{0039}",
        "\u{2CCA}" => "\u{0039}",
        "\u{118CC}" => "\u{0039}",
        "\u{118AC}" => "\u{0039}",
        "\u{118D6}" => "\u{0039}",
        "\u{0967}" => "\u{0669}",
        "\u{118E4}" => "\u{0669}",
        "\u{06F9}" => "\u{0669}",
        "\u{0CEF}" => "\u{0C6F}",
        "\u{2468}" => "\u{2788}",
        "\u{1F10A}" => "\u{0039}\u{002C}",
        "\u{2490}" => "\u{0039}\u{002E}",
        "\u{33E8}" => "\u{0039}\u{65E5}",
        "\u{32C8}" => "\u{0039}\u{6708}",
        "\u{3361}" => "\u{0039}\u{70B9}",
        "\u{237A}" => "\u{0061}",
        "\u{FF41}" => "\u{0061}",
        "\u{1D41A}" => "\u{0061}",
        "\u{1D44E}" => "\u{0061}",
        "\u{1D482}" => "\u{0061}",
        "\u{1D4B6}" => "\u{0061}",
        "\u{1D4EA}" => "\u{0061}",
        "\u{1D51E}" => "\u{0061}",
        "\u{1D552}" => "\u{0061}",
        "\u{1D586}" => "\u{0061}",
        "\u{1D5BA}" => "\u{0061}",
        "\u{1D5EE}" => "\u{0061}",
        "\u{1D622}" => "\u{0061}",
        "\u{1D656}" => "\u{0061}",
        "\u{1D68A}" => "\u{0061}",
        "\u{0251}" => "\u{0061}",
        "\u{03B1}" => "\u{0061}",
        "\u{1D6C2}" => "\u{0061}",
        "\u{1D6FC}" => "\u{0061}",
        "\u{1D736}" => "\u{0061}",
        "\u{1D770}" => "\u{0061}",
        "\u{1D7AA}" => "\u{0061}",
        "\u{0430}" => "\u{0061}",
        "\u{2DF6}" => "\u{0363}",
        "\u{FF21}" => "\u{0041}",
        "\u{1D400}" => "\u{0041}",
        "\u{1D434}" => "\u{0041}",
        "\u{1D468}" => "\u{0041}",
        "\u{1D49C}" => "\u{0041}",
        "\u{1D4D0}" => "\u{0041}",
        "\u{1D504}" => "\u{0041}",
        "\u{1D538}" => "\u{0041}",
        "\u{1D56C}" => "\u{0041}",
        "\u{1D5A0}" => "\u{0041}",
        "\u{1D5D4}" => "\u{0041}",
        "\u{1D608}" => "\u{0041}",
        "\u{1D63C}" => "\u{0041}",
        "\u{1D670}" => "\u{0041}",
        "\u{0391}" => "\u{0041}",
        "\u{1D6A8}" => "\u{0041}",
        "\u{1D6E2}" => "\u{0041}",
        "\u{1D71C}" => "\u{0041}",
        "\u{1D756}" => "\u{0041}",
        "\u{1D790}" => "\u{0041}",
        "\u{0410}" => "\u{0041}",
        "\u{13AA}" => "\u{0041}",
        "\u{15C5}" => "\u{0041}",
        "\u{A4EE}" => "\u{0041}",
        "\u{16F40}" => "\u{0041}",
        "\u{102A0}" => "\u{0041}",
        "\u{2376}" => "\u{0061}\u{0332}",
        "\u{01CE}" => "\u{0103}",
        "\u{01CD}" => "\u{0102}",
        "\u{0227}" => "\u{00E5}",
        "\u{0226}" => "\u{00C5}",
        "\u{1E9A}" => "\u{1EA3}",
        "\u{2100}" => "\u{0061}\u{002F}\u{0063}",
        "\u{2101}" => "\u{0061}\u{002F}\u{0073}",
        "\u{A733}" => "\u{0061}\u{0061}",
        "\u{A732}" => "\u{0041}\u{0041}",
        "\u{00E6}" => "\u{0061}\u{0065}",
        "\u{04D5}" => "\u{0061}\u{0065}",
        "\u{00C6}" => "\u{0041}\u{0045}",
        "\u{04D4}" => "\u{0041}\u{0045}",
        "\u{A735}" => "\u{0061}\u{006F}",
        "\u{A734}" => "\u{0041}\u{004F}",
        "\u{1F707}" => "\u{0041}\u{0052}",
        "\u{A737}" => "\u{0061}\u{0075}",
        "\u{A736}" => "\u{0041}\u{0055}",
        "\u{A739}" => "\u{0061}\u{0076}",
        "\u{A73B}" => "\u{0061}\u{0076}",
        "\u{A738}" => "\u{0041}\u{0056}",
        "\u{A73A}" => "\u{0041}\u{0056}",
        "\u{A73D}" => "\u{0061}\u{0079}",
        "\u{A73C}" => "\u{0041}\u{0059}",
        "\u{AB7A}" => "\u{1D00}",
        "\u{2200}" => "\u{2C6F}",
        "\u{1D217}" => "\u{2C6F}",
        "\u{15C4}" => "\u{2C6F}",
        "\u{A4EF}" => "\u{2C6F}",
        "\u{1041F}" => "\u{2C70}",
        "\u{1D41B}" => "\u{0062}",
        "\u{1D44F}" => "\u{0062}",
        "\u{1D483}" => "\u{0062}",
        "\u{1D4B7}" => "\u{0062}",
        "\u{1D4EB}" => "\u{0062}",
        "\u{1D51F}" => "\u{0062}",
        "\u{1D553}" => "\u{0062}",
        "\u{1D587}" => "\u{0062}",
        "\u{1D5BB}" => "\u{0062}",
        "\u{1D5EF}" => "\u{0062}",
        "\u{1D623}" => "\u{0062}",
        "\u{1D657}" => "\u{0062}",
        "\u{1D68B}" => "\u{0062}",
        "\u{0184}" => "\u{0062}",
        "\u{042C}" => "\u{0062}",
        "\u{13CF}" => "\u{0062}",
        "\u{1472}" => "\u{0062}",
        "\u{15AF}" => "\u{0062}",
        "\u{FF22}" => "\u{0042}",
        "\u{212C}" => "\u{0042}",
        "\u{1D401}" => "\u{0042}",
        "\u{1D435}" => "\u{0042}",
        "\u{1D469}" => "\u{0042}",
        "\u{1D4D1}" => "\u{0042}",
        "\u{1D505}" => "\u{0042}",
        "\u{1D539}" => "\u{0042}",
        "\u{1D56D}" => "\u{0042}",
        "\u{1D5A1}" => "\u{0042}",
        "\u{1D5D5}" => "\u{0042}",
        "\u{1D609}" => "\u{0042}",
        "\u{1D63D}" => "\u{0042}",
        "\u{1D671}" => "\u{0042}",
        "\u{A7B4}" => "\u{0042}",
        "\u{0392}" => "\u{0042}",
        "\u{1D6A9}" => "\u{0042}",
        "\u{1D6E3}" => "\u{0042}",
        "\u{1D71D}" => "\u{0042}",
        "\u{1D757}" => "\u{0042}",
        "\u{1D791}" => "\u{0042}",
        "\u{0412}" => "\u{0042}",
        "\u{13F4}" => "\u{0042}",
        "\u{15F7}" => "\u{0042}",
        "\u{A4D0}" => "\u{0042}",
        "\u{10282}" => "\u{0042}",
        "\u{102A1}" => "\u{0042}",
        "\u{10301}" => "\u{0042}",
        "\u{0253}" => "\u{0062}\u{0314}",
        "\u{1473}" => "\u{0062}\u{0307}",
        "\u{0183}" => "\u{0062}\u{0304}",
        "\u{0182}" => "\u{0062}\u{0304}",
        "\u{0411}" => "\u{0062}\u{0304}",
        "\u{0180}" => "\u{0062}\u{0335}",
        "\u{048D}" => "\u{0062}\u{0335}",
        "\u{048C}" => "\u{0062}\u{0335}",
        "\u{0463}" => "\u{0062}\u{0335}",
        "\u{0462}" => "\u{0062}\u{0335}",
        "\u{147F}" => "\u{0062}\u{00B7}",
        "\u{1481}" => "\u{0062}\u{0307}\u{00B7}",
        "\u{1488}" => "\u{0062}\u{0027}",
        "\u{042B}" => "\u{0062}\u{006C}",
        "\u{0432}" => "\u{0299}",
        "\u{13FC}" => "\u{0299}",
        "\u{FF43}" => "\u{0063}",
        "\u{217D}" => "\u{0063}",
        "\u{1D41C}" => "\u{0063}",
        "\u{1D450}" => "\u{0063}",
        "\u{1D484}" => "\u{0063}",
        "\u{1D4B8}" => "\u{0063}",
        "\u{1D4EC}" => "\u{0063}",
        "\u{1D520}" => "\u{0063}",
        "\u{1D554}" => "\u{0063}",
        "\u{1D588}" => "\u{0063}",
        "\u{1D5BC}" => "\u{0063}",
        "\u{1D5F0}" => "\u{0063}",
        "\u{1D624}" => "\u{0063}",
        "\u{1D658}" => "\u{0063}",
        "\u{1D68C}" => "\u{0063}",
        "\u{1D04}" => "\u{0063}",
        "\u{03F2}" => "\u{0063}",
        "\u{2CA5}" => "\u{0063}",
        "\u{0441}" => "\u{0063}",
        "\u{ABAF}" => "\u{0063}",
        "\u{1043D}" => "\u{0063}",
        "\u{2DED}" => "\u{0368}",
        "\u{1F74C}" => "\u{0043}",
        "\u{118F2}" => "\u{0043}",
        "\u{118E9}" => "\u{0043}",
        "\u{FF23}" => "\u{0043}",
        "\u{216D}" => "\u{0043}",
        "\u{2102}" => "\u{0043}",
        "\u{212D}" => "\u{0043}",
        "\u{1D402}" => "\u{0043}",
        "\u{1D436}" => "\u{0043}",
        "\u{1D46A}" => "\u{0043}",
        "\u{1D49E}" => "\u{0043}",
        "\u{1D4D2}" => "\u{0043}",
        "\u{1D56E}" => "\u{0043}",
        "\u{1D5A2}" => "\u{0043}",
        "\u{1D5D6}" => "\u{0043}",
        "\u{1D60A}" => "\u{0043}",
        "\u{1D63E}" => "\u{0043}",
        "\u{1D672}" => "\u{0043}",
        "\u{03F9}" => "\u{0043}",
        "\u{2CA4}" => "\u{0043}",
        "\u{0421}" => "\u{0043}",
        "\u{13DF}" => "\u{0043}",
        "\u{A4DA}" => "\u{0043}",
        "\u{102A2}" => "\u{0043}",
        "\u{10302}" => "\u{0043}",
        "\u{10415}" => "\u{0043}",
        "\u{1051C}" => "\u{0043}",
        "\u{00A2}" => "\u{0063}\u{0338}",
        "\u{023C}" => "\u{0063}\u{0338}",
        "\u{20A1}" => "\u{0043}\u{20EB}",
        "\u{1F16E}" => "\u{0043}\u{20E0}",
        "\u{00E7}" => "\u{0063}\u{0326}",
        "\u{04AB}" => "\u{0063}\u{0326}",
        "\u{00C7}" => "\u{0043}\u{0326}",
        "\u{04AA}" => "\u{0043}\u{0326}",
        "\u{0187}" => "\u{0043}\u{0027}",
        "\u{2105}" => "\u{0063}\u{002F}\u{006F}",
        "\u{2106}" => "\u{0063}\u{002F}\u{0075}",
        "\u{1F16D}" => "\u{33C4}\u{0009}\u{20DD}",
        "\u{22F4}" => "\u{A793}",
        "\u{025B}" => "\u{A793}",
        "\u{03B5}" => "\u{A793}",
        "\u{03F5}" => "\u{A793}",
        "\u{1D6C6}" => "\u{A793}",
        "\u{1D6DC}" => "\u{A793}",
        "\u{1D700}" => "\u{A793}",
        "\u{1D716}" => "\u{A793}",
        "\u{1D73A}" => "\u{A793}",
        "\u{1D750}" => "\u{A793}",
        "\u{1D774}" => "\u{A793}",
        "\u{1D78A}" => "\u{A793}",
        "\u{1D7AE}" => "\u{A793}",
        "\u{1D7C4}" => "\u{A793}",
        "\u{2C89}" => "\u{A793}",
        "\u{0454}" => "\u{A793}",
        "\u{0511}" => "\u{A793}",
        "\u{AB9B}" => "\u{A793}",
        "\u{118CE}" => "\u{A793}",
        "\u{10429}" => "\u{A793}",
        "\u{20AC}" => "\u{A792}",
        "\u{2C88}" => "\u{A792}",
        "\u{0404}" => "\u{A792}",
        "\u{2377}" => "\u{A793}\u{0332}",
        "\u{037D}" => "\u{A73F}",
        "\u{03FF}" => "\u{A73E}",
        "\u{217E}" => "\u{0064}",
        "\u{2146}" => "\u{0064}",
        "\u{1D41D}" => "\u{0064}",
        "\u{1D451}" => "\u{0064}",
        "\u{1D485}" => "\u{0064}",
        "\u{1D4B9}" => "\u{0064}",
        "\u{1D4ED}" => "\u{0064}",
        "\u{1D521}" => "\u{0064}",
        "\u{1D555}" => "\u{0064}",
        "\u{1D589}" => "\u{0064}",
        "\u{1D5BD}" => "\u{0064}",
        "\u{1D5F1}" => "\u{0064}",
        "\u{1D625}" => "\u{0064}",
        "\u{1D659}" => "\u{0064}",
        "\u{1D68D}" => "\u{0064}",
        "\u{0501}" => "\u{0064}",
        "\u{13E7}" => "\u{0064}",
        "\u{146F}" => "\u{0064}",
        "\u{A4D2}" => "\u{0064}",
        "\u{216E}" => "\u{0044}",
        "\u{2145}" => "\u{0044}",
        "\u{1D403}" => "\u{0044}",
        "\u{1D437}" => "\u{0044}",
        "\u{1D46B}" => "\u{0044}",
        "\u{1D49F}" => "\u{0044}",
        "\u{1D4D3}" => "\u{0044}",
        "\u{1D507}" => "\u{0044}",
        "\u{1D53B}" => "\u{0044}",
        "\u{1D56F}" => "\u{0044}",
        "\u{1D5A3}" => "\u{0044}",
        "\u{1D5D7}" => "\u{0044}",
        "\u{1D60B}" => "\u{0044}",
        "\u{1D63F}" => "\u{0044}",
        "\u{1D673}" => "\u{0044}",
        "\u{13A0}" => "\u{0044}",
        "\u{15DE}" => "\u{0044}",
        "\u{15EA}" => "\u{0044}",
        "\u{A4D3}" => "\u{0044}",
        "\u{0257}" => "\u{0064}\u{0314}",
        "\u{0256}" => "\u{0064}\u{0328}",
        "\u{018C}" => "\u{0064}\u{0304}",
        "\u{0111}" => "\u{0064}\u{0335}",
        "\u{0110}" => "\u{0044}\u{0335}",
        "\u{00D0}" => "\u{0044}\u{0335}",
        "\u{0189}" => "\u{0044}\u{0335}",
        "\u{20AB}" => "\u{0064}\u{0335}\u{0331}",
        "\u{A77A}" => "\u{A779}",
        "\u{147B}" => "\u{0064}\u{00B7}",
        "\u{1487}" => "\u{0064}\u{0027}",
        "\u{02A4}" => "\u{0064}\u{021D}",
        "\u{01F3}" => "\u{0064}\u{007A}",
        "\u{02A3}" => "\u{0064}\u{007A}",
        "\u{01F2}" => "\u{0044}\u{007A}",
        "\u{01F1}" => "\u{0044}\u{005A}",
        "\u{01C6}" => "\u{0064}\u{017E}",
        "\u{01C5}" => "\u{0044}\u{017E}",
        "\u{01C4}" => "\u{0044}\u{017D}",
        "\u{02A5}" => "\u{0064}\u{0291}",
        "\u{AB70}" => "\u{1D05}",
        "\u{2E39}" => "\u{1E9F}",
        "\u{03B4}" => "\u{1E9F}",
        "\u{1D6C5}" => "\u{1E9F}",
        "\u{1D6FF}" => "\u{1E9F}",
        "\u{1D739}" => "\u{1E9F}",
        "\u{1D773}" => "\u{1E9F}",
        "\u{1D7AD}" => "\u{1E9F}",
        "\u{056E}" => "\u{1E9F}",
        "\u{1577}" => "\u{1E9F}",
        "\u{212E}" => "\u{0065}",
        "\u{FF45}" => "\u{0065}",
        "\u{212F}" => "\u{0065}",
        "\u{2147}" => "\u{0065}",
        "\u{1D41E}" => "\u{0065}",
        "\u{1D452}" => "\u{0065}",
        "\u{1D486}" => "\u{0065}",
        "\u{1D4EE}" => "\u{0065}",
        "\u{1D522}" => "\u{0065}",
        "\u{1D556}" => "\u{0065}",
        "\u{1D58A}" => "\u{0065}",
        "\u{1D5BE}" => "\u{0065}",
        "\u{1D5F2}" => "\u{0065}",
        "\u{1D626}" => "\u{0065}",
        "\u{1D65A}" => "\u{0065}",
        "\u{1D68E}" => "\u{0065}",
        "\u{AB32}" => "\u{0065}",
        "\u{0435}" => "\u{0065}",
        "\u{04BD}" => "\u{0065}",
        "\u{2DF7}" => "\u{0364}",
        "\u{22FF}" => "\u{0045}",
        "\u{FF25}" => "\u{0045}",
        "\u{2130}" => "\u{0045}",
        "\u{1D404}" => "\u{0045}",
        "\u{1D438}" => "\u{0045}",
        "\u{1D46C}" => "\u{0045}",
        "\u{1D4D4}" => "\u{0045}",
        "\u{1D508}" => "\u{0045}",
        "\u{1D53C}" => "\u{0045}",
        "\u{1D570}" => "\u{0045}",
        "\u{1D5A4}" => "\u{0045}",
        "\u{1D5D8}" => "\u{0045}",
        "\u{1D60C}" => "\u{0045}",
        "\u{1D640}" => "\u{0045}",
        "\u{1D674}" => "\u{0045}",
        "\u{0395}" => "\u{0045}",
        "\u{1D6AC}" => "\u{0045}",
        "\u{1D6E6}" => "\u{0045}",
        "\u{1D720}" => "\u{0045}",
        "\u{1D75A}" => "\u{0045}",
        "\u{1D794}" => "\u{0045}",
        "\u{0415}" => "\u{0045}",
        "\u{2D39}" => "\u{0045}",
        "\u{13AC}" => "\u{0045}",
        "\u{A4F0}" => "\u{0045}",
        "\u{118A6}" => "\u{0045}",
        "\u{118AE}" => "\u{0045}",
        "\u{10286}" => "\u{0045}",
        "\u{011B}" => "\u{0115}",
        "\u{011A}" => "\u{0114}",
        "\u{0247}" => "\u{0065}\u{0338}",
        "\u{0246}" => "\u{0045}\u{0338}",
        "\u{04BF}" => "\u{0065}\u{0328}",
        "\u{AB7C}" => "\u{1D07}",
        "\u{0259}" => "\u{01DD}",
        "\u{04D9}" => "\u{01DD}",
        "\u{2203}" => "\u{018E}",
        "\u{2D3A}" => "\u{018E}",
        "\u{A4F1}" => "\u{018E}",
        "\u{025A}" => "\u{01DD}\u{02DE}",
        "\u{1D14}" => "\u{01DD}\u{006F}",
        "\u{AB41}" => "\u{01DD}\u{006F}\u{0338}",
        "\u{AB42}" => "\u{01DD}\u{006F}\u{0335}",
        "\u{04D8}" => "\u{018F}",
        "\u{1D221}" => "\u{0190}",
        "\u{2107}" => "\u{0190}",
        "\u{0510}" => "\u{0190}",
        "\u{13CB}" => "\u{0190}",
        "\u{16F2D}" => "\u{0190}",
        "\u{10401}" => "\u{0190}",
        "\u{1D9F}" => "\u{1D4B}",
        "\u{1D08}" => "\u{025C}",
        "\u{0437}" => "\u{025C}",
        "\u{0499}" => "\u{025C}\u{0326}",
        "\u{10442}" => "\u{025E}",
        "\u{A79D}" => "\u{029A}",
        "\u{1042A}" => "\u{029A}",
        "\u{1D41F}" => "\u{0066}",
        "\u{1D453}" => "\u{0066}",
        "\u{1D487}" => "\u{0066}",
        "\u{1D4BB}" => "\u{0066}",
        "\u{1D4EF}" => "\u{0066}",
        "\u{1D523}" => "\u{0066}",
        "\u{1D557}" => "\u{0066}",
        "\u{1D58B}" => "\u{0066}",
        "\u{1D5BF}" => "\u{0066}",
        "\u{1D5F3}" => "\u{0066}",
        "\u{1D627}" => "\u{0066}",
        "\u{1D65B}" => "\u{0066}",
        "\u{1D68F}" => "\u{0066}",
        "\u{AB35}" => "\u{0066}",
        "\u{A799}" => "\u{0066}",
        "\u{017F}" => "\u{0066}",
        "\u{1E9D}" => "\u{0066}",
        "\u{0584}" => "\u{0066}",
        "\u{1D213}" => "\u{0046}",
        "\u{2131}" => "\u{0046}",
        "\u{1D405}" => "\u{0046}",
        "\u{1D439}" => "\u{0046}",
        "\u{1D46D}" => "\u{0046}",
        "\u{1D4D5}" => "\u{0046}",
        "\u{1D509}" => "\u{0046}",
        "\u{1D53D}" => "\u{0046}",
        "\u{1D571}" => "\u{0046}",
        "\u{1D5A5}" => "\u{0046}",
        "\u{1D5D9}" => "\u{0046}",
        "\u{1D60D}" => "\u{0046}",
        "\u{1D641}" => "\u{0046}",
        "\u{1D675}" => "\u{0046}",
        "\u{A798}" => "\u{0046}",
        "\u{03DC}" => "\u{0046}",
        "\u{1D7CA}" => "\u{0046}",
        "\u{15B4}" => "\u{0046}",
        "\u{A4DD}" => "\u{0046}",
        "\u{118C2}" => "\u{0046}",
        "\u{118A2}" => "\u{0046}",
        "\u{10287}" => "\u{0046}",
        "\u{102A5}" => "\u{0046}",
        "\u{10525}" => "\u{0046}",
        "\u{0192}" => "\u{0066}\u{0326}",
        "\u{0191}" => "\u{0046}\u{0326}",
        "\u{1D6E}" => "\u{0066}\u{0334}",
        "\u{213B}" => "\u{0046}\u{0041}\u{0058}",
        "\u{FB00}" => "\u{0066}\u{0066}",
        "\u{FB03}" => "\u{0066}\u{0066}\u{0069}",
        "\u{FB04}" => "\u{0066}\u{0066}\u{006C}",
        "\u{FB01}" => "\u{0066}\u{0069}",
        "\u{FB02}" => "\u{0066}\u{006C}",
        "\u{02A9}" => "\u{0066}\u{014B}",
        "\u{15B5}" => "\u{2132}",
        "\u{A4DE}" => "\u{2132}",
        "\u{1D230}" => "\u{A7FB}",
        "\u{15B7}" => "\u{A7FB}",
        "\u{FF47}" => "\u{0067}",
        "\u{210A}" => "\u{0067}",
        "\u{1D420}" => "\u{0067}",
        "\u{1D454}" => "\u{0067}",
        "\u{1D488}" => "\u{0067}",
        "\u{1D4F0}" => "\u{0067}",
        "\u{1D524}" => "\u{0067}",
        "\u{1D558}" => "\u{0067}",
        "\u{1D58C}" => "\u{0067}",
        "\u{1D5C0}" => "\u{0067}",
        "\u{1D5F4}" => "\u{0067}",
        "\u{1D628}" => "\u{0067}",
        "\u{1D65C}" => "\u{0067}",
        "\u{1D690}" => "\u{0067}",
        "\u{0261}" => "\u{0067}",
        "\u{1D83}" => "\u{0067}",
        "\u{018D}" => "\u{0067}",
        "\u{0581}" => "\u{0067}",
        "\u{1D406}" => "\u{0047}",
        "\u{1D43A}" => "\u{0047}",
        "\u{1D46E}" => "\u{0047}",
        "\u{1D4A2}" => "\u{0047}",
        "\u{1D4D6}" => "\u{0047}",
        "\u{1D50A}" => "\u{0047}",
        "\u{1D53E}" => "\u{0047}",
        "\u{1D572}" => "\u{0047}",
        "\u{1D5A6}" => "\u{0047}",
        "\u{1D5DA}" => "\u{0047}",
        "\u{1D60E}" => "\u{0047}",
        "\u{1D642}" => "\u{0047}",
        "\u{1D676}" => "\u{0047}",
        "\u{050C}" => "\u{0047}",
        "\u{13C0}" => "\u{0047}",
        "\u{13F3}" => "\u{0047}",
        "\u{A4D6}" => "\u{0047}",
        "\u{1DA2}" => "\u{1D4D}",
        "\u{0260}" => "\u{0067}\u{0314}",
        "\u{01E7}" => "\u{011F}",
        "\u{01E6}" => "\u{011E}",
        "\u{01F5}" => "\u{0123}",
        "\u{01E5}" => "\u{0067}\u{0335}",
        "\u{01E4}" => "\u{0047}\u{0335}",
        "\u{0193}" => "\u{0047}\u{0027}",
        "\u{050D}" => "\u{0262}",
        "\u{AB90}" => "\u{0262}",
        "\u{13FB}" => "\u{0262}",
        "\u{FF48}" => "\u{0068}",
        "\u{210E}" => "\u{0068}",
        "\u{1D421}" => "\u{0068}",
        "\u{1D489}" => "\u{0068}",
        "\u{1D4BD}" => "\u{0068}",
        "\u{1D4F1}" => "\u{0068}",
        "\u{1D525}" => "\u{0068}",
        "\u{1D559}" => "\u{0068}",
        "\u{1D58D}" => "\u{0068}",
        "\u{1D5C1}" => "\u{0068}",
        "\u{1D5F5}" => "\u{0068}",
        "\u{1D629}" => "\u{0068}",
        "\u{1D65D}" => "\u{0068}",
        "\u{1D691}" => "\u{0068}",
        "\u{04BB}" => "\u{0068}",
        "\u{0570}" => "\u{0068}",
        "\u{13C2}" => "\u{0068}",
        "\u{FF28}" => "\u{0048}",
        "\u{210B}" => "\u{0048}",
        "\u{210C}" => "\u{0048}",
        "\u{210D}" => "\u{0048}",
        "\u{1D407}" => "\u{0048}",
        "\u{1D43B}" => "\u{0048}",
        "\u{1D46F}" => "\u{0048}",
        "\u{1D4D7}" => "\u{0048}",
        "\u{1D573}" => "\u{0048}",
        "\u{1D5A7}" => "\u{0048}",
        "\u{1D5DB}" => "\u{0048}",
        "\u{1D60F}" => "\u{0048}",
        "\u{1D643}" => "\u{0048}",
        "\u{1D677}" => "\u{0048}",
        "\u{0397}" => "\u{0048}",
        "\u{1D6AE}" => "\u{0048}",
        "\u{1D6E8}" => "\u{0048}",
        "\u{1D722}" => "\u{0048}",
        "\u{1D75C}" => "\u{0048}",
        "\u{1D796}" => "\u{0048}",
        "\u{2C8E}" => "\u{0048}",
        "\u{041D}" => "\u{0048}",
        "\u{13BB}" => "\u{0048}",
        "\u{157C}" => "\u{0048}",
        "\u{A4E7}" => "\u{0048}",
        "\u{102CF}" => "\u{0048}",
        "\u{1D78}" => "\u{1D34}",
        "\u{0266}" => "\u{0068}\u{0314}",
        "\u{A695}" => "\u{0068}\u{0314}",
        "\u{13F2}" => "\u{0068}\u{0314}",
        "\u{2C67}" => "\u{0048}\u{0329}",
        "\u{04A2}" => "\u{0048}\u{0329}",
        "\u{0127}" => "\u{0068}\u{0335}",
        "\u{210F}" => "\u{0068}\u{0335}",
        "\u{045B}" => "\u{0068}\u{0335}",
        "\u{0126}" => "\u{0048}\u{0335}",
        "\u{04C9}" => "\u{0048}\u{0326}",
        "\u{04C7}" => "\u{0048}\u{0326}",
        "\u{043D}" => "\u{029C}",
        "\u{AB8B}" => "\u{029C}",
        "\u{04A3}" => "\u{029C}\u{0329}",
        "\u{04CA}" => "\u{029C}\u{0326}",
        "\u{04C8}" => "\u{029C}\u{0326}",
        "\u{050A}" => "\u{01F6}",
        "\u{AB80}" => "\u{2C76}",
        "\u{0370}" => "\u{2C75}",
        "\u{13A8}" => "\u{2C75}",
        "\u{13B0}" => "\u{2C75}",
        "\u{A6B1}" => "\u{2C75}",
        "\u{A795}" => "\u{A727}",
        "\u{02DB}" => "\u{0069}",
        "\u{2373}" => "\u{0069}",
        "\u{FF49}" => "\u{0069}",
        "\u{2170}" => "\u{0069}",
        "\u{2139}" => "\u{0069}",
        "\u{2148}" => "\u{0069}",
        "\u{1D422}" => "\u{0069}",
        "\u{1D456}" => "\u{0069}",
        "\u{1D48A}" => "\u{0069}",
        "\u{1D4BE}" => "\u{0069}",
        "\u{1D4F2}" => "\u{0069}",
        "\u{1D526}" => "\u{0069}",
        "\u{1D55A}" => "\u{0069}",
        "\u{1D58E}" => "\u{0069}",
        "\u{1D5C2}" => "\u{0069}",
        "\u{1D5F6}" => "\u{0069}",
        "\u{1D62A}" => "\u{0069}",
        "\u{1D65E}" => "\u{0069}",
        "\u{1D692}" => "\u{0069}",
        "\u{0131}" => "\u{0069}",
        "\u{1D6A4}" => "\u{0069}",
        "\u{026A}" => "\u{0069}",
        "\u{0269}" => "\u{0069}",
        "\u{03B9}" => "\u{0069}",
        "\u{1FBE}" => "\u{0069}",
        "\u{037A}" => "\u{0069}",
        "\u{1D6CA}" => "\u{0069}",
        "\u{1D704}" => "\u{0069}",
        "\u{1D73E}" => "\u{0069}",
        "\u{1D778}" => "\u{0069}",
        "\u{1D7B2}" => "\u{0069}",
        "\u{0456}" => "\u{0069}",
        "\u{A647}" => "\u{0069}",
        "\u{04CF}" => "\u{0069}",
        "\u{AB75}" => "\u{0069}",
        "\u{13A5}" => "\u{0069}",
        "\u{118C3}" => "\u{0069}",
        "\u{24DB}" => "\u{24BE}",
        "\u{2378}" => "\u{0069}\u{0332}",
        "\u{01D0}" => "\u{012D}",
        "\u{01CF}" => "\u{012C}",
        "\u{0268}" => "\u{0069}\u{0335}",
        "\u{1D7B}" => "\u{0069}\u{0335}",
        "\u{1D7C}" => "\u{0069}\u{0335}",
        "\u{2171}" => "\u{0069}\u{0069}",
        "\u{2172}" => "\u{0069}\u{0069}\u{0069}",
        "\u{0133}" => "\u{0069}\u{006A}",
        "\u{2173}" => "\u{0069}\u{0076}",
        "\u{2178}" => "\u{0069}\u{0078}",
        "\u{FF4A}" => "\u{006A}",
        "\u{2149}" => "\u{006A}",
        "\u{1D423}" => "\u{006A}",
        "\u{1D457}" => "\u{006A}",
        "\u{1D48B}" => "\u{006A}",
        "\u{1D4BF}" => "\u{006A}",
        "\u{1D4F3}" => "\u{006A}",
        "\u{1D527}" => "\u{006A}",
        "\u{1D55B}" => "\u{006A}",
        "\u{1D58F}" => "\u{006A}",
        "\u{1D5C3}" => "\u{006A}",
        "\u{1D5F7}" => "\u{006A}",
        "\u{1D62B}" => "\u{006A}",
        "\u{1D65F}" => "\u{006A}",
        "\u{1D693}" => "\u{006A}",
        "\u{03F3}" => "\u{006A}",
        "\u{0458}" => "\u{006A}",
        "\u{FF2A}" => "\u{004A}",
        "\u{1D409}" => "\u{004A}",
        "\u{1D43D}" => "\u{004A}",
        "\u{1D471}" => "\u{004A}",
        "\u{1D4A5}" => "\u{004A}",
        "\u{1D4D9}" => "\u{004A}",
        "\u{1D50D}" => "\u{004A}",
        "\u{1D541}" => "\u{004A}",
        "\u{1D575}" => "\u{004A}",
        "\u{1D5A9}" => "\u{004A}",
        "\u{1D5DD}" => "\u{004A}",
        "\u{1D611}" => "\u{004A}",
        "\u{1D645}" => "\u{004A}",
        "\u{1D679}" => "\u{004A}",
        "\u{A7B2}" => "\u{004A}",
        "\u{037F}" => "\u{004A}",
        "\u{0408}" => "\u{004A}",
        "\u{13AB}" => "\u{004A}",
        "\u{148D}" => "\u{004A}",
        "\u{A4D9}" => "\u{004A}",
        "\u{0249}" => "\u{006A}\u{0335}",
        "\u{0248}" => "\u{004A}\u{0335}",
        "\u{1499}" => "\u{004A}\u{00B7}",
        "\u{1D6A5}" => "\u{0237}",
        "\u{0575}" => "\u{0237}",
        "\u{AB7B}" => "\u{1D0A}",
        "\u{1D424}" => "\u{006B}",
        "\u{1D458}" => "\u{006B}",
        "\u{1D48C}" => "\u{006B}",
        "\u{1D4C0}" => "\u{006B}",
        "\u{1D4F4}" => "\u{006B}",
        "\u{1D528}" => "\u{006B}",
        "\u{1D55C}" => "\u{006B}",
        "\u{1D590}" => "\u{006B}",
        "\u{1D5C4}" => "\u{006B}",
        "\u{1D5F8}" => "\u{006B}",
        "\u{1D62C}" => "\u{006B}",
        "\u{1D660}" => "\u{006B}",
        "\u{1D694}" => "\u{006B}",
        "\u{212A}" => "\u{004B}",
        "\u{FF2B}" => "\u{004B}",
        "\u{1D40A}" => "\u{004B}",
        "\u{1D43E}" => "\u{004B}",
        "\u{1D472}" => "\u{004B}",
        "\u{1D4A6}" => "\u{004B}",
        "\u{1D4DA}" => "\u{004B}",
        "\u{1D50E}" => "\u{004B}",
        "\u{1D542}" => "\u{004B}",
        "\u{1D576}" => "\u{004B}",
        "\u{1D5AA}" => "\u{004B}",
        "\u{1D5DE}" => "\u{004B}",
        "\u{1D612}" => "\u{004B}",
        "\u{1D646}" => "\u{004B}",
        "\u{1D67A}" => "\u{004B}",
        "\u{039A}" => "\u{004B}",
        "\u{1D6B1}" => "\u{004B}",
        "\u{1D6EB}" => "\u{004B}",
        "\u{1D725}" => "\u{004B}",
        "\u{1D75F}" => "\u{004B}",
        "\u{1D799}" => "\u{004B}",
        "\u{2C94}" => "\u{004B}",
        "\u{041A}" => "\u{004B}",
        "\u{13E6}" => "\u{004B}",
        "\u{16D5}" => "\u{004B}",
        "\u{A4D7}" => "\u{004B}",
        "\u{10518}" => "\u{004B}",
        "\u{0199}" => "\u{006B}\u{0314}",
        "\u{2C69}" => "\u{004B}\u{0329}",
        "\u{049A}" => "\u{004B}\u{0329}",
        "\u{20AD}" => "\u{004B}\u{0335}",
        "\u{A740}" => "\u{004B}\u{0335}",
        "\u{049E}" => "\u{004B}\u{0335}",
        "\u{0198}" => "\u{004B}\u{0027}",
        "\u{05C0}" => "\u{006C}",
        "\u{007C}" => "\u{006C}",
        "\u{2223}" => "\u{006C}",
        "\u{23FD}" => "\u{006C}",
        "\u{FFE8}" => "\u{006C}",
        "\u{0031}" => "\u{006C}",
        "\u{0661}" => "\u{006C}",
        "\u{06F1}" => "\u{006C}",
        "\u{10320}" => "\u{006C}",
        "\u{1E8C7}" => "\u{006C}",
        "\u{1D7CF}" => "\u{006C}",
        "\u{1D7D9}" => "\u{006C}",
        "\u{1D7E3}" => "\u{006C}",
        "\u{1D7ED}" => "\u{006C}",
        "\u{1D7F7}" => "\u{006C}",
        "\u{1FBF1}" => "\u{006C}",
        "\u{0049}" => "\u{006C}",
        "\u{FF29}" => "\u{006C}",
        "\u{2160}" => "\u{006C}",
        "\u{2110}" => "\u{006C}",
        "\u{2111}" => "\u{006C}",
        "\u{1D408}" => "\u{006C}",
        "\u{1D43C}" => "\u{006C}",
        "\u{1D470}" => "\u{006C}",
        "\u{1D4D8}" => "\u{006C}",
        "\u{1D540}" => "\u{006C}",
        "\u{1D574}" => "\u{006C}",
        "\u{1D5A8}" => "\u{006C}",
        "\u{1D5DC}" => "\u{006C}",
        "\u{1D610}" => "\u{006C}",
        "\u{1D644}" => "\u{006C}",
        "\u{1D678}" => "\u{006C}",
        "\u{0196}" => "\u{006C}",
        "\u{FF4C}" => "\u{006C}",
        "\u{217C}" => "\u{006C}",
        "\u{2113}" => "\u{006C}",
        "\u{1D425}" => "\u{006C}",
        "\u{1D459}" => "\u{006C}",
        "\u{1D48D}" => "\u{006C}",
        "\u{1D4C1}" => "\u{006C}",
        "\u{1D4F5}" => "\u{006C}",
        "\u{1D529}" => "\u{006C}",
        "\u{1D55D}" => "\u{006C}",
        "\u{1D591}" => "\u{006C}",
        "\u{1D5C5}" => "\u{006C}",
        "\u{1D5F9}" => "\u{006C}",
        "\u{1D62D}" => "\u{006C}",
        "\u{1D661}" => "\u{006C}",
        "\u{1D695}" => "\u{006C}",
        "\u{01C0}" => "\u{006C}",
        "\u{0399}" => "\u{006C}",
        "\u{1D6B0}" => "\u{006C}",
        "\u{1D6EA}" => "\u{006C}",
        "\u{1D724}" => "\u{006C}",
        "\u{1D75E}" => "\u{006C}",
        "\u{1D798}" => "\u{006C}",
        "\u{2C92}" => "\u{006C}",
        "\u{0406}" => "\u{006C}",
        "\u{04C0}" => "\u{006C}",
        "\u{05D5}" => "\u{006C}",
        "\u{05DF}" => "\u{006C}",
        "\u{0627}" => "\u{006C}",
        "\u{1EE00}" => "\u{006C}",
        "\u{1EE80}" => "\u{006C}",
        "\u{FE8E}" => "\u{006C}",
        "\u{FE8D}" => "\u{006C}",
        "\u{07CA}" => "\u{006C}",
        "\u{2D4F}" => "\u{006C}",
        "\u{16C1}" => "\u{006C}",
        "\u{A4F2}" => "\u{006C}",
        "\u{16F28}" => "\u{006C}",
        "\u{1028A}" => "\u{006C}",
        "\u{10309}" => "\u{006C}",
        "\u{1D22A}" => "\u{004C}",
        "\u{216C}" => "\u{004C}",
        "\u{2112}" => "\u{004C}",
        "\u{1D40B}" => "\u{004C}",
        "\u{1D43F}" => "\u{004C}",
        "\u{1D473}" => "\u{004C}",
        "\u{1D4DB}" => "\u{004C}",
        "\u{1D50F}" => "\u{004C}",
        "\u{1D543}" => "\u{004C}",
        "\u{1D577}" => "\u{004C}",
        "\u{1D5AB}" => "\u{004C}",
        "\u{1D5DF}" => "\u{004C}",
        "\u{1D613}" => "\u{004C}",
        "\u{1D647}" => "\u{004C}",
        "\u{1D67B}" => "\u{004C}",
        "\u{2CD0}" => "\u{004C}",
        "\u{13DE}" => "\u{004C}",
        "\u{14AA}" => "\u{004C}",
        "\u{A4E1}" => "\u{004C}",
        "\u{16F16}" => "\u{004C}",
        "\u{118A3}" => "\u{004C}",
        "\u{118B2}" => "\u{004C}",
        "\u{1041B}" => "\u{004C}",
        "\u{10526}" => "\u{004C}",
        "\u{FD3C}" => "\u{006C}\u{030B}",
        "\u{FD3D}" => "\u{006C}\u{030B}",
        "\u{0142}" => "\u{006C}\u{0338}",
        "\u{0141}" => "\u{004C}\u{0338}",
        "\u{026D}" => "\u{006C}\u{0328}",
        "\u{0197}" => "\u{006C}\u{0335}",
        "\u{019A}" => "\u{006C}\u{0335}",
        "\u{026B}" => "\u{006C}\u{0334}",
        "\u{0625}" => "\u{006C}\u{0655}",
        "\u{FE88}" => "\u{006C}\u{0655}",
        "\u{FE87}" => "\u{006C}\u{0655}",
        "\u{0673}" => "\u{006C}\u{0655}",
        "\u{0140}" => "\u{006C}\u{00B7}",
        "\u{013F}" => "\u{006C}\u{00B7}",
        "\u{14B7}" => "\u{006C}\u{00B7}",
        "\u{1F102}" => "\u{006C}\u{002C}",
        "\u{2488}" => "\u{006C}\u{002E}",
        "\u{05F1}" => "\u{006C}\u{0027}",
        "\u{2493}" => "\u{006C}\u{0032}\u{002E}",
        "\u{33EB}" => "\u{006C}\u{0032}\u{65E5}",
        "\u{32CB}" => "\u{006C}\u{0032}\u{6708}",
        "\u{3364}" => "\u{006C}\u{0032}\u{70B9}",
        "\u{2494}" => "\u{006C}\u{0033}\u{002E}",
        "\u{33EC}" => "\u{006C}\u{0033}\u{65E5}",
        "\u{3365}" => "\u{006C}\u{0033}\u{70B9}",
        "\u{2495}" => "\u{006C}\u{0034}\u{002E}",
        "\u{33ED}" => "\u{006C}\u{0034}\u{65E5}",
        "\u{3366}" => "\u{006C}\u{0034}\u{70B9}",
        "\u{2496}" => "\u{006C}\u{0035}\u{002E}",
        "\u{33EE}" => "\u{006C}\u{0035}\u{65E5}",
        "\u{3367}" => "\u{006C}\u{0035}\u{70B9}",
        "\u{2497}" => "\u{006C}\u{0036}\u{002E}",
        "\u{33EF}" => "\u{006C}\u{0036}\u{65E5}",
        "\u{3368}" => "\u{006C}\u{0036}\u{70B9}",
        "\u{2498}" => "\u{006C}\u{0037}\u{002E}",
        "\u{33F0}" => "\u{006C}\u{0037}\u{65E5}",
        "\u{3369}" => "\u{006C}\u{0037}\u{70B9}",
        "\u{2499}" => "\u{006C}\u{0038}\u{002E}",
        "\u{33F1}" => "\u{006C}\u{0038}\u{65E5}",
        "\u{336A}" => "\u{006C}\u{0038}\u{70B9}",
        "\u{249A}" => "\u{006C}\u{0039}\u{002E}",
        "\u{33F2}" => "\u{006C}\u{0039}\u{65E5}",
        "\u{336B}" => "\u{006C}\u{0039}\u{70B9}",
        "\u{01C9}" => "\u{006C}\u{006A}",
        "\u{0132}" => "\u{006C}\u{004A}",
        "\u{01C8}" => "\u{004C}\u{006A}",
        "\u{01C7}" => "\u{004C}\u{004A}",
        "\u{2016}" => "\u{006C}\u{006C}",
        "\u{2225}" => "\u{006C}\u{006C}",
        "\u{2161}" => "\u{006C}\u{006C}",
        "\u{01C1}" => "\u{006C}\u{006C}",
        "\u{05F0}" => "\u{006C}\u{006C}",
        "\u{10199}" => "\u{006C}\u{0335}\u{006C}\u{0335}",
        "\u{2492}" => "\u{006C}\u{006C}\u{002E}",
        "\u{2162}" => "\u{006C}\u{006C}\u{006C}",
        "\u{10198}" => "\u{006C}\u{0335}\u{006C}\u{0335}\u{0053}\u{0335}",
        "\u{33EA}" => "\u{006C}\u{006C}\u{65E5}",
        "\u{32CA}" => "\u{006C}\u{006C}\u{6708}",
        "\u{3363}" => "\u{006C}\u{006C}\u{70B9}",
        "\u{042E}" => "\u{006C}\u{004F}",
        "\u{2491}" => "\u{006C}\u{004F}\u{002E}",
        "\u{33E9}" => "\u{006C}\u{004F}\u{65E5}",
        "\u{32C9}" => "\u{006C}\u{004F}\u{6708}",
        "\u{3362}" => "\u{006C}\u{004F}\u{70B9}",
        "\u{02AA}" => "\u{006C}\u{0073}",
        "\u{20B6}" => "\u{006C}\u{0074}",
        "\u{2163}" => "\u{006C}\u{0056}",
        "\u{2168}" => "\u{006C}\u{0058}",
        "\u{026E}" => "\u{006C}\u{021D}",
        "\u{02AB}" => "\u{006C}\u{007A}",
        "\u{0623}" => "\u{006C}\u{0674}",
        "\u{FE84}" => "\u{006C}\u{0674}",
        "\u{FE83}" => "\u{006C}\u{0674}",
        "\u{0672}" => "\u{006C}\u{0674}",
        "\u{0675}" => "\u{006C}\u{0674}",
        "\u{FDF3}" => "\u{006C}\u{0643}\u{0628}\u{0631}",
        "\u{FDF2}" => "\u{006C}\u{0644}\u{0644}\u{0651}\u{0670}\u{006F}",
        "\u{33E0}" => "\u{006C}\u{65E5}",
        "\u{32C0}" => "\u{006C}\u{6708}",
        "\u{3359}" => "\u{006C}\u{70B9}",
        "\u{2CD1}" => "\u{029F}",
        "\u{ABAE}" => "\u{029F}",
        "\u{10443}" => "\u{029F}",
        "\u{FF2D}" => "\u{004D}",
        "\u{216F}" => "\u{004D}",
        "\u{2133}" => "\u{004D}",
        "\u{1D40C}" => "\u{004D}",
        "\u{1D440}" => "\u{004D}",
        "\u{1D474}" => "\u{004D}",
        "\u{1D4DC}" => "\u{004D}",
        "\u{1D510}" => "\u{004D}",
        "\u{1D544}" => "\u{004D}",
        "\u{1D578}" => "\u{004D}",
        "\u{1D5AC}" => "\u{004D}",
        "\u{1D5E0}" => "\u{004D}",
        "\u{1D614}" => "\u{004D}",
        "\u{1D648}" => "\u{004D}",
        "\u{1D67C}" => "\u{004D}",
        "\u{039C}" => "\u{004D}",
        "\u{1D6B3}" => "\u{004D}",
        "\u{1D6ED}" => "\u{004D}",
        "\u{1D727}" => "\u{004D}",
        "\u{1D761}" => "\u{004D}",
        "\u{1D79B}" => "\u{004D}",
        "\u{03FA}" => "\u{004D}",
        "\u{2C98}" => "\u{004D}",
        "\u{041C}" => "\u{004D}",
        "\u{13B7}" => "\u{004D}",
        "\u{15F0}" => "\u{004D}",
        "\u{16D6}" => "\u{004D}",
        "\u{A4DF}" => "\u{004D}",
        "\u{102B0}" => "\u{004D}",
        "\u{10311}" => "\u{004D}",
        "\u{04CD}" => "\u{004D}\u{0326}",
        "\u{1F76B}" => "\u{004D}\u{0042}",
        "\u{2DE8}" => "\u{1DDF}",
        "\u{1D427}" => "\u{006E}",
        "\u{1D45B}" => "\u{006E}",
        "\u{1D48F}" => "\u{006E}",
        "\u{1D4C3}" => "\u{006E}",
        "\u{1D4F7}" => "\u{006E}",
        "\u{1D52B}" => "\u{006E}",
        "\u{1D55F}" => "\u{006E}",
        "\u{1D593}" => "\u{006E}",
        "\u{1D5C7}" => "\u{006E}",
        "\u{1D5FB}" => "\u{006E}",
        "\u{1D62F}" => "\u{006E}",
        "\u{1D663}" => "\u{006E}",
        "\u{1D697}" => "\u{006E}",
        "\u{0578}" => "\u{006E}",
        "\u{057C}" => "\u{006E}",
        "\u{FF2E}" => "\u{004E}",
        "\u{2115}" => "\u{004E}",
        "\u{1D40D}" => "\u{004E}",
        "\u{1D441}" => "\u{004E}",
        "\u{1D475}" => "\u{004E}",
        "\u{1D4A9}" => "\u{004E}",
        "\u{1D4DD}" => "\u{004E}",
        "\u{1D511}" => "\u{004E}",
        "\u{1D579}" => "\u{004E}",
        "\u{1D5AD}" => "\u{004E}",
        "\u{1D5E1}" => "\u{004E}",
        "\u{1D615}" => "\u{004E}",
        "\u{1D649}" => "\u{004E}",
        "\u{1D67D}" => "\u{004E}",
        "\u{039D}" => "\u{004E}",
        "\u{1D6B4}" => "\u{004E}",
        "\u{1D6EE}" => "\u{004E}",
        "\u{1D728}" => "\u{004E}",
        "\u{1D762}" => "\u{004E}",
        "\u{1D79C}" => "\u{004E}",
        "\u{2C9A}" => "\u{004E}",
        "\u{A4E0}" => "\u{004E}",
        "\u{10513}" => "\u{004E}",
        "\u{1018E}" => "\u{004E}\u{030A}",
        "\u{0273}" => "\u{006E}\u{0328}",
        "\u{019E}" => "\u{006E}\u{0329}",
        "\u{03B7}" => "\u{006E}\u{0329}",
        "\u{1D6C8}" => "\u{006E}\u{0329}",
        "\u{1D702}" => "\u{006E}\u{0329}",
        "\u{1D73C}" => "\u{006E}\u{0329}",
        "\u{1D776}" => "\u{006E}\u{0329}",
        "\u{1D7B0}" => "\u{006E}\u{0329}",
        "\u{019D}" => "\u{004E}\u{0326}",
        "\u{1D70}" => "\u{006E}\u{0334}",
        "\u{01CC}" => "\u{006E}\u{006A}",
        "\u{01CB}" => "\u{004E}\u{006A}",
        "\u{01CA}" => "\u{004E}\u{004A}",
        "\u{2116}" => "\u{004E}\u{006F}",
        "\u{0377}" => "\u{1D0E}",
        "\u{0438}" => "\u{1D0E}",
        "\u{1044D}" => "\u{1D0E}",
        "\u{0146}" => "\u{0272}",
        "\u{0C02}" => "\u{006F}",
        "\u{0C82}" => "\u{006F}",
        "\u{0D02}" => "\u{006F}",
        "\u{0D82}" => "\u{006F}",
        "\u{0966}" => "\u{006F}",
        "\u{0A66}" => "\u{006F}",
        "\u{0AE6}" => "\u{006F}",
        "\u{0BE6}" => "\u{006F}",
        "\u{0C66}" => "\u{006F}",
        "\u{0CE6}" => "\u{006F}",
        "\u{0D66}" => "\u{006F}",
        "\u{0E50}" => "\u{006F}",
        "\u{0ED0}" => "\u{006F}",
        "\u{1040}" => "\u{006F}",
        "\u{0665}" => "\u{006F}",
        "\u{06F5}" => "\u{006F}",
        "\u{FF4F}" => "\u{006F}",
        "\u{2134}" => "\u{006F}",
        "\u{1D428}" => "\u{006F}",
        "\u{1D45C}" => "\u{006F}",
        "\u{1D490}" => "\u{006F}",
        "\u{1D4F8}" => "\u{006F}",
        "\u{1D52C}" => "\u{006F}",
        "\u{1D560}" => "\u{006F}",
        "\u{1D594}" => "\u{006F}",
        "\u{1D5C8}" => "\u{006F}",
        "\u{1D5FC}" => "\u{006F}",
        "\u{1D630}" => "\u{006F}",
        "\u{1D664}" => "\u{006F}",
        "\u{1D698}" => "\u{006F}",
        "\u{1D0F}" => "\u{006F}",
        "\u{1D11}" => "\u{006F}",
        "\u{AB3D}" => "\u{006F}",
        "\u{03BF}" => "\u{006F}",
        "\u{1D6D0}" => "\u{006F}",
        "\u{1D70A}" => "\u{006F}",
        "\u{1D744}" => "\u{006F}",
        "\u{1D77E}" => "\u{006F}",
        "\u{1D7B8}" => "\u{006F}",
        "\u{03C3}" => "\u{006F}",
        "\u{1D6D4}" => "\u{006F}",
        "\u{1D70E}" => "\u{006F}",
        "\u{1D748}" => "\u{006F}",
        "\u{1D782}" => "\u{006F}",
        "\u{1D7BC}" => "\u{006F}",
        "\u{2C9F}" => "\u{006F}",
        "\u{043E}" => "\u{006F}",
        "\u{10FF}" => "\u{006F}",
        "\u{0585}" => "\u{006F}",
        "\u{05E1}" => "\u{006F}",
        "\u{0647}" => "\u{006F}",
        "\u{1EE24}" => "\u{006F}",
        "\u{1EE64}" => "\u{006F}",
        "\u{1EE84}" => "\u{006F}",
        "\u{FEEB}" => "\u{006F}",
        "\u{FEEC}" => "\u{006F}",
        "\u{FEEA}" => "\u{006F}",
        "\u{FEE9}" => "\u{006F}",
        "\u{06BE}" => "\u{006F}",
        "\u{FBAC}" => "\u{006F}",
        "\u{FBAD}" => "\u{006F}",
        "\u{FBAB}" => "\u{006F}",
        "\u{FBAA}" => "\u{006F}",
        "\u{06C1}" => "\u{006F}",
        "\u{FBA8}" => "\u{006F}",
        "\u{FBA9}" => "\u{006F}",
        "\u{FBA7}" => "\u{006F}",
        "\u{FBA6}" => "\u{006F}",
        "\u{06D5}" => "\u{006F}",
        "\u{0D20}" => "\u{006F}",
        "\u{101D}" => "\u{006F}",
        "\u{104EA}" => "\u{006F}",
        "\u{118C8}" => "\u{006F}",
        "\u{118D7}" => "\u{006F}",
        "\u{1042C}" => "\u{006F}",
        "\u{0030}" => "\u{004F}",
        "\u{07C0}" => "\u{004F}",
        "\u{09E6}" => "\u{004F}",
        "\u{0B66}" => "\u{004F}",
        "\u{3007}" => "\u{004F}",
        "\u{114D0}" => "\u{004F}",
        "\u{118E0}" => "\u{004F}",
        "\u{1D7CE}" => "\u{004F}",
        "\u{1D7D8}" => "\u{004F}",
        "\u{1D7E2}" => "\u{004F}",
        "\u{1D7EC}" => "\u{004F}",
        "\u{1D7F6}" => "\u{004F}",
        "\u{1FBF0}" => "\u{004F}",
        "\u{FF2F}" => "\u{004F}",
        "\u{1D40E}" => "\u{004F}",
        "\u{1D442}" => "\u{004F}",
        "\u{1D476}" => "\u{004F}",
        "\u{1D4AA}" => "\u{004F}",
        "\u{1D4DE}" => "\u{004F}",
        "\u{1D512}" => "\u{004F}",
        "\u{1D546}" => "\u{004F}",
        "\u{1D57A}" => "\u{004F}",
        "\u{1D5AE}" => "\u{004F}",
        "\u{1D5E2}" => "\u{004F}",
        "\u{1D616}" => "\u{004F}",
        "\u{1D64A}" => "\u{004F}",
        "\u{1D67E}" => "\u{004F}",
        "\u{039F}" => "\u{004F}",
        "\u{1D6B6}" => "\u{004F}",
        "\u{1D6F0}" => "\u{004F}",
        "\u{1D72A}" => "\u{004F}",
        "\u{1D764}" => "\u{004F}",
        "\u{1D79E}" => "\u{004F}",
        "\u{2C9E}" => "\u{004F}",
        "\u{041E}" => "\u{004F}",
        "\u{0555}" => "\u{004F}",
        "\u{2D54}" => "\u{004F}",
        "\u{12D0}" => "\u{004F}",
        "\u{0B20}" => "\u{004F}",
        "\u{104C2}" => "\u{004F}",
        "\u{A4F3}" => "\u{004F}",
        "\u{118B5}" => "\u{004F}",
        "\u{10292}" => "\u{004F}",
        "\u{102AB}" => "\u{004F}",
        "\u{10404}" => "\u{004F}",
        "\u{10516}" => "\u{004F}",
        "\u{2070}" => "\u{00BA}",
        "\u{1D52}" => "\u{00BA}",
        "\u{01D2}" => "\u{014F}",
        "\u{01D1}" => "\u{014E}",
        "\u{06FF}" => "\u{006F}\u{0302}",
        "\u{0150}" => "\u{00D6}",
        "\u{00F8}" => "\u{006F}\u{0338}",
        "\u{AB3E}" => "\u{006F}\u{0338}",
        "\u{00D8}" => "\u{004F}\u{0338}",
        "\u{2D41}" => "\u{004F}\u{0338}",
        "\u{01FE}" => "\u{004F}\u{0338}\u{0301}",
        "\u{0275}" => "\u{006F}\u{0335}",
        "\u{A74B}" => "\u{006F}\u{0335}",
        "\u{04E9}" => "\u{006F}\u{0335}",
        "\u{0473}" => "\u{006F}\u{0335}",
        "\u{AB8E}" => "\u{006F}\u{0335}",
        "\u{ABBB}" => "\u{006F}\u{0335}",
        "\u{2296}" => "\u{004F}\u{0335}",
        "\u{229D}" => "\u{004F}\u{0335}",
        "\u{236C}" => "\u{004F}\u{0335}",
        "\u{1D21A}" => "\u{004F}\u{0335}",
        "\u{1F714}" => "\u{004F}\u{0335}",
        "\u{019F}" => "\u{004F}\u{0335}",
        "\u{A74A}" => "\u{004F}\u{0335}",
        "\u{03B8}" => "\u{004F}\u{0335}",
        "\u{03D1}" => "\u{004F}\u{0335}",
        "\u{1D6C9}" => "\u{004F}\u{0335}",
        "\u{1D6DD}" => "\u{004F}\u{0335}",
        "\u{1D703}" => "\u{004F}\u{0335}",
        "\u{1D717}" => "\u{004F}\u{0335}",
        "\u{1D73D}" => "\u{004F}\u{0335}",
        "\u{1D751}" => "\u{004F}\u{0335}",
        "\u{1D777}" => "\u{004F}\u{0335}",
        "\u{1D78B}" => "\u{004F}\u{0335}",
        "\u{1D7B1}" => "\u{004F}\u{0335}",
        "\u{1D7C5}" => "\u{004F}\u{0335}",
        "\u{0398}" => "\u{004F}\u{0335}",
        "\u{03F4}" => "\u{004F}\u{0335}",
        "\u{1D6AF}" => "\u{004F}\u{0335}",
        "\u{1D6B9}" => "\u{004F}\u{0335}",
        "\u{1D6E9}" => "\u{004F}\u{0335}",
        "\u{1D6F3}" => "\u{004F}\u{0335}",
        "\u{1D723}" => "\u{004F}\u{0335}",
        "\u{1D72D}" => "\u{004F}\u{0335}",
        "\u{1D75D}" => "\u{004F}\u{0335}",
        "\u{1D767}" => "\u{004F}\u{0335}",
        "\u{1D797}" => "\u{004F}\u{0335}",
        "\u{1D7A1}" => "\u{004F}\u{0335}",
        "\u{04E8}" => "\u{004F}\u{0335}",
        "\u{0472}" => "\u{004F}\u{0335}",
        "\u{2D31}" => "\u{004F}\u{0335}",
        "\u{13BE}" => "\u{004F}\u{0335}",
        "\u{13EB}" => "\u{004F}\u{0335}",
        "\u{AB74}" => "\u{006F}\u{031B}",
        "\u{FCD9}" => "\u{006F}\u{0670}",
        "\u{1F101}" => "\u{004F}\u{002C}",
        "\u{1F100}" => "\u{004F}\u{002E}",
        "\u{01A1}" => "\u{006F}\u{0027}",
        "\u{01A0}" => "\u{004F}\u{0027}",
        "\u{13A4}" => "\u{004F}\u{0027}",
        "\u{0025}" => "\u{00BA}\u{002F}\u{2080}",
        "\u{066A}" => "\u{00BA}\u{002F}\u{2080}",
        "\u{2052}" => "\u{00BA}\u{002F}\u{2080}",
        "\u{2030}" => "\u{00BA}\u{002F}\u{2080}\u{2080}",
        "\u{0609}" => "\u{00BA}\u{002F}\u{2080}\u{2080}",
        "\u{2031}" => "\u{00BA}\u{002F}\u{2080}\u{2080}\u{2080}",
        "\u{060A}" => "\u{00BA}\u{002F}\u{2080}\u{2080}\u{2080}",
        "\u{0153}" => "\u{006F}\u{0065}",
        "\u{0152}" => "\u{004F}\u{0045}",
        "\u{0276}" => "\u{006F}\u{1D07}",
        "\u{221E}" => "\u{006F}\u{006F}",
        "\u{A74F}" => "\u{006F}\u{006F}",
        "\u{A699}" => "\u{006F}\u{006F}",
        "\u{A74E}" => "\u{004F}\u{004F}",
        "\u{A698}" => "\u{004F}\u{004F}",
        "\u{FCD7}" => "\u{006F}\u{062C}",
        "\u{FC51}" => "\u{006F}\u{062C}",
        "\u{FCD8}" => "\u{006F}\u{0645}",
        "\u{FC52}" => "\u{006F}\u{0645}",
        "\u{FD93}" => "\u{006F}\u{0645}\u{062C}",
        "\u{FD94}" => "\u{006F}\u{0645}\u{0645}",
        "\u{FC53}" => "\u{006F}\u{0649}",
        "\u{FC54}" => "\u{006F}\u{0649}",
        "\u{0D5F}" => "\u{006F}\u{0D30}\u{006F}",
        "\u{1010}" => "\u{006F}\u{102C}",
        "\u{3358}" => "\u{004F}\u{70B9}",
        "\u{2184}" => "\u{0254}",
        "\u{1D10}" => "\u{0254}",
        "\u{037B}" => "\u{0254}",
        "\u{1044B}" => "\u{0254}",
        "\u{2183}" => "\u{0186}",
        "\u{03FD}" => "\u{0186}",
        "\u{A4DB}" => "\u{0186}",
        "\u{10423}" => "\u{0186}",
        "\u{AB3F}" => "\u{0254}\u{0338}",
        "\u{AB62}" => "\u{0254}\u{0065}",
        "\u{1043F}" => "\u{0277}",
        "\u{2374}" => "\u{0070}",
        "\u{FF50}" => "\u{0070}",
        "\u{1D429}" => "\u{0070}",
        "\u{1D45D}" => "\u{0070}",
        "\u{1D491}" => "\u{0070}",
        "\u{1D4C5}" => "\u{0070}",
        "\u{1D4F9}" => "\u{0070}",
        "\u{1D52D}" => "\u{0070}",
        "\u{1D561}" => "\u{0070}",
        "\u{1D595}" => "\u{0070}",
        "\u{1D5C9}" => "\u{0070}",
        "\u{1D5FD}" => "\u{0070}",
        "\u{1D631}" => "\u{0070}",
        "\u{1D665}" => "\u{0070}",
        "\u{1D699}" => "\u{0070}",
        "\u{03C1}" => "\u{0070}",
        "\u{03F1}" => "\u{0070}",
        "\u{1D6D2}" => "\u{0070}",
        "\u{1D6E0}" => "\u{0070}",
        "\u{1D70C}" => "\u{0070}",
        "\u{1D71A}" => "\u{0070}",
        "\u{1D746}" => "\u{0070}",
        "\u{1D754}" => "\u{0070}",
        "\u{1D780}" => "\u{0070}",
        "\u{1D78E}" => "\u{0070}",
        "\u{1D7BA}" => "\u{0070}",
        "\u{1D7C8}" => "\u{0070}",
        "\u{2CA3}" => "\u{0070}",
        "\u{0440}" => "\u{0070}",
        "\u{FF30}" => "\u{0050}",
        "\u{2119}" => "\u{0050}",
        "\u{1D40F}" => "\u{0050}",
        "\u{1D443}" => "\u{0050}",
        "\u{1D477}" => "\u{0050}",
        "\u{1D4AB}" => "\u{0050}",
        "\u{1D4DF}" => "\u{0050}",
        "\u{1D513}" => "\u{0050}",
        "\u{1D57B}" => "\u{0050}",
        "\u{1D5AF}" => "\u{0050}",
        "\u{1D5E3}" => "\u{0050}",
        "\u{1D617}" => "\u{0050}",
        "\u{1D64B}" => "\u{0050}",
        "\u{1D67F}" => "\u{0050}",
        "\u{03A1}" => "\u{0050}",
        "\u{1D6B8}" => "\u{0050}",
        "\u{1D6F2}" => "\u{0050}",
        "\u{1D72C}" => "\u{0050}",
        "\u{1D766}" => "\u{0050}",
        "\u{1D7A0}" => "\u{0050}",
        "\u{2CA2}" => "\u{0050}",
        "\u{0420}" => "\u{0050}",
        "\u{13E2}" => "\u{0050}",
        "\u{146D}" => "\u{0050}",
        "\u{A4D1}" => "\u{0050}",
        "\u{10295}" => "\u{0050}",
        "\u{01A5}" => "\u{0070}\u{0314}",
        "\u{1D7D}" => "\u{0070}\u{0335}",
        "\u{1477}" => "\u{0070}\u{00B7}",
        "\u{1486}" => "\u{0050}\u{0027}",
        "\u{1D29}" => "\u{1D18}",
        "\u{ABB2}" => "\u{1D18}",
        "\u{03C6}" => "\u{0278}",
        "\u{03D5}" => "\u{0278}",
        "\u{1D6D7}" => "\u{0278}",
        "\u{1D6DF}" => "\u{0278}",
        "\u{1D711}" => "\u{0278}",
        "\u{1D719}" => "\u{0278}",
        "\u{1D74B}" => "\u{0278}",
        "\u{1D753}" => "\u{0278}",
        "\u{1D785}" => "\u{0278}",
        "\u{1D78D}" => "\u{0278}",
        "\u{1D7BF}" => "\u{0278}",
        "\u{1D7C7}" => "\u{0278}",
        "\u{2CAB}" => "\u{0278}",
        "\u{0444}" => "\u{0278}",
        "\u{1D42A}" => "\u{0071}",
        "\u{1D45E}" => "\u{0071}",
        "\u{1D492}" => "\u{0071}",
        "\u{1D4C6}" => "\u{0071}",
        "\u{1D4FA}" => "\u{0071}",
        "\u{1D52E}" => "\u{0071}",
        "\u{1D562}" => "\u{0071}",
        "\u{1D596}" => "\u{0071}",
        "\u{1D5CA}" => "\u{0071}",
        "\u{1D5FE}" => "\u{0071}",
        "\u{1D632}" => "\u{0071}",
        "\u{1D666}" => "\u{0071}",
        "\u{1D69A}" => "\u{0071}",
        "\u{051B}" => "\u{0071}",
        "\u{0563}" => "\u{0071}",
        "\u{0566}" => "\u{0071}",
        "\u{211A}" => "\u{0051}",
        "\u{1D410}" => "\u{0051}",
        "\u{1D444}" => "\u{0051}",
        "\u{1D478}" => "\u{0051}",
        "\u{1D4AC}" => "\u{0051}",
        "\u{1D4E0}" => "\u{0051}",
        "\u{1D514}" => "\u{0051}",
        "\u{1D57C}" => "\u{0051}",
        "\u{1D5B0}" => "\u{0051}",
        "\u{1D5E4}" => "\u{0051}",
        "\u{1D618}" => "\u{0051}",
        "\u{1D64C}" => "\u{0051}",
        "\u{1D680}" => "\u{0051}",
        "\u{2D55}" => "\u{0051}",
        "\u{02A0}" => "\u{0071}\u{0314}",
        "\u{1F700}" => "\u{0051}\u{0045}",
        "\u{1D90}" => "\u{024B}",
        "\u{1D0B}" => "\u{0138}",
        "\u{03BA}" => "\u{0138}",
        "\u{03F0}" => "\u{0138}",
        "\u{1D6CB}" => "\u{0138}",
        "\u{1D6DE}" => "\u{0138}",
        "\u{1D705}" => "\u{0138}",
        "\u{1D718}" => "\u{0138}",
        "\u{1D73F}" => "\u{0138}",
        "\u{1D752}" => "\u{0138}",
        "\u{1D779}" => "\u{0138}",
        "\u{1D78C}" => "\u{0138}",
        "\u{1D7B3}" => "\u{0138}",
        "\u{1D7C6}" => "\u{0138}",
        "\u{2C95}" => "\u{0138}",
        "\u{043A}" => "\u{0138}",
        "\u{ABB6}" => "\u{0138}",
        "\u{049B}" => "\u{0138}\u{0329}",
        "\u{049F}" => "\u{0138}\u{0335}",
        "\u{1D42B}" => "\u{0072}",
        "\u{1D45F}" => "\u{0072}",
        "\u{1D493}" => "\u{0072}",
        "\u{1D4C7}" => "\u{0072}",
        "\u{1D4FB}" => "\u{0072}",
        "\u{1D52F}" => "\u{0072}",
        "\u{1D563}" => "\u{0072}",
        "\u{1D597}" => "\u{0072}",
        "\u{1D5CB}" => "\u{0072}",
        "\u{1D5FF}" => "\u{0072}",
        "\u{1D633}" => "\u{0072}",
        "\u{1D667}" => "\u{0072}",
        "\u{1D69B}" => "\u{0072}",
        "\u{AB47}" => "\u{0072}",
        "\u{AB48}" => "\u{0072}",
        "\u{1D26}" => "\u{0072}",
        "\u{2C85}" => "\u{0072}",
        "\u{0433}" => "\u{0072}",
        "\u{AB81}" => "\u{0072}",
        "\u{1D216}" => "\u{0052}",
        "\u{211B}" => "\u{0052}",
        "\u{211C}" => "\u{0052}",
        "\u{211D}" => "\u{0052}",
        "\u{1D411}" => "\u{0052}",
        "\u{1D445}" => "\u{0052}",
        "\u{1D479}" => "\u{0052}",
        "\u{1D4E1}" => "\u{0052}",
        "\u{1D57D}" => "\u{0052}",
        "\u{1D5B1}" => "\u{0052}",
        "\u{1D5E5}" => "\u{0052}",
        "\u{1D619}" => "\u{0052}",
        "\u{1D64D}" => "\u{0052}",
        "\u{1D681}" => "\u{0052}",
        "\u{01A6}" => "\u{0052}",
        "\u{13A1}" => "\u{0052}",
        "\u{13D2}" => "\u{0052}",
        "\u{104B4}" => "\u{0052}",
        "\u{1587}" => "\u{0052}",
        "\u{A4E3}" => "\u{0052}",
        "\u{16F35}" => "\u{0052}",
        "\u{027D}" => "\u{0072}\u{0328}",
        "\u{027C}" => "\u{0072}\u{0329}",
        "\u{024D}" => "\u{0072}\u{0335}",
        "\u{0493}" => "\u{0072}\u{0335}",
        "\u{1D72}" => "\u{0072}\u{0334}",
        "\u{0491}" => "\u{0072}\u{0027}",
        "\u{118E3}" => "\u{0072}\u{006E}",
        "\u{006D}" => "\u{0072}\u{006E}",
        "\u{217F}" => "\u{0072}\u{006E}",
        "\u{1D426}" => "\u{0072}\u{006E}",
        "\u{1D45A}" => "\u{0072}\u{006E}",
        "\u{1D48E}" => "\u{0072}\u{006E}",
        "\u{1D4C2}" => "\u{0072}\u{006E}",
        "\u{1D4F6}" => "\u{0072}\u{006E}",
        "\u{1D52A}" => "\u{0072}\u{006E}",
        "\u{1D55E}" => "\u{0072}\u{006E}",
        "\u{1D592}" => "\u{0072}\u{006E}",
        "\u{1D5C6}" => "\u{0072}\u{006E}",
        "\u{1D5FA}" => "\u{0072}\u{006E}",
        "\u{1D62E}" => "\u{0072}\u{006E}",
        "\u{1D662}" => "\u{0072}\u{006E}",
        "\u{1D696}" => "\u{0072}\u{006E}",
        "\u{11700}" => "\u{0072}\u{006E}",
        "\u{20A5}" => "\u{0072}\u{006E}\u{0338}",
        "\u{0271}" => "\u{0072}\u{006E}\u{0326}",
        "\u{1D6F}" => "\u{0072}\u{006E}\u{0334}",
        "\u{20A8}" => "\u{0052}\u{0073}",
        "\u{AB71}" => "\u{0280}",
        "\u{ABA2}" => "\u{0280}",
        "\u{044F}" => "\u{1D19}",
        "\u{1D73}" => "\u{027E}\u{0334}",
        "\u{2129}" => "\u{027F}",
        "\u{FF53}" => "\u{0073}",
        "\u{1D42C}" => "\u{0073}",
        "\u{1D460}" => "\u{0073}",
        "\u{1D494}" => "\u{0073}",
        "\u{1D4C8}" => "\u{0073}",
        "\u{1D4FC}" => "\u{0073}",
        "\u{1D530}" => "\u{0073}",
        "\u{1D564}" => "\u{0073}",
        "\u{1D598}" => "\u{0073}",
        "\u{1D5CC}" => "\u{0073}",
        "\u{1D600}" => "\u{0073}",
        "\u{1D634}" => "\u{0073}",
        "\u{1D668}" => "\u{0073}",
        "\u{1D69C}" => "\u{0073}",
        "\u{A731}" => "\u{0073}",
        "\u{01BD}" => "\u{0073}",
        "\u{0455}" => "\u{0073}",
        "\u{ABAA}" => "\u{0073}",
        "\u{118C1}" => "\u{0073}",
        "\u{10448}" => "\u{0073}",
        "\u{FF33}" => "\u{0053}",
        "\u{1D412}" => "\u{0053}",
        "\u{1D446}" => "\u{0053}",
        "\u{1D47A}" => "\u{0053}",
        "\u{1D4AE}" => "\u{0053}",
        "\u{1D4E2}" => "\u{0053}",
        "\u{1D516}" => "\u{0053}",
        "\u{1D54A}" => "\u{0053}",
        "\u{1D57E}" => "\u{0053}",
        "\u{1D5B2}" => "\u{0053}",
        "\u{1D5E6}" => "\u{0053}",
        "\u{1D61A}" => "\u{0053}",
        "\u{1D64E}" => "\u{0053}",
        "\u{1D682}" => "\u{0053}",
        "\u{0405}" => "\u{0053}",
        "\u{054F}" => "\u{0053}",
        "\u{13D5}" => "\u{0053}",
        "\u{13DA}" => "\u{0053}",
        "\u{A4E2}" => "\u{0053}",
        "\u{16F3A}" => "\u{0053}",
        "\u{10296}" => "\u{0053}",
        "\u{10420}" => "\u{0053}",
        "\u{0282}" => "\u{0073}\u{0328}",
        "\u{1D74}" => "\u{0073}\u{0334}",
        "\u{A7B5}" => "\u{00DF}",
        "\u{03B2}" => "\u{00DF}",
        "\u{03D0}" => "\u{00DF}",
        "\u{1D6C3}" => "\u{00DF}",
        "\u{1D6FD}" => "\u{00DF}",
        "\u{1D737}" => "\u{00DF}",
        "\u{1D771}" => "\u{00DF}",
        "\u{1D7AB}" => "\u{00DF}",
        "\u{13F0}" => "\u{00DF}",
        "\u{1F75C}" => "\u{0073}\u{0073}\u{0073}",
        "\u{FB06}" => "\u{0073}\u{0074}",
        "\u{222B}" => "\u{0283}",
        "\u{AB4D}" => "\u{0283}",
        "\u{2211}" => "\u{01A9}",
        "\u{2140}" => "\u{01A9}",
        "\u{03A3}" => "\u{01A9}",
        "\u{1D6BA}" => "\u{01A9}",
        "\u{1D6F4}" => "\u{01A9}",
        "\u{1D72E}" => "\u{01A9}",
        "\u{1D768}" => "\u{01A9}",
        "\u{1D7A2}" => "\u{01A9}",
        "\u{2D49}" => "\u{01A9}",
        "\u{222C}" => "\u{0283}\u{0283}",
        "\u{222D}" => "\u{0283}\u{0283}\u{0283}",
        "\u{2A0C}" => "\u{0283}\u{0283}\u{0283}\u{0283}",
        "\u{1D42D}" => "\u{0074}",
        "\u{1D461}" => "\u{0074}",
        "\u{1D495}" => "\u{0074}",
        "\u{1D4C9}" => "\u{0074}",
        "\u{1D4FD}" => "\u{0074}",
        "\u{1D531}" => "\u{0074}",
        "\u{1D565}" => "\u{0074}",
        "\u{1D599}" => "\u{0074}",
        "\u{1D5CD}" => "\u{0074}",
        "\u{1D601}" => "\u{0074}",
        "\u{1D635}" => "\u{0074}",
        "\u{1D669}" => "\u{0074}",
        "\u{1D69D}" => "\u{0074}",
        "\u{22A4}" => "\u{0054}",
        "\u{27D9}" => "\u{0054}",
        "\u{1F768}" => "\u{0054}",
        "\u{FF34}" => "\u{0054}",
        "\u{1D413}" => "\u{0054}",
        "\u{1D447}" => "\u{0054}",
        "\u{1D47B}" => "\u{0054}",
        "\u{1D4AF}" => "\u{0054}",
        "\u{1D4E3}" => "\u{0054}",
        "\u{1D517}" => "\u{0054}",
        "\u{1D54B}" => "\u{0054}",
        "\u{1D57F}" => "\u{0054}",
        "\u{1D5B3}" => "\u{0054}",
        "\u{1D5E7}" => "\u{0054}",
        "\u{1D61B}" => "\u{0054}",
        "\u{1D64F}" => "\u{0054}",
        "\u{1D683}" => "\u{0054}",
        "\u{03A4}" => "\u{0054}",
        "\u{1D6BB}" => "\u{0054}",
        "\u{1D6F5}" => "\u{0054}",
        "\u{1D72F}" => "\u{0054}",
        "\u{1D769}" => "\u{0054}",
        "\u{1D7A3}" => "\u{0054}",
        "\u{2CA6}" => "\u{0054}",
        "\u{0422}" => "\u{0054}",
        "\u{13A2}" => "\u{0054}",
        "\u{A4D4}" => "\u{0054}",
        "\u{16F0A}" => "\u{0054}",
        "\u{118BC}" => "\u{0054}",
        "\u{10297}" => "\u{0054}",
        "\u{102B1}" => "\u{0054}",
        "\u{10315}" => "\u{0054}",
        "\u{01AD}" => "\u{0074}\u{0314}",
        "\u{2361}" => "\u{0054}\u{0308}",
        "\u{023E}" => "\u{0054}\u{0338}",
        "\u{021A}" => "\u{0162}",
        "\u{01AE}" => "\u{0054}\u{0328}",
        "\u{04AC}" => "\u{0054}\u{0329}",
        "\u{20AE}" => "\u{0054}\u{20EB}",
        "\u{0167}" => "\u{0074}\u{0335}",
        "\u{0166}" => "\u{0054}\u{0335}",
        "\u{1D75}" => "\u{0074}\u{0334}",
        "\u{10A0}" => "\u{A786}",
        "\u{A728}" => "\u{0054}\u{0033}",
        "\u{02A8}" => "\u{0074}\u{0255}",
        "\u{2121}" => "\u{0054}\u{0045}\u{004C}",
        "\u{A777}" => "\u{0074}\u{0066}",
        "\u{02A6}" => "\u{0074}\u{0073}",
        "\u{02A7}" => "\u{0074}\u{0283}",
        "\u{A729}" => "\u{0074}\u{021D}",
        "\u{03C4}" => "\u{1D1B}",
        "\u{1D6D5}" => "\u{1D1B}",
        "\u{1D70F}" => "\u{1D1B}",
        "\u{1D749}" => "\u{1D1B}",
        "\u{1D783}" => "\u{1D1B}",
        "\u{1D7BD}" => "\u{1D1B}",
        "\u{0442}" => "\u{1D1B}",
        "\u{AB72}" => "\u{1D1B}",
        "\u{04AD}" => "\u{1D1B}\u{0329}",
        "\u{0163}" => "\u{01AB}",
        "\u{021B}" => "\u{01AB}",
        "\u{13BF}" => "\u{01AB}",
        "\u{1D42E}" => "\u{0075}",
        "\u{1D462}" => "\u{0075}",
        "\u{1D496}" => "\u{0075}",
        "\u{1D4CA}" => "\u{0075}",
        "\u{1D4FE}" => "\u{0075}",
        "\u{1D532}" => "\u{0075}",
        "\u{1D566}" => "\u{0075}",
        "\u{1D59A}" => "\u{0075}",
        "\u{1D5CE}" => "\u{0075}",
        "\u{1D602}" => "\u{0075}",
        "\u{1D636}" => "\u{0075}",
        "\u{1D66A}" => "\u{0075}",
        "\u{1D69E}" => "\u{0075}",
        "\u{A79F}" => "\u{0075}",
        "\u{1D1C}" => "\u{0075}",
        "\u{AB4E}" => "\u{0075}",
        "\u{AB52}" => "\u{0075}",
        "\u{028B}" => "\u{0075}",
        "\u{03C5}" => "\u{0075}",
        "\u{1D6D6}" => "\u{0075}",
        "\u{1D710}" => "\u{0075}",
        "\u{1D74A}" => "\u{0075}",
        "\u{1D784}" => "\u{0075}",
        "\u{1D7BE}" => "\u{0075}",
        "\u{057D}" => "\u{0075}",
        "\u{104F6}" => "\u{0075}",
        "\u{118D8}" => "\u{0075}",
        "\u{222A}" => "\u{0055}",
        "\u{22C3}" => "\u{0055}",
        "\u{1D414}" => "\u{0055}",
        "\u{1D448}" => "\u{0055}",
        "\u{1D47C}" => "\u{0055}",
        "\u{1D4B0}" => "\u{0055}",
        "\u{1D4E4}" => "\u{0055}",
        "\u{1D518}" => "\u{0055}",
        "\u{1D54C}" => "\u{0055}",
        "\u{1D580}" => "\u{0055}",
        "\u{1D5B4}" => "\u{0055}",
        "\u{1D5E8}" => "\u{0055}",
        "\u{1D61C}" => "\u{0055}",
        "\u{1D650}" => "\u{0055}",
        "\u{1D684}" => "\u{0055}",
        "\u{054D}" => "\u{0055}",
        "\u{1200}" => "\u{0055}",
        "\u{104CE}" => "\u{0055}",
        "\u{144C}" => "\u{0055}",
        "\u{A4F4}" => "\u{0055}",
        "\u{16F42}" => "\u{0055}",
        "\u{118B8}" => "\u{0055}",
        "\u{01D4}" => "\u{016D}",
        "\u{01D3}" => "\u{016C}",
        "\u{1D7E}" => "\u{0075}\u{0335}",
        "\u{AB9C}" => "\u{0075}\u{0335}",
        "\u{0244}" => "\u{0055}\u{0335}",
        "\u{13CC}" => "\u{0055}\u{0335}",
        "\u{1458}" => "\u{0055}\u{00B7}",
        "\u{1467}" => "\u{0055}\u{0027}",
        "\u{1D6B}" => "\u{0075}\u{0065}",
        "\u{AB63}" => "\u{0075}\u{006F}",
        "\u{1E43}" => "\u{AB51}",
        "\u{057A}" => "\u{0270}",
        "\u{1223}" => "\u{0270}",
        "\u{2127}" => "\u{01B1}",
        "\u{162E}" => "\u{01B1}",
        "\u{1634}" => "\u{01B1}",
        "\u{1D7F}" => "\u{028A}\u{0335}",
        "\u{2228}" => "\u{0076}",
        "\u{22C1}" => "\u{0076}",
        "\u{FF56}" => "\u{0076}",
        "\u{2174}" => "\u{0076}",
        "\u{1D42F}" => "\u{0076}",
        "\u{1D463}" => "\u{0076}",
        "\u{1D497}" => "\u{0076}",
        "\u{1D4CB}" => "\u{0076}",
        "\u{1D4FF}" => "\u{0076}",
        "\u{1D533}" => "\u{0076}",
        "\u{1D567}" => "\u{0076}",
        "\u{1D59B}" => "\u{0076}",
        "\u{1D5CF}" => "\u{0076}",
        "\u{1D603}" => "\u{0076}",
        "\u{1D637}" => "\u{0076}",
        "\u{1D66B}" => "\u{0076}",
        "\u{1D69F}" => "\u{0076}",
        "\u{1D20}" => "\u{0076}",
        "\u{03BD}" => "\u{0076}",
        "\u{1D6CE}" => "\u{0076}",
        "\u{1D708}" => "\u{0076}",
        "\u{1D742}" => "\u{0076}",
        "\u{1D77C}" => "\u{0076}",
        "\u{1D7B6}" => "\u{0076}",
        "\u{0475}" => "\u{0076}",
        "\u{05D8}" => "\u{0076}",
        "\u{11706}" => "\u{0076}",
        "\u{ABA9}" => "\u{0076}",
        "\u{118C0}" => "\u{0076}",
        "\u{1D20D}" => "\u{0056}",
        "\u{0667}" => "\u{0056}",
        "\u{06F7}" => "\u{0056}",
        "\u{2164}" => "\u{0056}",
        "\u{1D415}" => "\u{0056}",
        "\u{1D449}" => "\u{0056}",
        "\u{1D47D}" => "\u{0056}",
        "\u{1D4B1}" => "\u{0056}",
        "\u{1D4E5}" => "\u{0056}",
        "\u{1D519}" => "\u{0056}",
        "\u{1D54D}" => "\u{0056}",
        "\u{1D581}" => "\u{0056}",
        "\u{1D5B5}" => "\u{0056}",
        "\u{1D5E9}" => "\u{0056}",
        "\u{1D61D}" => "\u{0056}",
        "\u{1D651}" => "\u{0056}",
        "\u{1D685}" => "\u{0056}",
        "\u{0474}" => "\u{0056}",
        "\u{2D38}" => "\u{0056}",
        "\u{13D9}" => "\u{0056}",
        "\u{142F}" => "\u{0056}",
        "\u{A6DF}" => "\u{0056}",
        "\u{A4E6}" => "\u{0056}",
        "\u{16F08}" => "\u{0056}",
        "\u{118A0}" => "\u{0056}",
        "\u{1051D}" => "\u{0056}",
        "\u{10197}" => "\u{0056}\u{0335}",
        "\u{143B}" => "\u{0056}\u{00B7}",
        "\u{1F76C}" => "\u{0056}\u{0042}",
        "\u{2175}" => "\u{0076}\u{0069}",
        "\u{2176}" => "\u{0076}\u{0069}\u{0069}",
        "\u{2177}" => "\u{0076}\u{0069}\u{0069}\u{0069}",
        "\u{2165}" => "\u{0056}\u{006C}",
        "\u{2166}" => "\u{0056}\u{006C}\u{006C}",
        "\u{2167}" => "\u{0056}\u{006C}\u{006C}\u{006C}",
        "\u{1F708}" => "\u{0056}\u{1DE4}",
        "\u{1D27}" => "\u{028C}",
        "\u{104D8}" => "\u{028C}",
        "\u{0668}" => "\u{0245}",
        "\u{06F8}" => "\u{0245}",
        "\u{039B}" => "\u{0245}",
        "\u{1D6B2}" => "\u{0245}",
        "\u{1D6EC}" => "\u{0245}",
        "\u{1D726}" => "\u{0245}",
        "\u{1D760}" => "\u{0245}",
        "\u{1D79A}" => "\u{0245}",
        "\u{041B}" => "\u{0245}",
        "\u{2D37}" => "\u{0245}",
        "\u{104B0}" => "\u{0245}",
        "\u{1431}" => "\u{0245}",
        "\u{A6CE}" => "\u{0245}",
        "\u{A4E5}" => "\u{0245}",
        "\u{16F3D}" => "\u{0245}",
        "\u{1028D}" => "\u{0245}",
        "\u{04C5}" => "\u{0245}\u{0326}",
        "\u{143D}" => "\u{0245}\u{00B7}",
        "\u{026F}" => "\u{0077}",
        "\u{1D430}" => "\u{0077}",
        "\u{1D464}" => "\u{0077}",
        "\u{1D498}" => "\u{0077}",
        "\u{1D4CC}" => "\u{0077}",
        "\u{1D500}" => "\u{0077}",
        "\u{1D534}" => "\u{0077}",
        "\u{1D568}" => "\u{0077}",
        "\u{1D59C}" => "\u{0077}",
        "\u{1D5D0}" => "\u{0077}",
        "\u{1D604}" => "\u{0077}",
        "\u{1D638}" => "\u{0077}",
        "\u{1D66C}" => "\u{0077}",
        "\u{1D6A0}" => "\u{0077}",
        "\u{1D21}" => "\u{0077}",
        "\u{0461}" => "\u{0077}",
        "\u{051D}" => "\u{0077}",
        "\u{0561}" => "\u{0077}",
        "\u{1170A}" => "\u{0077}",
        "\u{1170E}" => "\u{0077}",
        "\u{1170F}" => "\u{0077}",
        "\u{AB83}" => "\u{0077}",
        "\u{118EF}" => "\u{0057}",
        "\u{118E6}" => "\u{0057}",
        "\u{1D416}" => "\u{0057}",
        "\u{1D44A}" => "\u{0057}",
        "\u{1D47E}" => "\u{0057}",
        "\u{1D4B2}" => "\u{0057}",
        "\u{1D4E6}" => "\u{0057}",
        "\u{1D51A}" => "\u{0057}",
        "\u{1D54E}" => "\u{0057}",
        "\u{1D582}" => "\u{0057}",
        "\u{1D5B6}" => "\u{0057}",
        "\u{1D5EA}" => "\u{0057}",
        "\u{1D61E}" => "\u{0057}",
        "\u{1D652}" => "\u{0057}",
        "\u{1D686}" => "\u{0057}",
        "\u{051C}" => "\u{0057}",
        "\u{13B3}" => "\u{0057}",
        "\u{13D4}" => "\u{0057}",
        "\u{A4EA}" => "\u{0057}",
        "\u{047D}" => "\u{0077}\u{0486}\u{0487}",
        "\u{114C5}" => "\u{0077}\u{0307}",
        "\u{20A9}" => "\u{0057}\u{0335}",
        "\u{A761}" => "\u{0077}\u{0326}",
        "\u{1D0D}" => "\u{028D}",
        "\u{043C}" => "\u{028D}",
        "\u{AB87}" => "\u{028D}",
        "\u{04CE}" => "\u{028D}\u{0326}",
        "\u{166E}" => "\u{0078}",
        "\u{00D7}" => "\u{0078}",
        "\u{292B}" => "\u{0078}",
        "\u{292C}" => "\u{0078}",
        "\u{2A2F}" => "\u{0078}",
        "\u{FF58}" => "\u{0078}",
        "\u{2179}" => "\u{0078}",
        "\u{1D431}" => "\u{0078}",
        "\u{1D465}" => "\u{0078}",
        "\u{1D499}" => "\u{0078}",
        "\u{1D4CD}" => "\u{0078}",
        "\u{1D501}" => "\u{0078}",
        "\u{1D535}" => "\u{0078}",
        "\u{1D569}" => "\u{0078}",
        "\u{1D59D}" => "\u{0078}",
        "\u{1D5D1}" => "\u{0078}",
        "\u{1D605}" => "\u{0078}",
        "\u{1D639}" => "\u{0078}",
        "\u{1D66D}" => "\u{0078}",
        "\u{1D6A1}" => "\u{0078}",
        "\u{0445}" => "\u{0078}",
        "\u{1541}" => "\u{0078}",
        "\u{157D}" => "\u{0078}",
        "\u{2DEF}" => "\u{036F}",
        "\u{166D}" => "\u{0058}",
        "\u{2573}" => "\u{0058}",
        "\u{10322}" => "\u{0058}",
        "\u{118EC}" => "\u{0058}",
        "\u{FF38}" => "\u{0058}",
        "\u{2169}" => "\u{0058}",
        "\u{1D417}" => "\u{0058}",
        "\u{1D44B}" => "\u{0058}",
        "\u{1D47F}" => "\u{0058}",
        "\u{1D4B3}" => "\u{0058}",
        "\u{1D4E7}" => "\u{0058}",
        "\u{1D51B}" => "\u{0058}",
        "\u{1D54F}" => "\u{0058}",
        "\u{1D583}" => "\u{0058}",
        "\u{1D5B7}" => "\u{0058}",
        "\u{1D5EB}" => "\u{0058}",
        "\u{1D61F}" => "\u{0058}",
        "\u{1D653}" => "\u{0058}",
        "\u{1D687}" => "\u{0058}",
        "\u{A7B3}" => "\u{0058}",
        "\u{03A7}" => "\u{0058}",
        "\u{1D6BE}" => "\u{0058}",
        "\u{1D6F8}" => "\u{0058}",
        "\u{1D732}" => "\u{0058}",
        "\u{1D76C}" => "\u{0058}",
        "\u{1D7A6}" => "\u{0058}",
        "\u{2CAC}" => "\u{0058}",
        "\u{0425}" => "\u{0058}",
        "\u{2D5D}" => "\u{0058}",
        "\u{16B7}" => "\u{0058}",
        "\u{A4EB}" => "\u{0058}",
        "\u{10290}" => "\u{0058}",
        "\u{102B4}" => "\u{0058}",
        "\u{10317}" => "\u{0058}",
        "\u{10527}" => "\u{0058}",
        "\u{2A30}" => "\u{0078}\u{0307}",
        "\u{04B2}" => "\u{0058}\u{0329}",
        "\u{10196}" => "\u{0058}\u{0335}",
        "\u{217A}" => "\u{0078}\u{0069}",
        "\u{217B}" => "\u{0078}\u{0069}\u{0069}",
        "\u{216A}" => "\u{0058}\u{006C}",
        "\u{216B}" => "\u{0058}\u{006C}\u{006C}",
        "\u{0263}" => "\u{0079}",
        "\u{1D8C}" => "\u{0079}",
        "\u{FF59}" => "\u{0079}",
        "\u{1D432}" => "\u{0079}",
        "\u{1D466}" => "\u{0079}",
        "\u{1D49A}" => "\u{0079}",
        "\u{1D4CE}" => "\u{0079}",
        "\u{1D502}" => "\u{0079}",
        "\u{1D536}" => "\u{0079}",
        "\u{1D56A}" => "\u{0079}",
        "\u{1D59E}" => "\u{0079}",
        "\u{1D5D2}" => "\u{0079}",
        "\u{1D606}" => "\u{0079}",
        "\u{1D63A}" => "\u{0079}",
        "\u{1D66E}" => "\u{0079}",
        "\u{1D6A2}" => "\u{0079}",
        "\u{028F}" => "\u{0079}",
        "\u{1EFF}" => "\u{0079}",
        "\u{AB5A}" => "\u{0079}",
        "\u{03B3}" => "\u{0079}",
        "\u{213D}" => "\u{0079}",
        "\u{1D6C4}" => "\u{0079}",
        "\u{1D6FE}" => "\u{0079}",
        "\u{1D738}" => "\u{0079}",
        "\u{1D772}" => "\u{0079}",
        "\u{1D7AC}" => "\u{0079}",
        "\u{0443}" => "\u{0079}",
        "\u{04AF}" => "\u{0079}",
        "\u{10E7}" => "\u{0079}",
        "\u{118DC}" => "\u{0079}",
        "\u{FF39}" => "\u{0059}",
        "\u{1D418}" => "\u{0059}",
        "\u{1D44C}" => "\u{0059}",
        "\u{1D480}" => "\u{0059}",
        "\u{1D4B4}" => "\u{0059}",
        "\u{1D4E8}" => "\u{0059}",
        "\u{1D51C}" => "\u{0059}",
        "\u{1D550}" => "\u{0059}",
        "\u{1D584}" => "\u{0059}",
        "\u{1D5B8}" => "\u{0059}",
        "\u{1D5EC}" => "\u{0059}",
        "\u{1D620}" => "\u{0059}",
        "\u{1D654}" => "\u{0059}",
        "\u{1D688}" => "\u{0059}",
        "\u{03A5}" => "\u{0059}",
        "\u{03D2}" => "\u{0059}",
        "\u{1D6BC}" => "\u{0059}",
        "\u{1D6F6}" => "\u{0059}",
        "\u{1D730}" => "\u{0059}",
        "\u{1D76A}" => "\u{0059}",
        "\u{1D7A4}" => "\u{0059}",
        "\u{2CA8}" => "\u{0059}",
        "\u{0423}" => "\u{0059}",
        "\u{04AE}" => "\u{0059}",
        "\u{13A9}" => "\u{0059}",
        "\u{13BD}" => "\u{0059}",
        "\u{A4EC}" => "\u{0059}",
        "\u{16F43}" => "\u{0059}",
        "\u{118A4}" => "\u{0059}",
        "\u{102B2}" => "\u{0059}",
        "\u{01B4}" => "\u{0079}\u{0314}",
        "\u{024F}" => "\u{0079}\u{0335}",
        "\u{04B1}" => "\u{0079}\u{0335}",
        "\u{00A5}" => "\u{0059}\u{0335}",
        "\u{024E}" => "\u{0059}\u{0335}",
        "\u{04B0}" => "\u{0059}\u{0335}",
        "\u{0292}" => "\u{021D}",
        "\u{A76B}" => "\u{021D}",
        "\u{2CCD}" => "\u{021D}",
        "\u{04E1}" => "\u{021D}",
        "\u{10F3}" => "\u{021D}",
        "\u{1D433}" => "\u{007A}",
        "\u{1D467}" => "\u{007A}",
        "\u{1D49B}" => "\u{007A}",
        "\u{1D4CF}" => "\u{007A}",
        "\u{1D503}" => "\u{007A}",
        "\u{1D537}" => "\u{007A}",
        "\u{1D56B}" => "\u{007A}",
        "\u{1D59F}" => "\u{007A}",
        "\u{1D5D3}" => "\u{007A}",
        "\u{1D607}" => "\u{007A}",
        "\u{1D63B}" => "\u{007A}",
        "\u{1D66F}" => "\u{007A}",
        "\u{1D6A3}" => "\u{007A}",
        "\u{1D22}" => "\u{007A}",
        "\u{AB93}" => "\u{007A}",
        "\u{118C4}" => "\u{007A}",
        "\u{102F5}" => "\u{005A}",
        "\u{118E5}" => "\u{005A}",
        "\u{FF3A}" => "\u{005A}",
        "\u{2124}" => "\u{005A}",
        "\u{2128}" => "\u{005A}",
        "\u{1D419}" => "\u{005A}",
        "\u{1D44D}" => "\u{005A}",
        "\u{1D481}" => "\u{005A}",
        "\u{1D4B5}" => "\u{005A}",
        "\u{1D4E9}" => "\u{005A}",
        "\u{1D585}" => "\u{005A}",
        "\u{1D5B9}" => "\u{005A}",
        "\u{1D5ED}" => "\u{005A}",
        "\u{1D621}" => "\u{005A}",
        "\u{1D655}" => "\u{005A}",
        "\u{1D689}" => "\u{005A}",
        "\u{0396}" => "\u{005A}",
        "\u{1D6AD}" => "\u{005A}",
        "\u{1D6E7}" => "\u{005A}",
        "\u{1D721}" => "\u{005A}",
        "\u{1D75B}" => "\u{005A}",
        "\u{1D795}" => "\u{005A}",
        "\u{13C3}" => "\u{005A}",
        "\u{A4DC}" => "\u{005A}",
        "\u{118A9}" => "\u{005A}",
        "\u{0290}" => "\u{007A}\u{0328}",
        "\u{01B6}" => "\u{007A}\u{0335}",
        "\u{01B5}" => "\u{005A}\u{0335}",
        "\u{0225}" => "\u{007A}\u{0326}",
        "\u{0224}" => "\u{005A}\u{0326}",
        "\u{1D76}" => "\u{007A}\u{0334}",
        "\u{01BF}" => "\u{00FE}",
        "\u{03F8}" => "\u{00FE}",
        "\u{03F7}" => "\u{00DE}",
        "\u{104C4}" => "\u{00DE}",
        "\u{2079}" => "\u{A770}",
        "\u{1D24}" => "\u{01A8}",
        "\u{03E9}" => "\u{01A8}",
        "\u{A645}" => "\u{01A8}",
        "\u{044C}" => "\u{0185}",
        "\u{AB9F}" => "\u{0185}",
        "\u{044B}" => "\u{0185}\u{0069}",
        "\u{AB7E}" => "\u{0242}",
        "\u{02E4}" => "\u{02C1}",
        "\u{A6CD}" => "\u{02A1}",
        "\u{2299}" => "\u{0298}",
        "\u{2609}" => "\u{0298}",
        "\u{2A00}" => "\u{0298}",
        "\u{A668}" => "\u{0298}",
        "\u{2D59}" => "\u{0298}",
        "\u{104C3}" => "\u{0298}",
        "\u{213E}" => "\u{0393}",
        "\u{1D6AA}" => "\u{0393}",
        "\u{1D6E4}" => "\u{0393}",
        "\u{1D71E}" => "\u{0393}",
        "\u{1D758}" => "\u{0393}",
        "\u{1D792}" => "\u{0393}",
        "\u{2C84}" => "\u{0393}",
        "\u{0413}" => "\u{0393}",
        "\u{13B1}" => "\u{0393}",
        "\u{14A5}" => "\u{0393}",
        "\u{16F07}" => "\u{0393}",
        "\u{0492}" => "\u{0393}\u{0335}",
        "\u{14AF}" => "\u{0393}\u{00B7}",
        "\u{0490}" => "\u{0393}\u{0027}",
        "\u{2206}" => "\u{0394}",
        "\u{25B3}" => "\u{0394}",
        "\u{1F702}" => "\u{0394}",
        "\u{1D6AB}" => "\u{0394}",
        "\u{1D6E5}" => "\u{0394}",
        "\u{1D71F}" => "\u{0394}",
        "\u{1D759}" => "\u{0394}",
        "\u{1D793}" => "\u{0394}",
        "\u{2C86}" => "\u{0394}",
        "\u{2D60}" => "\u{0394}",
        "\u{1403}" => "\u{0394}",
        "\u{16F1A}" => "\u{0394}",
        "\u{10285}" => "\u{0394}",
        "\u{102A3}" => "\u{0394}",
        "\u{2359}" => "\u{0394}\u{0332}",
        "\u{140F}" => "\u{0394}\u{00B7}",
        "\u{142C}" => "\u{0394}\u{1420}",
        "\u{1D7CB}" => "\u{03DD}",
        "\u{1D6C7}" => "\u{03B6}",
        "\u{1D701}" => "\u{03B6}",
        "\u{1D73B}" => "\u{03B6}",
        "\u{1D775}" => "\u{03B6}",
        "\u{1D7AF}" => "\u{03B6}",
        "\u{2CE4}" => "\u{03D7}",
        "\u{1D6CC}" => "\u{03BB}",
        "\u{1D706}" => "\u{03BB}",
        "\u{1D740}" => "\u{03BB}",
        "\u{1D77A}" => "\u{03BB}",
        "\u{1D7B4}" => "\u{03BB}",
        "\u{2C96}" => "\u{03BB}",
        "\u{104DB}" => "\u{03BB}",
        "\u{00B5}" => "\u{03BC}",
        "\u{1D6CD}" => "\u{03BC}",
        "\u{1D707}" => "\u{03BC}",
        "\u{1D741}" => "\u{03BC}",
        "\u{1D77B}" => "\u{03BC}",
        "\u{1D7B5}" => "\u{03BC}",
        "\u{1D6CF}" => "\u{03BE}",
        "\u{1D709}" => "\u{03BE}",
        "\u{1D743}" => "\u{03BE}",
        "\u{1D77D}" => "\u{03BE}",
        "\u{1D7B7}" => "\u{03BE}",
        "\u{1D6B5}" => "\u{039E}",
        "\u{1D6EF}" => "\u{039E}",
        "\u{1D729}" => "\u{039E}",
        "\u{1D763}" => "\u{039E}",
        "\u{1D79D}" => "\u{039E}",
        "\u{03D6}" => "\u{03C0}",
        "\u{213C}" => "\u{03C0}",
        "\u{1D6D1}" => "\u{03C0}",
        "\u{1D6E1}" => "\u{03C0}",
        "\u{1D70B}" => "\u{03C0}",
        "\u{1D71B}" => "\u{03C0}",
        "\u{1D745}" => "\u{03C0}",
        "\u{1D755}" => "\u{03C0}",
        "\u{1D77F}" => "\u{03C0}",
        "\u{1D78F}" => "\u{03C0}",
        "\u{1D7B9}" => "\u{03C0}",
        "\u{1D7C9}" => "\u{03C0}",
        "\u{1D28}" => "\u{03C0}",
        "\u{043F}" => "\u{03C0}",
        "\u{220F}" => "\u{03A0}",
        "\u{213F}" => "\u{03A0}",
        "\u{1D6B7}" => "\u{03A0}",
        "\u{1D6F1}" => "\u{03A0}",
        "\u{1D72B}" => "\u{03A0}",
        "\u{1D765}" => "\u{03A0}",
        "\u{1D79F}" => "\u{03A0}",
        "\u{2CA0}" => "\u{03A0}",
        "\u{041F}" => "\u{03A0}",
        "\u{A6DB}" => "\u{03A0}",
        "\u{102AD}" => "\u{03D8}",
        "\u{10312}" => "\u{03D8}",
        "\u{03DB}" => "\u{03C2}",
        "\u{1D6D3}" => "\u{03C2}",
        "\u{1D70D}" => "\u{03C2}",
        "\u{1D747}" => "\u{03C2}",
        "\u{1D781}" => "\u{03C2}",
        "\u{1D7BB}" => "\u{03C2}",
        "\u{1D6BD}" => "\u{03A6}",
        "\u{1D6F7}" => "\u{03A6}",
        "\u{1D731}" => "\u{03A6}",
        "\u{1D76B}" => "\u{03A6}",
        "\u{1D7A5}" => "\u{03A6}",
        "\u{2CAA}" => "\u{03A6}",
        "\u{0424}" => "\u{03A6}",
        "\u{0553}" => "\u{03A6}",
        "\u{1240}" => "\u{03A6}",
        "\u{16F0}" => "\u{03A6}",
        "\u{102B3}" => "\u{03A6}",
        "\u{AB53}" => "\u{03C7}",
        "\u{AB55}" => "\u{03C7}",
        "\u{1D6D8}" => "\u{03C7}",
        "\u{1D712}" => "\u{03C7}",
        "\u{1D74C}" => "\u{03C7}",
        "\u{1D786}" => "\u{03C7}",
        "\u{1D7C0}" => "\u{03C7}",
        "\u{2CAD}" => "\u{03C7}",
        "\u{1D6D9}" => "\u{03C8}",
        "\u{1D713}" => "\u{03C8}",
        "\u{1D74D}" => "\u{03C8}",
        "\u{1D787}" => "\u{03C8}",
        "\u{1D7C1}" => "\u{03C8}",
        "\u{0471}" => "\u{03C8}",
        "\u{104F9}" => "\u{03C8}",
        "\u{1D6BF}" => "\u{03A8}",
        "\u{1D6F9}" => "\u{03A8}",
        "\u{1D733}" => "\u{03A8}",
        "\u{1D76D}" => "\u{03A8}",
        "\u{1D7A7}" => "\u{03A8}",
        "\u{2CAE}" => "\u{03A8}",
        "\u{0470}" => "\u{03A8}",
        "\u{104D1}" => "\u{03A8}",
        "\u{16D8}" => "\u{03A8}",
        "\u{102B5}" => "\u{03A8}",
        "\u{2375}" => "\u{03C9}",
        "\u{A7B7}" => "\u{03C9}",
        "\u{1D6DA}" => "\u{03C9}",
        "\u{1D714}" => "\u{03C9}",
        "\u{1D74E}" => "\u{03C9}",
        "\u{1D788}" => "\u{03C9}",
        "\u{1D7C2}" => "\u{03C9}",
        "\u{2CB1}" => "\u{03C9}",
        "\u{A64D}" => "\u{03C9}",
        "\u{2126}" => "\u{03A9}",
        "\u{1D6C0}" => "\u{03A9}",
        "\u{1D6FA}" => "\u{03A9}",
        "\u{1D734}" => "\u{03A9}",
        "\u{1D76E}" => "\u{03A9}",
        "\u{1D7A8}" => "\u{03A9}",
        "\u{162F}" => "\u{03A9}",
        "\u{1635}" => "\u{03A9}",
        "\u{102B6}" => "\u{03A9}",
        "\u{2379}" => "\u{03C9}\u{0332}",
        "\u{1F7D}" => "\u{1FF4}",
        "\u{2630}" => "\u{2CB6}",
        "\u{2CDC}" => "\u{03EC}",
        "\u{0497}" => "\u{0436}\u{0329}",
        "\u{0496}" => "\u{0416}\u{0329}",
        "\u{1D20B}" => "\u{0418}",
        "\u{0376}" => "\u{0418}",
        "\u{A6A1}" => "\u{0418}",
        "\u{10425}" => "\u{0418}",
        "\u{0419}" => "\u{040D}",
        "\u{048A}" => "\u{040D}\u{0326}",
        "\u{045D}" => "\u{0439}",
        "\u{048B}" => "\u{0439}\u{0326}",
        "\u{104BC}" => "\u{04C3}",
        "\u{1D2B}" => "\u{043B}",
        "\u{04C6}" => "\u{043B}\u{0326}",
        "\u{AB60}" => "\u{0459}",
        "\u{104EB}" => "\u{A669}",
        "\u{1DEE}" => "\u{2DEC}",
        "\u{104CD}" => "\u{040B}",
        "\u{1D202}" => "\u{04FE}",
        "\u{1D222}" => "\u{0460}",
        "\u{13C7}" => "\u{0460}",
        "\u{15EF}" => "\u{0460}",
        "\u{047C}" => "\u{0460}\u{0486}\u{0487}",
        "\u{18ED}" => "\u{0460}\u{00B7}",
        "\u{A7B6}" => "\u{A64C}",
        "\u{04CC}" => "\u{04B7}",
        "\u{04CB}" => "\u{04B6}",
        "\u{04BE}" => "\u{04BC}\u{0328}",
        "\u{2CBD}" => "\u{0448}",
        "\u{2CBC}" => "\u{0428}",
        "\u{A650}" => "\u{042A}\u{006C}",
        "\u{2108}" => "\u{042D}",
        "\u{1F701}" => "\u{A658}",
        "\u{16F1C}" => "\u{A658}",
        "\u{A992}" => "\u{2C3F}",
        "\u{0587}" => "\u{0565}\u{0582}",
        "\u{1294}" => "\u{0571}",
        "\u{FB14}" => "\u{0574}\u{0565}",
        "\u{FB15}" => "\u{0574}\u{056B}",
        "\u{FB17}" => "\u{0574}\u{056D}",
        "\u{FB13}" => "\u{0574}\u{0576}",
        "\u{2229}" => "\u{0548}",
        "\u{22C2}" => "\u{0548}",
        "\u{1D245}" => "\u{0548}",
        "\u{1260}" => "\u{0548}",
        "\u{144E}" => "\u{0548}",
        "\u{A4F5}" => "\u{0548}",
        "\u{145A}" => "\u{0548}\u{00B7}",
        "\u{1468}" => "\u{0548}\u{0027}",
        "\u{FB16}" => "\u{057E}\u{0576}",
        "\u{20BD}" => "\u{0554}",
        "\u{02D3}" => "\u{0559}",
        "\u{02BF}" => "\u{0559}",
        "\u{2135}" => "\u{05D0}",
        "\u{FB21}" => "\u{05D0}",
        "\u{FB2F}" => "\u{FB2E}",
        "\u{FB30}" => "\u{FB2E}",
        "\u{FB4F}" => "\u{05D0}\u{05DC}",
        "\u{2136}" => "\u{05D1}",
        "\u{2137}" => "\u{05D2}",
        "\u{2138}" => "\u{05D3}",
        "\u{FB22}" => "\u{05D3}",
        "\u{FB23}" => "\u{05D4}",
        "\u{FB39}" => "\u{FB1D}",
        "\u{FB24}" => "\u{05DB}",
        "\u{FB25}" => "\u{05DC}",
        "\u{FB26}" => "\u{05DD}",
        "\u{FB20}" => "\u{05E2}",
        "\u{FB27}" => "\u{05E8}",
        "\u{FB2B}" => "\u{FB2A}",
        "\u{FB49}" => "\u{FB2A}",
        "\u{FB2D}" => "\u{FB2C}",
        "\u{FB28}" => "\u{05EA}",
        "\u{FE80}" => "\u{0621}",
        "\u{06FD}" => "\u{0621}\u{0348}",
        "\u{FE82}" => "\u{0622}",
        "\u{FE81}" => "\u{0622}",
        "\u{FB51}" => "\u{0671}",
        "\u{FB50}" => "\u{0671}",
        "\u{1EE01}" => "\u{0628}",
        "\u{1EE21}" => "\u{0628}",
        "\u{1EE61}" => "\u{0628}",
        "\u{1EE81}" => "\u{0628}",
        "\u{1EEA1}" => "\u{0628}",
        "\u{FE91}" => "\u{0628}",
        "\u{FE92}" => "\u{0628}",
        "\u{FE90}" => "\u{0628}",
        "\u{FE8F}" => "\u{0628}",
        "\u{0751}" => "\u{0628}\u{06DB}",
        "\u{08B6}" => "\u{0628}\u{06E2}",
        "\u{08A1}" => "\u{0628}\u{0654}",
        "\u{FCA0}" => "\u{0628}\u{006F}",
        "\u{FCE2}" => "\u{0628}\u{006F}",
        "\u{FC9C}" => "\u{0628}\u{062C}",
        "\u{FC05}" => "\u{0628}\u{062C}",
        "\u{FC9D}" => "\u{0628}\u{062D}",
        "\u{FC06}" => "\u{0628}\u{062D}",
        "\u{FDC2}" => "\u{0628}\u{062D}\u{0649}",
        "\u{FC9E}" => "\u{0628}\u{062E}",
        "\u{FC07}" => "\u{0628}\u{062E}",
        "\u{FCD2}" => "\u{0628}\u{062E}",
        "\u{FC4B}" => "\u{0628}\u{062E}",
        "\u{FD9E}" => "\u{0628}\u{062E}\u{0649}",
        "\u{FC6A}" => "\u{0628}\u{0631}",
        "\u{FC6B}" => "\u{0628}\u{0632}",
        "\u{FC9F}" => "\u{0628}\u{0645}",
        "\u{FCE1}" => "\u{0628}\u{0645}",
        "\u{FC6C}" => "\u{0628}\u{0645}",
        "\u{FC08}" => "\u{0628}\u{0645}",
        "\u{FC6D}" => "\u{0628}\u{0646}",
        "\u{FC6E}" => "\u{0628}\u{0649}",
        "\u{FC09}" => "\u{0628}\u{0649}",
        "\u{FC6F}" => "\u{0628}\u{0649}",
        "\u{FC0A}" => "\u{0628}\u{0649}",
        "\u{FB54}" => "\u{067B}",
        "\u{FB55}" => "\u{067B}",
        "\u{FB53}" => "\u{067B}",
        "\u{FB52}" => "\u{067B}",
        "\u{06D0}" => "\u{067B}",
        "\u{FBE6}" => "\u{067B}",
        "\u{FBE7}" => "\u{067B}",
        "\u{FBE5}" => "\u{067B}",
        "\u{FBE4}" => "\u{067B}",
        "\u{FB5C}" => "\u{0680}",
        "\u{FB5D}" => "\u{0680}",
        "\u{FB5B}" => "\u{0680}",
        "\u{FB5A}" => "\u{0680}",
        "\u{08A9}" => "\u{0754}",
        "\u{0767}" => "\u{0754}",
        "\u{2365}" => "\u{0629}",
        "\u{00F6}" => "\u{0629}",
        "\u{FE94}" => "\u{0629}",
        "\u{FE93}" => "\u{0629}",
        "\u{06C3}" => "\u{0629}",
        "\u{1EE15}" => "\u{062A}",
        "\u{1EE35}" => "\u{062A}",
        "\u{1EE75}" => "\u{062A}",
        "\u{1EE95}" => "\u{062A}",
        "\u{1EEB5}" => "\u{062A}",
        "\u{FE97}" => "\u{062A}",
        "\u{FE98}" => "\u{062A}",
        "\u{FE96}" => "\u{062A}",
        "\u{FE95}" => "\u{062A}",
        "\u{FCA5}" => "\u{062A}\u{006F}",
        "\u{FCE4}" => "\u{062A}\u{006F}",
        "\u{FCA1}" => "\u{062A}\u{062C}",
        "\u{FC0B}" => "\u{062A}\u{062C}",
        "\u{FD50}" => "\u{062A}\u{062C}\u{0645}",
        "\u{FDA0}" => "\u{062A}\u{062C}\u{0649}",
        "\u{FD9F}" => "\u{062A}\u{062C}\u{0649}",
        "\u{FCA2}" => "\u{062A}\u{062D}",
        "\u{FC0C}" => "\u{062A}\u{062D}",
        "\u{FD52}" => "\u{062A}\u{062D}\u{062C}",
        "\u{FD51}" => "\u{062A}\u{062D}\u{062C}",
        "\u{FD53}" => "\u{062A}\u{062D}\u{0645}",
        "\u{FCA3}" => "\u{062A}\u{062E}",
        "\u{FC0D}" => "\u{062A}\u{062E}",
        "\u{FD54}" => "\u{062A}\u{062E}\u{0645}",
        "\u{FDA2}" => "\u{062A}\u{062E}\u{0649}",
        "\u{FDA1}" => "\u{062A}\u{062E}\u{0649}",
        "\u{FC70}" => "\u{062A}\u{0631}",
        "\u{FC71}" => "\u{062A}\u{0632}",
        "\u{FCA4}" => "\u{062A}\u{0645}",
        "\u{FCE3}" => "\u{062A}\u{0645}",
        "\u{FC72}" => "\u{062A}\u{0645}",
        "\u{FC0E}" => "\u{062A}\u{0645}",
        "\u{FD55}" => "\u{062A}\u{0645}\u{062C}",
        "\u{FD56}" => "\u{062A}\u{0645}\u{062D}",
        "\u{FD57}" => "\u{062A}\u{0645}\u{062E}",
        "\u{FDA4}" => "\u{062A}\u{0645}\u{0649}",
        "\u{FDA3}" => "\u{062A}\u{0645}\u{0649}",
        "\u{FC73}" => "\u{062A}\u{0646}",
        "\u{FC74}" => "\u{062A}\u{0649}",
        "\u{FC0F}" => "\u{062A}\u{0649}",
        "\u{FC75}" => "\u{062A}\u{0649}",
        "\u{FC10}" => "\u{062A}\u{0649}",
        "\u{FB60}" => "\u{067A}",
        "\u{FB61}" => "\u{067A}",
        "\u{FB5F}" => "\u{067A}",
        "\u{FB5E}" => "\u{067A}",
        "\u{FB64}" => "\u{067F}",
        "\u{FB65}" => "\u{067F}",
        "\u{FB63}" => "\u{067F}",
        "\u{FB62}" => "\u{067F}",
        "\u{1EE02}" => "\u{062C}",
        "\u{1EE22}" => "\u{062C}",
        "\u{1EE42}" => "\u{062C}",
        "\u{1EE62}" => "\u{062C}",
        "\u{1EE82}" => "\u{062C}",
        "\u{1EEA2}" => "\u{062C}",
        "\u{FE9F}" => "\u{062C}",
        "\u{FEA0}" => "\u{062C}",
        "\u{FE9E}" => "\u{062C}",
        "\u{FE9D}" => "\u{062C}",
        "\u{FCA7}" => "\u{062C}\u{062D}",
        "\u{FC15}" => "\u{062C}\u{062D}",
        "\u{FDA6}" => "\u{062C}\u{062D}\u{0649}",
        "\u{FDBE}" => "\u{062C}\u{062D}\u{0649}",
        "\u{FDFB}" => "\u{062C}\u{0644}\u{0020}\u{062C}\u{0644}\u{006C}\u{0644}\u{006F}",
        "\u{FCA8}" => "\u{062C}\u{0645}",
        "\u{FC16}" => "\u{062C}\u{0645}",
        "\u{FD59}" => "\u{062C}\u{0645}\u{062D}",
        "\u{FD58}" => "\u{062C}\u{0645}\u{062D}",
        "\u{FDA7}" => "\u{062C}\u{0645}\u{0649}",
        "\u{FDA5}" => "\u{062C}\u{0645}\u{0649}",
        "\u{FD1D}" => "\u{062C}\u{0649}",
        "\u{FD01}" => "\u{062C}\u{0649}",
        "\u{FD1E}" => "\u{062C}\u{0649}",
        "\u{FD02}" => "\u{062C}\u{0649}",
        "\u{FB78}" => "\u{0683}",
        "\u{FB79}" => "\u{0683}",
        "\u{FB77}" => "\u{0683}",
        "\u{FB76}" => "\u{0683}",
        "\u{FB74}" => "\u{0684}",
        "\u{FB75}" => "\u{0684}",
        "\u{FB73}" => "\u{0684}",
        "\u{FB72}" => "\u{0684}",
        "\u{FB7C}" => "\u{0686}",
        "\u{FB7D}" => "\u{0686}",
        "\u{FB7B}" => "\u{0686}",
        "\u{FB7A}" => "\u{0686}",
        "\u{FB80}" => "\u{0687}",
        "\u{FB81}" => "\u{0687}",
        "\u{FB7F}" => "\u{0687}",
        "\u{FB7E}" => "\u{0687}",
        "\u{1EE07}" => "\u{062D}",
        "\u{1EE27}" => "\u{062D}",
        "\u{1EE47}" => "\u{062D}",
        "\u{1EE67}" => "\u{062D}",
        "\u{1EE87}" => "\u{062D}",
        "\u{1EEA7}" => "\u{062D}",
        "\u{FEA3}" => "\u{062D}",
        "\u{FEA4}" => "\u{062D}",
        "\u{FEA2}" => "\u{062D}",
        "\u{FEA1}" => "\u{062D}",
        "\u{0685}" => "\u{062D}\u{06DB}",
        "\u{0681}" => "\u{062D}\u{0654}",
        "\u{0772}" => "\u{062D}\u{0654}",
        "\u{FCA9}" => "\u{062D}\u{062C}",
        "\u{FC17}" => "\u{062D}\u{062C}",
        "\u{FDBF}" => "\u{062D}\u{062C}\u{0649}",
        "\u{FCAA}" => "\u{062D}\u{0645}",
        "\u{FC18}" => "\u{062D}\u{0645}",
        "\u{FD5B}" => "\u{062D}\u{0645}\u{0649}",
        "\u{FD5A}" => "\u{062D}\u{0645}\u{0649}",
        "\u{FD1B}" => "\u{062D}\u{0649}",
        "\u{FCFF}" => "\u{062D}\u{0649}",
        "\u{FD1C}" => "\u{062D}\u{0649}",
        "\u{FD00}" => "\u{062D}\u{0649}",
        "\u{1EE17}" => "\u{062E}",
        "\u{1EE37}" => "\u{062E}",
        "\u{1EE57}" => "\u{062E}",
        "\u{1EE77}" => "\u{062E}",
        "\u{1EE97}" => "\u{062E}",
        "\u{1EEB7}" => "\u{062E}",
        "\u{FEA7}" => "\u{062E}",
        "\u{FEA8}" => "\u{062E}",
        "\u{FEA6}" => "\u{062E}",
        "\u{FEA5}" => "\u{062E}",
        "\u{FCAB}" => "\u{062E}\u{062C}",
        "\u{FC19}" => "\u{062E}\u{062C}",
        "\u{FC1A}" => "\u{062E}\u{062D}",
        "\u{FCAC}" => "\u{062E}\u{0645}",
        "\u{FC1B}" => "\u{062E}\u{0645}",
        "\u{FD1F}" => "\u{062E}\u{0649}",
        "\u{FD03}" => "\u{062E}\u{0649}",
        "\u{FD20}" => "\u{062E}\u{0649}",
        "\u{FD04}" => "\u{062E}\u{0649}",
        "\u{102E1}" => "\u{062F}",
        "\u{1EE03}" => "\u{062F}",
        "\u{1EE83}" => "\u{062F}",
        "\u{1EEA3}" => "\u{062F}",
        "\u{FEAA}" => "\u{062F}",
        "\u{FEA9}" => "\u{062F}",
        "\u{0688}" => "\u{062F}\u{0615}",
        "\u{FB89}" => "\u{062F}\u{0615}",
        "\u{FB88}" => "\u{062F}\u{0615}",
        "\u{068E}" => "\u{062F}\u{06DB}",
        "\u{FB87}" => "\u{062F}\u{06DB}",
        "\u{FB86}" => "\u{062F}\u{06DB}",
        "\u{06EE}" => "\u{062F}\u{0302}",
        "\u{08AE}" => "\u{062F}\u{0324}\u{0323}",
        "\u{1EE18}" => "\u{0630}",
        "\u{1EE98}" => "\u{0630}",
        "\u{1EEB8}" => "\u{0630}",
        "\u{FEAC}" => "\u{0630}",
        "\u{FEAB}" => "\u{0630}",
        "\u{FC5B}" => "\u{0630}\u{0670}",
        "\u{068B}" => "\u{068A}\u{0615}",
        "\u{FB85}" => "\u{068C}",
        "\u{FB84}" => "\u{068C}",
        "\u{FB83}" => "\u{068D}",
        "\u{FB82}" => "\u{068D}",
        "\u{1EE13}" => "\u{0631}",
        "\u{1EE93}" => "\u{0631}",
        "\u{1EEB3}" => "\u{0631}",
        "\u{FEAE}" => "\u{0631}",
        "\u{FEAD}" => "\u{0631}",
        "\u{0691}" => "\u{0631}\u{0615}",
        "\u{FB8D}" => "\u{0631}\u{0615}",
        "\u{FB8C}" => "\u{0631}\u{0615}",
        "\u{0698}" => "\u{0631}\u{06DB}",
        "\u{FB8B}" => "\u{0631}\u{06DB}",
        "\u{FB8A}" => "\u{0631}\u{06DB}",
        "\u{0692}" => "\u{0631}\u{0306}",
        "\u{08B9}" => "\u{0631}\u{0306}\u{0307}",
        "\u{06EF}" => "\u{0631}\u{0302}",
        "\u{076C}" => "\u{0631}\u{0654}",
        "\u{FC5C}" => "\u{0631}\u{0670}",
        "\u{FDF6}" => "\u{0631}\u{0633}\u{0648}\u{0644}",
        "\u{FDFC}" => "\u{0631}\u{0649}\u{006C}\u{0644}",
        "\u{1EE06}" => "\u{0632}",
        "\u{1EE86}" => "\u{0632}",
        "\u{1EEA6}" => "\u{0632}",
        "\u{FEB0}" => "\u{0632}",
        "\u{FEAF}" => "\u{0632}",
        "\u{08B2}" => "\u{0632}\u{0302}",
        "\u{0771}" => "\u{0697}\u{0615}",
        "\u{1EE0E}" => "\u{0633}",
        "\u{1EE2E}" => "\u{0633}",
        "\u{1EE4E}" => "\u{0633}",
        "\u{1EE6E}" => "\u{0633}",
        "\u{1EE8E}" => "\u{0633}",
        "\u{1EEAE}" => "\u{0633}",
        "\u{FEB3}" => "\u{0633}",
        "\u{FEB4}" => "\u{0633}",
        "\u{FEB2}" => "\u{0633}",
        "\u{FEB1}" => "\u{0633}",
        "\u{0634}" => "\u{0633}\u{06DB}",
        "\u{1EE14}" => "\u{0633}\u{06DB}",
        "\u{1EE34}" => "\u{0633}\u{06DB}",
        "\u{1EE54}" => "\u{0633}\u{06DB}",
        "\u{1EE74}" => "\u{0633}\u{06DB}",
        "\u{1EE94}" => "\u{0633}\u{06DB}",
        "\u{1EEB4}" => "\u{0633}\u{06DB}",
        "\u{FEB7}" => "\u{0633}\u{06DB}",
        "\u{FEB8}" => "\u{0633}\u{06DB}",
        "\u{FEB6}" => "\u{0633}\u{06DB}",
        "\u{FEB5}" => "\u{0633}\u{06DB}",
        "\u{077E}" => "\u{0633}\u{0302}",
        "\u{FD31}" => "\u{0633}\u{006F}",
        "\u{FCE8}" => "\u{0633}\u{006F}",
        "\u{FD32}" => "\u{0633}\u{06DB}\u{006F}",
        "\u{FCEA}" => "\u{0633}\u{06DB}\u{006F}",
        "\u{FCAD}" => "\u{0633}\u{062C}",
        "\u{FD34}" => "\u{0633}\u{062C}",
        "\u{FC1C}" => "\u{0633}\u{062C}",
        "\u{FD2D}" => "\u{0633}\u{06DB}\u{062C}",
        "\u{FD37}" => "\u{0633}\u{06DB}\u{062C}",
        "\u{FD25}" => "\u{0633}\u{06DB}\u{062C}",
        "\u{FD09}" => "\u{0633}\u{06DB}\u{062C}",
        "\u{FD5D}" => "\u{0633}\u{062C}\u{062D}",
        "\u{FD5E}" => "\u{0633}\u{062C}\u{0649}",
        "\u{FD69}" => "\u{0633}\u{06DB}\u{062C}\u{0649}",
        "\u{FCAE}" => "\u{0633}\u{062D}",
        "\u{FD35}" => "\u{0633}\u{062D}",
        "\u{FC1D}" => "\u{0633}\u{062D}",
        "\u{FD2E}" => "\u{0633}\u{06DB}\u{062D}",
        "\u{FD38}" => "\u{0633}\u{06DB}\u{062D}",
        "\u{FD26}" => "\u{0633}\u{06DB}\u{062D}",
        "\u{FD0A}" => "\u{0633}\u{06DB}\u{062D}",
        "\u{FD5C}" => "\u{0633}\u{062D}\u{062C}",
        "\u{FD68}" => "\u{0633}\u{06DB}\u{062D}\u{0645}",
        "\u{FD67}" => "\u{0633}\u{06DB}\u{062D}\u{0645}",
        "\u{FDAA}" => "\u{0633}\u{06DB}\u{062D}\u{0649}",
        "\u{FCAF}" => "\u{0633}\u{062E}",
        "\u{FD36}" => "\u{0633}\u{062E}",
        "\u{FC1E}" => "\u{0633}\u{062E}",
        "\u{FD2F}" => "\u{0633}\u{06DB}\u{062E}",
        "\u{FD39}" => "\u{0633}\u{06DB}\u{062E}",
        "\u{FD27}" => "\u{0633}\u{06DB}\u{062E}",
        "\u{FD0B}" => "\u{0633}\u{06DB}\u{062E}",
        "\u{FDA8}" => "\u{0633}\u{062E}\u{0649}",
        "\u{FDC6}" => "\u{0633}\u{062E}\u{0649}",
        "\u{FD2A}" => "\u{0633}\u{0631}",
        "\u{FD0E}" => "\u{0633}\u{0631}",
        "\u{FD29}" => "\u{0633}\u{06DB}\u{0631}",
        "\u{FD0D}" => "\u{0633}\u{06DB}\u{0631}",
        "\u{FCB0}" => "\u{0633}\u{0645}",
        "\u{FCE7}" => "\u{0633}\u{0645}",
        "\u{FC1F}" => "\u{0633}\u{0645}",
        "\u{FD30}" => "\u{0633}\u{06DB}\u{0645}",
        "\u{FCE9}" => "\u{0633}\u{06DB}\u{0645}",
        "\u{FD28}" => "\u{0633}\u{06DB}\u{0645}",
        "\u{FD0C}" => "\u{0633}\u{06DB}\u{0645}",
        "\u{FD61}" => "\u{0633}\u{0645}\u{062C}",
        "\u{FD60}" => "\u{0633}\u{0645}\u{062D}",
        "\u{FD5F}" => "\u{0633}\u{0645}\u{062D}",
        "\u{FD6B}" => "\u{0633}\u{06DB}\u{0645}\u{062E}",
        "\u{FD6A}" => "\u{0633}\u{06DB}\u{0645}\u{062E}",
        "\u{FD63}" => "\u{0633}\u{0645}\u{0645}",
        "\u{FD62}" => "\u{0633}\u{0645}\u{0645}",
        "\u{FD6D}" => "\u{0633}\u{06DB}\u{0645}\u{0645}",
        "\u{FD6C}" => "\u{0633}\u{06DB}\u{0645}\u{0645}",
        "\u{FD17}" => "\u{0633}\u{0649}",
        "\u{FCFB}" => "\u{0633}\u{0649}",
        "\u{FD18}" => "\u{0633}\u{0649}",
        "\u{FCFC}" => "\u{0633}\u{0649}",
        "\u{FD19}" => "\u{0633}\u{06DB}\u{0649}",
        "\u{FCFD}" => "\u{0633}\u{06DB}\u{0649}",
        "\u{FD1A}" => "\u{0633}\u{06DB}\u{0649}",
        "\u{FCFE}" => "\u{0633}\u{06DB}\u{0649}",
        "\u{102F2}" => "\u{0635}",
        "\u{1EE11}" => "\u{0635}",
        "\u{1EE31}" => "\u{0635}",
        "\u{1EE51}" => "\u{0635}",
        "\u{1EE71}" => "\u{0635}",
        "\u{1EE91}" => "\u{0635}",
        "\u{1EEB1}" => "\u{0635}",
        "\u{FEBB}" => "\u{0635}",
        "\u{FEBC}" => "\u{0635}",
        "\u{FEBA}" => "\u{0635}",
        "\u{FEB9}" => "\u{0635}",
        "\u{069E}" => "\u{0635}\u{06DB}",
        "\u{08AF}" => "\u{0635}\u{0324}\u{0323}",
        "\u{FCB1}" => "\u{0635}\u{062D}",
        "\u{FC20}" => "\u{0635}\u{062D}",
        "\u{FD65}" => "\u{0635}\u{062D}\u{062D}",
        "\u{FD64}" => "\u{0635}\u{062D}\u{062D}",
        "\u{FDA9}" => "\u{0635}\u{062D}\u{0649}",
        "\u{FCB2}" => "\u{0635}\u{062E}",
        "\u{FD2B}" => "\u{0635}\u{0631}",
        "\u{FD0F}" => "\u{0635}\u{0631}",
        "\u{FDF5}" => "\u{0635}\u{0644}\u{0639}\u{0645}",
        "\u{FDF9}" => "\u{0635}\u{0644}\u{0649}",
        "\u{FDF0}" => "\u{0635}\u{0644}\u{0649}",
        "\u{FDFA}" => "\u{0635}\u{0644}\u{0649}\u{0020}\u{006C}\u{0644}\u{0644}\u{006F}\u{0020}\u{0639}\u{0644}\u{0649}\u{006F}\u{0020}\u{0648}\u{0633}\u{0644}\u{0645}",
        "\u{FCB3}" => "\u{0635}\u{0645}",
        "\u{FC21}" => "\u{0635}\u{0645}",
        "\u{FDC5}" => "\u{0635}\u{0645}\u{0645}",
        "\u{FD66}" => "\u{0635}\u{0645}\u{0645}",
        "\u{FD21}" => "\u{0635}\u{0649}",
        "\u{FD05}" => "\u{0635}\u{0649}",
        "\u{FD22}" => "\u{0635}\u{0649}",
        "\u{FD06}" => "\u{0635}\u{0649}",
        "\u{1EE19}" => "\u{0636}",
        "\u{1EE39}" => "\u{0636}",
        "\u{1EE59}" => "\u{0636}",
        "\u{1EE79}" => "\u{0636}",
        "\u{1EE99}" => "\u{0636}",
        "\u{1EEB9}" => "\u{0636}",
        "\u{FEBF}" => "\u{0636}",
        "\u{FEC0}" => "\u{0636}",
        "\u{FEBE}" => "\u{0636}",
        "\u{FEBD}" => "\u{0636}",
        "\u{FCB4}" => "\u{0636}\u{062C}",
        "\u{FC22}" => "\u{0636}\u{062C}",
        "\u{FCB5}" => "\u{0636}\u{062D}",
        "\u{FC23}" => "\u{0636}\u{062D}",
        "\u{FD6E}" => "\u{0636}\u{062D}\u{0649}",
        "\u{FDAB}" => "\u{0636}\u{062D}\u{0649}",
        "\u{FCB6}" => "\u{0636}\u{062E}",
        "\u{FC24}" => "\u{0636}\u{062E}",
        "\u{FD70}" => "\u{0636}\u{062E}\u{0645}",
        "\u{FD6F}" => "\u{0636}\u{062E}\u{0645}",
        "\u{FD2C}" => "\u{0636}\u{0631}",
        "\u{FD10}" => "\u{0636}\u{0631}",
        "\u{FCB7}" => "\u{0636}\u{0645}",
        "\u{FC25}" => "\u{0636}\u{0645}",
        "\u{FD23}" => "\u{0636}\u{0649}",
        "\u{FD07}" => "\u{0636}\u{0649}",
        "\u{FD24}" => "\u{0636}\u{0649}",
        "\u{FD08}" => "\u{0636}\u{0649}",
        "\u{102E8}" => "\u{0637}",
        "\u{1EE08}" => "\u{0637}",
        "\u{1EE68}" => "\u{0637}",
        "\u{1EE88}" => "\u{0637}",
        "\u{1EEA8}" => "\u{0637}",
        "\u{FEC3}" => "\u{0637}",
        "\u{FEC4}" => "\u{0637}",
        "\u{FEC2}" => "\u{0637}",
        "\u{FEC1}" => "\u{0637}",
        "\u{069F}" => "\u{0637}\u{06DB}",
        "\u{FCB8}" => "\u{0637}\u{062D}",
        "\u{FC26}" => "\u{0637}\u{062D}",
        "\u{FD33}" => "\u{0637}\u{0645}",
        "\u{FD3A}" => "\u{0637}\u{0645}",
        "\u{FC27}" => "\u{0637}\u{0645}",
        "\u{FD72}" => "\u{0637}\u{0645}\u{062D}",
        "\u{FD71}" => "\u{0637}\u{0645}\u{062D}",
        "\u{FD73}" => "\u{0637}\u{0645}\u{0645}",
        "\u{FD74}" => "\u{0637}\u{0645}\u{0649}",
        "\u{FD11}" => "\u{0637}\u{0649}",
        "\u{FCF5}" => "\u{0637}\u{0649}",
        "\u{FD12}" => "\u{0637}\u{0649}",
        "\u{FCF6}" => "\u{0637}\u{0649}",
        "\u{1EE1A}" => "\u{0638}",
        "\u{1EE7A}" => "\u{0638}",
        "\u{1EE9A}" => "\u{0638}",
        "\u{1EEBA}" => "\u{0638}",
        "\u{FEC7}" => "\u{0638}",
        "\u{FEC8}" => "\u{0638}",
        "\u{FEC6}" => "\u{0638}",
        "\u{FEC5}" => "\u{0638}",
        "\u{FCB9}" => "\u{0638}\u{0645}",
        "\u{FD3B}" => "\u{0638}\u{0645}",
        "\u{FC28}" => "\u{0638}\u{0645}",
        "\u{060F}" => "\u{0639}",
        "\u{1EE0F}" => "\u{0639}",
        "\u{1EE2F}" => "\u{0639}",
        "\u{1EE4F}" => "\u{0639}",
        "\u{1EE6F}" => "\u{0639}",
        "\u{1EE8F}" => "\u{0639}",
        "\u{1EEAF}" => "\u{0639}",
        "\u{FECB}" => "\u{0639}",
        "\u{FECC}" => "\u{0639}",
        "\u{FECA}" => "\u{0639}",
        "\u{FEC9}" => "\u{0639}",
        "\u{FCBA}" => "\u{0639}\u{062C}",
        "\u{FC29}" => "\u{0639}\u{062C}",
        "\u{FDC4}" => "\u{0639}\u{062C}\u{0645}",
        "\u{FD75}" => "\u{0639}\u{062C}\u{0645}",
        "\u{FDF7}" => "\u{0639}\u{0644}\u{0649}\u{006F}",
        "\u{FCBB}" => "\u{0639}\u{0645}",
        "\u{FC2A}" => "\u{0639}\u{0645}",
        "\u{FD77}" => "\u{0639}\u{0645}\u{0645}",
        "\u{FD76}" => "\u{0639}\u{0645}\u{0645}",
        "\u{FD78}" => "\u{0639}\u{0645}\u{0649}",
        "\u{FDB6}" => "\u{0639}\u{0645}\u{0649}",
        "\u{FD13}" => "\u{0639}\u{0649}",
        "\u{FCF7}" => "\u{0639}\u{0649}",
        "\u{FD14}" => "\u{0639}\u{0649}",
        "\u{FCF8}" => "\u{0639}\u{0649}",
        "\u{1EE1B}" => "\u{063A}",
        "\u{1EE3B}" => "\u{063A}",
        "\u{1EE5B}" => "\u{063A}",
        "\u{1EE7B}" => "\u{063A}",
        "\u{1EE9B}" => "\u{063A}",
        "\u{1EEBB}" => "\u{063A}",
        "\u{FECF}" => "\u{063A}",
        "\u{FED0}" => "\u{063A}",
        "\u{FECE}" => "\u{063A}",
        "\u{FECD}" => "\u{063A}",
        "\u{FCBC}" => "\u{063A}\u{062C}",
        "\u{FC2B}" => "\u{063A}\u{062C}",
        "\u{FCBD}" => "\u{063A}\u{0645}",
        "\u{FC2C}" => "\u{063A}\u{0645}",
        "\u{FD79}" => "\u{063A}\u{0645}\u{0645}",
        "\u{FD7B}" => "\u{063A}\u{0645}\u{0649}",
        "\u{FD7A}" => "\u{063A}\u{0645}\u{0649}",
        "\u{FD15}" => "\u{063A}\u{0649}",
        "\u{FCF9}" => "\u{063A}\u{0649}",
        "\u{FD16}" => "\u{063A}\u{0649}",
        "\u{FCFA}" => "\u{063A}\u{0649}",
        "\u{1EE10}" => "\u{0641}",
        "\u{1EE30}" => "\u{0641}",
        "\u{1EE70}" => "\u{0641}",
        "\u{1EE90}" => "\u{0641}",
        "\u{1EEB0}" => "\u{0641}",
        "\u{FED3}" => "\u{0641}",
        "\u{FED4}" => "\u{0641}",
        "\u{FED2}" => "\u{0641}",
        "\u{FED1}" => "\u{0641}",
        "\u{06A7}" => "\u{0641}",
        "\u{FCBE}" => "\u{0641}\u{062C}",
        "\u{FC2D}" => "\u{0641}\u{062C}",
        "\u{FCBF}" => "\u{0641}\u{062D}",
        "\u{FC2E}" => "\u{0641}\u{062D}",
        "\u{FCC0}" => "\u{0641}\u{062E}",
        "\u{FC2F}" => "\u{0641}\u{062E}",
        "\u{FD7D}" => "\u{0641}\u{062E}\u{0645}",
        "\u{FD7C}" => "\u{0641}\u{062E}\u{0645}",
        "\u{FCC1}" => "\u{0641}\u{0645}",
        "\u{FC30}" => "\u{0641}\u{0645}",
        "\u{FDC1}" => "\u{0641}\u{0645}\u{0649}",
        "\u{FC7C}" => "\u{0641}\u{0649}",
        "\u{FC31}" => "\u{0641}\u{0649}",
        "\u{FC7D}" => "\u{0641}\u{0649}",
        "\u{FC32}" => "\u{0641}\u{0649}",
        "\u{1EE1E}" => "\u{06A1}",
        "\u{1EE7E}" => "\u{06A1}",
        "\u{08BB}" => "\u{06A1}",
        "\u{066F}" => "\u{06A1}",
        "\u{1EE1F}" => "\u{06A1}",
        "\u{1EE5F}" => "\u{06A1}",
        "\u{08BC}" => "\u{06A1}",
        "\u{06A4}" => "\u{06A1}\u{06DB}",
        "\u{FB6C}" => "\u{06A1}\u{06DB}",
        "\u{FB6D}" => "\u{06A1}\u{06DB}",
        "\u{FB6B}" => "\u{06A1}\u{06DB}",
        "\u{FB6A}" => "\u{06A1}\u{06DB}",
        "\u{06A8}" => "\u{06A1}\u{06DB}",
        "\u{08A4}" => "\u{06A2}\u{06DB}",
        "\u{FB70}" => "\u{06A6}",
        "\u{FB71}" => "\u{06A6}",
        "\u{FB6F}" => "\u{06A6}",
        "\u{FB6E}" => "\u{06A6}",
        "\u{1EE12}" => "\u{0642}",
        "\u{1EE32}" => "\u{0642}",
        "\u{1EE52}" => "\u{0642}",
        "\u{1EE72}" => "\u{0642}",
        "\u{1EE92}" => "\u{0642}",
        "\u{1EEB2}" => "\u{0642}",
        "\u{FED7}" => "\u{0642}",
        "\u{FED8}" => "\u{0642}",
        "\u{FED6}" => "\u{0642}",
        "\u{FED5}" => "\u{0642}",
        "\u{FCC2}" => "\u{0642}\u{062D}",
        "\u{FC33}" => "\u{0642}\u{062D}",
        "\u{FDF1}" => "\u{0642}\u{0644}\u{0649}",
        "\u{FCC3}" => "\u{0642}\u{0645}",
        "\u{FC34}" => "\u{0642}\u{0645}",
        "\u{FDB4}" => "\u{0642}\u{0645}\u{062D}",
        "\u{FD7E}" => "\u{0642}\u{0645}\u{062D}",
        "\u{FD7F}" => "\u{0642}\u{0645}\u{0645}",
        "\u{FDB2}" => "\u{0642}\u{0645}\u{0649}",
        "\u{FC7E}" => "\u{0642}\u{0649}",
        "\u{FC35}" => "\u{0642}\u{0649}",
        "\u{FC7F}" => "\u{0642}\u{0649}",
        "\u{FC36}" => "\u{0642}\u{0649}",
        "\u{1EE0A}" => "\u{0643}",
        "\u{1EE2A}" => "\u{0643}",
        "\u{1EE6A}" => "\u{0643}",
        "\u{FEDB}" => "\u{0643}",
        "\u{FEDC}" => "\u{0643}",
        "\u{FEDA}" => "\u{0643}",
        "\u{FED9}" => "\u{0643}",
        "\u{06A9}" => "\u{0643}",
        "\u{FB90}" => "\u{0643}",
        "\u{FB91}" => "\u{0643}",
        "\u{FB8F}" => "\u{0643}",
        "\u{FB8E}" => "\u{0643}",
        "\u{06AA}" => "\u{0643}",
        "\u{06AD}" => "\u{0643}\u{06DB}",
        "\u{FBD5}" => "\u{0643}\u{06DB}",
        "\u{FBD6}" => "\u{0643}\u{06DB}",
        "\u{FBD4}" => "\u{0643}\u{06DB}",
        "\u{FBD3}" => "\u{0643}\u{06DB}",
        "\u{0763}" => "\u{0643}\u{06DB}",
        "\u{FC80}" => "\u{0643}\u{006C}",
        "\u{FC37}" => "\u{0643}\u{006C}",
        "\u{FCC4}" => "\u{0643}\u{062C}",
        "\u{FC38}" => "\u{0643}\u{062C}",
        "\u{FCC5}" => "\u{0643}\u{062D}",
        "\u{FC39}" => "\u{0643}\u{062D}",
        "\u{FCC6}" => "\u{0643}\u{062E}",
        "\u{FC3A}" => "\u{0643}\u{062E}",
        "\u{FCC7}" => "\u{0643}\u{0644}",
        "\u{FCEB}" => "\u{0643}\u{0644}",
        "\u{FC81}" => "\u{0643}\u{0644}",
        "\u{FC3B}" => "\u{0643}\u{0644}",
        "\u{FCC8}" => "\u{0643}\u{0645}",
        "\u{FCEC}" => "\u{0643}\u{0645}",
        "\u{FC82}" => "\u{0643}\u{0645}",
        "\u{FC3C}" => "\u{0643}\u{0645}",
        "\u{FDC3}" => "\u{0643}\u{0645}\u{0645}",
        "\u{FDBB}" => "\u{0643}\u{0645}\u{0645}",
        "\u{FDB7}" => "\u{0643}\u{0645}\u{0649}",
        "\u{FC83}" => "\u{0643}\u{0649}",
        "\u{FC3D}" => "\u{0643}\u{0649}",
        "\u{FC84}" => "\u{0643}\u{0649}",
        "\u{FC3E}" => "\u{0643}\u{0649}",
        "\u{0762}" => "\u{06AC}",
        "\u{FB94}" => "\u{06AF}",
        "\u{FB95}" => "\u{06AF}",
        "\u{FB93}" => "\u{06AF}",
        "\u{FB92}" => "\u{06AF}",
        "\u{08B0}" => "\u{06AF}",
        "\u{06B4}" => "\u{06AF}\u{06DB}",
        "\u{FB9C}" => "\u{06B1}",
        "\u{FB9D}" => "\u{06B1}",
        "\u{FB9B}" => "\u{06B1}",
        "\u{FB9A}" => "\u{06B1}",
        "\u{FB98}" => "\u{06B3}",
        "\u{FB99}" => "\u{06B3}",
        "\u{FB97}" => "\u{06B3}",
        "\u{FB96}" => "\u{06B3}",
        "\u{1EE0B}" => "\u{0644}",
        "\u{1EE2B}" => "\u{0644}",
        "\u{1EE4B}" => "\u{0644}",
        "\u{1EE8B}" => "\u{0644}",
        "\u{1EEAB}" => "\u{0644}",
        "\u{FEDF}" => "\u{0644}",
        "\u{FEE0}" => "\u{0644}",
        "\u{FEDE}" => "\u{0644}",
        "\u{FEDD}" => "\u{0644}",
        "\u{06B7}" => "\u{0644}\u{06DB}",
        "\u{06B5}" => "\u{0644}\u{0306}",
        "\u{FEFC}" => "\u{0644}\u{006C}",
        "\u{FEFB}" => "\u{0644}\u{006C}",
        "\u{FEFA}" => "\u{0644}\u{006C}\u{0655}",
        "\u{FEF9}" => "\u{0644}\u{006C}\u{0655}",
        "\u{FEF8}" => "\u{0644}\u{006C}\u{0674}",
        "\u{FEF7}" => "\u{0644}\u{006C}\u{0674}",
        "\u{FCCD}" => "\u{0644}\u{006F}",
        "\u{FEF6}" => "\u{0644}\u{0622}",
        "\u{FEF5}" => "\u{0644}\u{0622}",
        "\u{FCC9}" => "\u{0644}\u{062C}",
        "\u{FC3F}" => "\u{0644}\u{062C}",
        "\u{FD83}" => "\u{0644}\u{062C}\u{062C}",
        "\u{FD84}" => "\u{0644}\u{062C}\u{062C}",
        "\u{FDBA}" => "\u{0644}\u{062C}\u{0645}",
        "\u{FDBC}" => "\u{0644}\u{062C}\u{0645}",
        "\u{FDAC}" => "\u{0644}\u{062C}\u{0649}",
        "\u{FCCA}" => "\u{0644}\u{062D}",
        "\u{FC40}" => "\u{0644}\u{062D}",
        "\u{FDB5}" => "\u{0644}\u{062D}\u{0645}",
        "\u{FD80}" => "\u{0644}\u{062D}\u{0645}",
        "\u{FD82}" => "\u{0644}\u{062D}\u{0649}",
        "\u{FD81}" => "\u{0644}\u{062D}\u{0649}",
        "\u{FCCB}" => "\u{0644}\u{062E}",
        "\u{FC41}" => "\u{0644}\u{062E}",
        "\u{FD86}" => "\u{0644}\u{062E}\u{0645}",
        "\u{FD85}" => "\u{0644}\u{062E}\u{0645}",
        "\u{FCCC}" => "\u{0644}\u{0645}",
        "\u{FCED}" => "\u{0644}\u{0645}",
        "\u{FC85}" => "\u{0644}\u{0645}",
        "\u{FC42}" => "\u{0644}\u{0645}",
        "\u{FD88}" => "\u{0644}\u{0645}\u{062D}",
        "\u{FD87}" => "\u{0644}\u{0645}\u{062D}",
        "\u{FDAD}" => "\u{0644}\u{0645}\u{0649}",
        "\u{FC86}" => "\u{0644}\u{0649}",
        "\u{FC43}" => "\u{0644}\u{0649}",
        "\u{FC87}" => "\u{0644}\u{0649}",
        "\u{FC44}" => "\u{0644}\u{0649}",
        "\u{1EE0C}" => "\u{0645}",
        "\u{1EE2C}" => "\u{0645}",
        "\u{1EE6C}" => "\u{0645}",
        "\u{1EE8C}" => "\u{0645}",
        "\u{1EEAC}" => "\u{0645}",
        "\u{FEE3}" => "\u{0645}",
        "\u{FEE4}" => "\u{0645}",
        "\u{FEE2}" => "\u{0645}",
        "\u{FEE1}" => "\u{0645}",
        "\u{08A7}" => "\u{0645}\u{06DB}",
        "\u{06FE}" => "\u{0645}\u{0348}",
        "\u{FC88}" => "\u{0645}\u{006C}",
        "\u{FCCE}" => "\u{0645}\u{062C}",
        "\u{FC45}" => "\u{0645}\u{062C}",
        "\u{FD8C}" => "\u{0645}\u{062C}\u{062D}",
        "\u{FD92}" => "\u{0645}\u{062C}\u{062E}",
        "\u{FD8D}" => "\u{0645}\u{062C}\u{0645}",
        "\u{FDC0}" => "\u{0645}\u{062C}\u{0649}",
        "\u{FCCF}" => "\u{0645}\u{062D}",
        "\u{FC46}" => "\u{0645}\u{062D}",
        "\u{FD89}" => "\u{0645}\u{062D}\u{062C}",
        "\u{FD8A}" => "\u{0645}\u{062D}\u{0645}",
        "\u{FDF4}" => "\u{0645}\u{062D}\u{0645}\u{062F}",
        "\u{FD8B}" => "\u{0645}\u{062D}\u{0649}",
        "\u{FCD0}" => "\u{0645}\u{062E}",
        "\u{FC47}" => "\u{0645}\u{062E}",
        "\u{FD8E}" => "\u{0645}\u{062E}\u{062C}",
        "\u{FD8F}" => "\u{0645}\u{062E}\u{0645}",
        "\u{FDB9}" => "\u{0645}\u{062E}\u{0649}",
        "\u{FCD1}" => "\u{0645}\u{0645}",
        "\u{FC89}" => "\u{0645}\u{0645}",
        "\u{FC48}" => "\u{0645}\u{0645}",
        "\u{FDB1}" => "\u{0645}\u{0645}\u{0649}",
        "\u{FC49}" => "\u{0645}\u{0649}",
        "\u{FC4A}" => "\u{0645}\u{0649}",
        "\u{1EE0D}" => "\u{0646}",
        "\u{1EE2D}" => "\u{0646}",
        "\u{1EE4D}" => "\u{0646}",
        "\u{1EE6D}" => "\u{0646}",
        "\u{1EE8D}" => "\u{0646}",
        "\u{1EEAD}" => "\u{0646}",
        "\u{FEE7}" => "\u{0646}",
        "\u{FEE8}" => "\u{0646}",
        "\u{FEE6}" => "\u{0646}",
        "\u{FEE5}" => "\u{0646}",
        "\u{0768}" => "\u{0646}\u{0615}",
        "\u{0769}" => "\u{0646}\u{0306}",
        "\u{FCD6}" => "\u{0646}\u{006F}",
        "\u{FCEF}" => "\u{0646}\u{006F}",
        "\u{FDB8}" => "\u{0646}\u{062C}\u{062D}",
        "\u{FDBD}" => "\u{0646}\u{062C}\u{062D}",
        "\u{FD98}" => "\u{0646}\u{062C}\u{0645}",
        "\u{FD97}" => "\u{0646}\u{062C}\u{0645}",
        "\u{FD99}" => "\u{0646}\u{062C}\u{0649}",
        "\u{FDC7}" => "\u{0646}\u{062C}\u{0649}",
        "\u{FCD3}" => "\u{0646}\u{062D}",
        "\u{FC4C}" => "\u{0646}\u{062D}",
        "\u{FD95}" => "\u{0646}\u{062D}\u{0645}",
        "\u{FD96}" => "\u{0646}\u{062D}\u{0649}",
        "\u{FDB3}" => "\u{0646}\u{062D}\u{0649}",
        "\u{FCD4}" => "\u{0646}\u{062E}",
        "\u{FC4D}" => "\u{0646}\u{062E}",
        "\u{FC8A}" => "\u{0646}\u{0631}",
        "\u{FC8B}" => "\u{0646}\u{0632}",
        "\u{FCD5}" => "\u{0646}\u{0645}",
        "\u{FCEE}" => "\u{0646}\u{0645}",
        "\u{FC8C}" => "\u{0646}\u{0645}",
        "\u{FC4E}" => "\u{0646}\u{0645}",
        "\u{FD9B}" => "\u{0646}\u{0645}\u{0649}",
        "\u{FD9A}" => "\u{0646}\u{0645}\u{0649}",
        "\u{FC8D}" => "\u{0646}\u{0646}",
        "\u{FC8E}" => "\u{0646}\u{0649}",
        "\u{FC4F}" => "\u{0646}\u{0649}",
        "\u{FC8F}" => "\u{0646}\u{0649}",
        "\u{FC50}" => "\u{0646}\u{0649}",
        "\u{06C2}" => "\u{06C0}",
        "\u{FBA5}" => "\u{06C0}",
        "\u{FBA4}" => "\u{06C0}",
        "\u{102E4}" => "\u{0648}",
        "\u{1EE05}" => "\u{0648}",
        "\u{1EE85}" => "\u{0648}",
        "\u{1EEA5}" => "\u{0648}",
        "\u{FEEE}" => "\u{0648}",
        "\u{FEED}" => "\u{0648}",
        "\u{08B1}" => "\u{0648}",
        "\u{06CB}" => "\u{0648}\u{06DB}",
        "\u{FBDF}" => "\u{0648}\u{06DB}",
        "\u{FBDE}" => "\u{0648}\u{06DB}",
        "\u{06C7}" => "\u{0648}\u{0313}",
        "\u{FBD8}" => "\u{0648}\u{0313}",
        "\u{FBD7}" => "\u{0648}\u{0313}",
        "\u{06C6}" => "\u{0648}\u{0306}",
        "\u{FBDA}" => "\u{0648}\u{0306}",
        "\u{FBD9}" => "\u{0648}\u{0306}",
        "\u{06C9}" => "\u{0648}\u{0302}",
        "\u{FBE3}" => "\u{0648}\u{0302}",
        "\u{FBE2}" => "\u{0648}\u{0302}",
        "\u{06C8}" => "\u{0648}\u{0670}",
        "\u{FBDC}" => "\u{0648}\u{0670}",
        "\u{FBDB}" => "\u{0648}\u{0670}",
        "\u{0624}" => "\u{0648}\u{0674}",
        "\u{FE86}" => "\u{0648}\u{0674}",
        "\u{FE85}" => "\u{0648}\u{0674}",
        "\u{0676}" => "\u{0648}\u{0674}",
        "\u{0677}" => "\u{0648}\u{0313}\u{0674}",
        "\u{FBDD}" => "\u{0648}\u{0313}\u{0674}",
        "\u{FDF8}" => "\u{0648}\u{0633}\u{0644}\u{0645}",
        "\u{FBE1}" => "\u{06C5}",
        "\u{FBE0}" => "\u{06C5}",
        "\u{066E}" => "\u{0649}",
        "\u{1EE1C}" => "\u{0649}",
        "\u{1EE7C}" => "\u{0649}",
        "\u{06BA}" => "\u{0649}",
        "\u{1EE1D}" => "\u{0649}",
        "\u{1EE5D}" => "\u{0649}",
        "\u{FB9F}" => "\u{0649}",
        "\u{FB9E}" => "\u{0649}",
        "\u{08BD}" => "\u{0649}",
        "\u{FBE8}" => "\u{0649}",
        "\u{FBE9}" => "\u{0649}",
        "\u{FEF0}" => "\u{0649}",
        "\u{FEEF}" => "\u{0649}",
        "\u{064A}" => "\u{0649}",
        "\u{1EE09}" => "\u{0649}",
        "\u{1EE29}" => "\u{0649}",
        "\u{1EE49}" => "\u{0649}",
        "\u{1EE69}" => "\u{0649}",
        "\u{1EE89}" => "\u{0649}",
        "\u{1EEA9}" => "\u{0649}",
        "\u{FEF3}" => "\u{0649}",
        "\u{FEF4}" => "\u{0649}",
        "\u{FEF2}" => "\u{0649}",
        "\u{FEF1}" => "\u{0649}",
        "\u{06CC}" => "\u{0649}",
        "\u{FBFE}" => "\u{0649}",
        "\u{FBFF}" => "\u{0649}",
        "\u{FBFD}" => "\u{0649}",
        "\u{FBFC}" => "\u{0649}",
        "\u{06D2}" => "\u{0649}",
        "\u{FBAF}" => "\u{0649}",
        "\u{FBAE}" => "\u{0649}",
        "\u{0679}" => "\u{0649}\u{0615}",
        "\u{FB68}" => "\u{0649}\u{0615}",
        "\u{FB69}" => "\u{0649}\u{0615}",
        "\u{FB67}" => "\u{0649}\u{0615}",
        "\u{FB66}" => "\u{0649}\u{0615}",
        "\u{06BB}" => "\u{0649}\u{0615}",
        "\u{FBA2}" => "\u{0649}\u{0615}",
        "\u{FBA3}" => "\u{0649}\u{0615}",
        "\u{FBA1}" => "\u{0649}\u{0615}",
        "\u{FBA0}" => "\u{0649}\u{0615}",
        "\u{067E}" => "\u{0649}\u{06DB}",
        "\u{FB58}" => "\u{0649}\u{06DB}",
        "\u{FB59}" => "\u{0649}\u{06DB}",
        "\u{FB57}" => "\u{0649}\u{06DB}",
        "\u{FB56}" => "\u{0649}\u{06DB}",
        "\u{062B}" => "\u{0649}\u{06DB}",
        "\u{1EE16}" => "\u{0649}\u{06DB}",
        "\u{1EE36}" => "\u{0649}\u{06DB}",
        "\u{1EE76}" => "\u{0649}\u{06DB}",
        "\u{1EE96}" => "\u{0649}\u{06DB}",
        "\u{1EEB6}" => "\u{0649}\u{06DB}",
        "\u{FE9B}" => "\u{0649}\u{06DB}",
        "\u{FE9C}" => "\u{0649}\u{06DB}",
        "\u{FE9A}" => "\u{0649}\u{06DB}",
        "\u{FE99}" => "\u{0649}\u{06DB}",
        "\u{06BD}" => "\u{0649}\u{06DB}",
        "\u{06D1}" => "\u{0649}\u{06DB}",
        "\u{063F}" => "\u{0649}\u{06DB}",
        "\u{08B7}" => "\u{0649}\u{06DB}\u{06E2}",
        "\u{0756}" => "\u{0649}\u{0306}",
        "\u{06CE}" => "\u{0649}\u{0306}",
        "\u{08BA}" => "\u{0649}\u{0306}\u{0307}",
        "\u{063D}" => "\u{0649}\u{0302}",
        "\u{08A8}" => "\u{0649}\u{0654}",
        "\u{FC90}" => "\u{0649}\u{0670}",
        "\u{FC5D}" => "\u{0649}\u{0670}",
        "\u{FCDE}" => "\u{0649}\u{006F}",
        "\u{FCF1}" => "\u{0649}\u{006F}",
        "\u{FCE6}" => "\u{0649}\u{06DB}\u{006F}",
        "\u{0626}" => "\u{0649}\u{0674}",
        "\u{FE8B}" => "\u{0649}\u{0674}",
        "\u{FE8C}" => "\u{0649}\u{0674}",
        "\u{FE8A}" => "\u{0649}\u{0674}",
        "\u{FE89}" => "\u{0649}\u{0674}",
        "\u{0678}" => "\u{0649}\u{0674}",
        "\u{FBEB}" => "\u{0649}\u{0674}\u{006C}",
        "\u{FBEA}" => "\u{0649}\u{0674}\u{006C}",
        "\u{FC9B}" => "\u{0649}\u{0674}\u{006F}",
        "\u{FCE0}" => "\u{0649}\u{0674}\u{006F}",
        "\u{FBED}" => "\u{0649}\u{0674}\u{006F}",
        "\u{FBEC}" => "\u{0649}\u{0674}\u{006F}",
        "\u{FBF8}" => "\u{0649}\u{0674}\u{067B}",
        "\u{FBF7}" => "\u{0649}\u{0674}\u{067B}",
        "\u{FBF6}" => "\u{0649}\u{0674}\u{067B}",
        "\u{FC97}" => "\u{0649}\u{0674}\u{062C}",
        "\u{FC00}" => "\u{0649}\u{0674}\u{062C}",
        "\u{FC98}" => "\u{0649}\u{0674}\u{062D}",
        "\u{FC01}" => "\u{0649}\u{0674}\u{062D}",
        "\u{FC99}" => "\u{0649}\u{0674}\u{062E}",
        "\u{FC64}" => "\u{0649}\u{0674}\u{0631}",
        "\u{FC65}" => "\u{0649}\u{0674}\u{0632}",
        "\u{FC9A}" => "\u{0649}\u{0674}\u{0645}",
        "\u{FCDF}" => "\u{0649}\u{0674}\u{0645}",
        "\u{FC66}" => "\u{0649}\u{0674}\u{0645}",
        "\u{FC02}" => "\u{0649}\u{0674}\u{0645}",
        "\u{FC67}" => "\u{0649}\u{0674}\u{0646}",
        "\u{FBEF}" => "\u{0649}\u{0674}\u{0648}",
        "\u{FBEE}" => "\u{0649}\u{0674}\u{0648}",
        "\u{FBF1}" => "\u{0649}\u{0674}\u{0648}\u{0313}",
        "\u{FBF0}" => "\u{0649}\u{0674}\u{0648}\u{0313}",
        "\u{FBF3}" => "\u{0649}\u{0674}\u{0648}\u{0306}",
        "\u{FBF2}" => "\u{0649}\u{0674}\u{0648}\u{0306}",
        "\u{FBF5}" => "\u{0649}\u{0674}\u{0648}\u{0670}",
        "\u{FBF4}" => "\u{0649}\u{0674}\u{0648}\u{0670}",
        "\u{FBFB}" => "\u{0649}\u{0674}\u{0649}",
        "\u{FBFA}" => "\u{0649}\u{0674}\u{0649}",
        "\u{FC68}" => "\u{0649}\u{0674}\u{0649}",
        "\u{FBF9}" => "\u{0649}\u{0674}\u{0649}",
        "\u{FC03}" => "\u{0649}\u{0674}\u{0649}",
        "\u{FC69}" => "\u{0649}\u{0674}\u{0649}",
        "\u{FC04}" => "\u{0649}\u{0674}\u{0649}",
        "\u{FCDA}" => "\u{0649}\u{062C}",
        "\u{FC55}" => "\u{0649}\u{062C}",
        "\u{FC11}" => "\u{0649}\u{06DB}\u{062C}",
        "\u{FDAF}" => "\u{0649}\u{062C}\u{0649}",
        "\u{FCDB}" => "\u{0649}\u{062D}",
        "\u{FC56}" => "\u{0649}\u{062D}",
        "\u{FDAE}" => "\u{0649}\u{062D}\u{0649}",
        "\u{FCDC}" => "\u{0649}\u{062E}",
        "\u{FC57}" => "\u{0649}\u{062E}",
        "\u{FC91}" => "\u{0649}\u{0631}",
        "\u{FC76}" => "\u{0649}\u{06DB}\u{0631}",
        "\u{FC92}" => "\u{0649}\u{0632}",
        "\u{FC77}" => "\u{0649}\u{06DB}\u{0632}",
        "\u{FCDD}" => "\u{0649}\u{0645}",
        "\u{FCF0}" => "\u{0649}\u{0645}",
        "\u{FC93}" => "\u{0649}\u{0645}",
        "\u{FC58}" => "\u{0649}\u{0645}",
        "\u{FCA6}" => "\u{0649}\u{06DB}\u{0645}",
        "\u{FCE5}" => "\u{0649}\u{06DB}\u{0645}",
        "\u{FC78}" => "\u{0649}\u{06DB}\u{0645}",
        "\u{FC12}" => "\u{0649}\u{06DB}\u{0645}",
        "\u{FD9D}" => "\u{0649}\u{0645}\u{0645}",
        "\u{FD9C}" => "\u{0649}\u{0645}\u{0645}",
        "\u{FDB0}" => "\u{0649}\u{0645}\u{0649}",
        "\u{FC94}" => "\u{0649}\u{0646}",
        "\u{FC79}" => "\u{0649}\u{06DB}\u{0646}",
        "\u{FC95}" => "\u{0649}\u{0649}",
        "\u{FC59}" => "\u{0649}\u{0649}",
        "\u{FC96}" => "\u{0649}\u{0649}",
        "\u{FC5A}" => "\u{0649}\u{0649}",
        "\u{FC7A}" => "\u{0649}\u{06DB}\u{0649}",
        "\u{FC13}" => "\u{0649}\u{06DB}\u{0649}",
        "\u{FC7B}" => "\u{0649}\u{06DB}\u{0649}",
        "\u{FC14}" => "\u{0649}\u{06DB}\u{0649}",
        "\u{FBB1}" => "\u{06D3}",
        "\u{FBB0}" => "\u{06D3}",
        "\u{102B8}" => "\u{2D40}",
        "\u{205E}" => "\u{2D42}",
        "\u{2E3D}" => "\u{2D42}",
        "\u{2999}" => "\u{2D42}",
        "\u{FE19}" => "\u{2D57}",
        "\u{205D}" => "\u{2D57}",
        "\u{22EE}" => "\u{2D57}",
        "\u{0544}" => "\u{1206}",
        "\u{054C}" => "\u{1261}",
        "\u{053B}" => "\u{12AE}",
        "\u{054A}" => "\u{1323}",
        "\u{0906}" => "\u{0905}\u{093E}",
        "\u{0912}" => "\u{0905}\u{093E}\u{0946}",
        "\u{0913}" => "\u{0905}\u{093E}\u{0947}",
        "\u{0914}" => "\u{0905}\u{093E}\u{0948}",
        "\u{0904}" => "\u{0905}\u{0946}",
        "\u{0911}" => "\u{0905}\u{0949}",
        "\u{090D}" => "\u{090F}\u{0945}",
        "\u{090E}" => "\u{090F}\u{0946}",
        "\u{0910}" => "\u{090F}\u{0947}",
        "\u{0908}" => "\u{0930}\u{094D}\u{0907}",
        "\u{0ABD}" => "\u{093D}",
        "\u{111DC}" => "\u{A8FB}",
        "\u{111CB}" => "\u{093A}",
        "\u{0AC1}" => "\u{0941}",
        "\u{0AC2}" => "\u{0942}",
        "\u{0A4B}" => "\u{0946}",
        "\u{0A4D}" => "\u{094D}",
        "\u{0ACD}" => "\u{094D}",
        "\u{0986}" => "\u{0985}\u{09BE}",
        "\u{09E0}" => "\u{098B}\u{09C3}",
        "\u{09E1}" => "\u{098B}\u{09C3}",
        "\u{11492}" => "\u{0998}",
        "\u{11494}" => "\u{099A}",
        "\u{11496}" => "\u{099C}",
        "\u{11498}" => "\u{099E}",
        "\u{11499}" => "\u{099F}",
        "\u{1149B}" => "\u{09A1}",
        "\u{114AA}" => "\u{09A3}",
        "\u{1149E}" => "\u{09A4}",
        "\u{1149F}" => "\u{09A5}",
        "\u{114A0}" => "\u{09A6}",
        "\u{114A1}" => "\u{09A7}",
        "\u{114A2}" => "\u{09A8}",
        "\u{114A3}" => "\u{09AA}",
        "\u{114A9}" => "\u{09AC}",
        "\u{114A7}" => "\u{09AE}",
        "\u{114A8}" => "\u{09AF}",
        "\u{114AB}" => "\u{09B0}",
        "\u{1149D}" => "\u{09B2}",
        "\u{114AD}" => "\u{09B7}",
        "\u{114AE}" => "\u{09B8}",
        "\u{114C4}" => "\u{09BD}",
        "\u{114B0}" => "\u{09BE}",
        "\u{114B1}" => "\u{09BF}",
        "\u{114B9}" => "\u{09C7}",
        "\u{114BC}" => "\u{09CB}",
        "\u{114BE}" => "\u{09CC}",
        "\u{114C2}" => "\u{09CD}",
        "\u{114BD}" => "\u{09D7}",
        "\u{0A09}" => "\u{0A73}\u{0A41}",
        "\u{0A0A}" => "\u{0A73}\u{0A42}",
        "\u{0A06}" => "\u{0A05}\u{0A3E}",
        "\u{0A10}" => "\u{0A05}\u{0A48}",
        "\u{0A14}" => "\u{0A05}\u{0A4C}",
        "\u{0A07}" => "\u{0A72}\u{0A3F}",
        "\u{0A08}" => "\u{0A72}\u{0A40}",
        "\u{0A0F}" => "\u{0A72}\u{0A47}",
        "\u{0A86}" => "\u{0A85}\u{0ABE}",
        "\u{0A91}" => "\u{0A85}\u{0ABE}\u{0AC5}",
        "\u{0A93}" => "\u{0A85}\u{0ABE}\u{0AC7}",
        "\u{0A94}" => "\u{0A85}\u{0ABE}\u{0AC8}",
        "\u{0A8D}" => "\u{0A85}\u{0AC5}",
        "\u{0A8F}" => "\u{0A85}\u{0AC7}",
        "\u{0A90}" => "\u{0A85}\u{0AC8}",
        "\u{0B06}" => "\u{0B05}\u{0B3E}",
        "\u{0BEE}" => "\u{0B85}",
        "\u{0BB0}" => "\u{0B88}",
        "\u{0BBE}" => "\u{0B88}",
        "\u{0BEB}" => "\u{0B88}\u{0BC1}",
        "\u{0BE8}" => "\u{0B89}",
        "\u{0D09}" => "\u{0B89}",
        "\u{0B8A}" => "\u{0B89}\u{0BB3}",
        "\u{0D0A}" => "\u{0B89}\u{0D57}",
        "\u{0BED}" => "\u{0B8E}",
        "\u{0BF7}" => "\u{0B8E}\u{0BB5}",
        "\u{0B9C}" => "\u{0B90}",
        "\u{0D1C}" => "\u{0B90}",
        "\u{0BE7}" => "\u{0B95}",
        "\u{0BEA}" => "\u{0B9A}",
        "\u{0BEC}" => "\u{0B9A}\u{0BC1}",
        "\u{0BF2}" => "\u{0B9A}\u{0BC2}",
        "\u{0D3A}" => "\u{0B9F}\u{0BBF}",
        "\u{0D23}" => "\u{0BA3}",
        "\u{0BFA}" => "\u{0BA8}\u{0BC0}",
        "\u{0BF4}" => "\u{0BAE}\u{0BC0}",
        "\u{0BF0}" => "\u{0BAF}",
        "\u{0D34}" => "\u{0BB4}",
        "\u{0BD7}" => "\u{0BB3}",
        "\u{0BC8}" => "\u{0BA9}",
        "\u{0D36}" => "\u{0BB6}",
        "\u{0BF8}" => "\u{0BB7}",
        "\u{0D3F}" => "\u{0BBF}",
        "\u{0D40}" => "\u{0BBF}",
        "\u{0BCA}" => "\u{0BC6}\u{0B88}",
        "\u{0BCC}" => "\u{0BC6}\u{0BB3}",
        "\u{0BCB}" => "\u{0BC7}\u{0B88}",
        "\u{0C85}" => "\u{0C05}",
        "\u{0C86}" => "\u{0C06}",
        "\u{0C87}" => "\u{0C07}",
        "\u{0C60}" => "\u{0C0B}\u{0C3E}",
        "\u{0C61}" => "\u{0C0C}\u{0C3E}",
        "\u{0C92}" => "\u{0C12}",
        "\u{0C14}" => "\u{0C12}\u{0C4C}",
        "\u{0C94}" => "\u{0C12}\u{0C4C}",
        "\u{0C13}" => "\u{0C12}\u{0C55}",
        "\u{0C93}" => "\u{0C12}\u{0C55}",
        "\u{0C9C}" => "\u{0C1C}",
        "\u{0C9E}" => "\u{0C1E}",
        "\u{0C22}" => "\u{0C21}\u{0323}",
        "\u{0CA3}" => "\u{0C23}",
        "\u{0C25}" => "\u{0C27}\u{05BC}",
        "\u{0C2D}" => "\u{0C2C}\u{0323}",
        "\u{0CAF}" => "\u{0C2F}",
        "\u{0C20}" => "\u{0C30}\u{05BC}",
        "\u{0CB1}" => "\u{0C31}",
        "\u{0CB2}" => "\u{0C32}",
        "\u{0C37}" => "\u{0C35}\u{0323}",
        "\u{0C39}" => "\u{0C35}\u{0C3E}",
        "\u{0C2E}" => "\u{0C35}\u{0C41}",
        "\u{0C42}" => "\u{0C41}\u{0C3E}",
        "\u{0C44}" => "\u{0C43}\u{0C3E}",
        "\u{0CE1}" => "\u{0C8C}\u{0CBE}",
        "\u{0D08}" => "\u{0D07}\u{0D57}",
        "\u{0D10}" => "\u{0D0E}\u{0D46}",
        "\u{0D13}" => "\u{0D12}\u{0D3E}",
        "\u{0D14}" => "\u{0D12}\u{0D57}",
        "\u{0D61}" => "\u{0D1E}",
        "\u{0D6B}" => "\u{0D26}\u{0D4D}\u{0D30}",
        "\u{0D79}" => "\u{0D28}\u{0D41}",
        "\u{0D0C}" => "\u{0D28}\u{0D41}",
        "\u{0D19}" => "\u{0D28}\u{0D41}",
        "\u{0D6F}" => "\u{0D28}\u{0D4D}",
        "\u{0D7B}" => "\u{0D28}\u{0D4D}",
        "\u{0D6C}" => "\u{0D28}\u{0D4D}\u{0D28}",
        "\u{0D5A}" => "\u{0D28}\u{0D4D}\u{0D2E}",
        "\u{0D31}" => "\u{0D30}",
        "\u{0D6A}" => "\u{0D30}\u{0D4D}",
        "\u{0D7C}" => "\u{0D30}\u{0D4D}",
        "\u{0D6E}" => "\u{0D35}\u{0D4D}\u{0D30}",
        "\u{0D76}" => "\u{0D39}\u{0D4D}\u{0D2E}",
        "\u{0D42}" => "\u{0D41}",
        "\u{0D43}" => "\u{0D41}",
        "\u{0D48}" => "\u{0D46}\u{0D46}",
        "\u{0DEA}" => "\u{0DA2}",
        "\u{0DEB}" => "\u{0DAF}",
        "\u{11413}" => "\u{11434}\u{11442}\u{11412}",
        "\u{11419}" => "\u{11434}\u{11442}\u{11418}",
        "\u{11424}" => "\u{11434}\u{11442}\u{11423}",
        "\u{1142A}" => "\u{11434}\u{11442}\u{11429}",
        "\u{1142D}" => "\u{11434}\u{11442}\u{1142C}",
        "\u{1142F}" => "\u{11434}\u{11442}\u{1142E}",
        "\u{115D8}" => "\u{11582}",
        "\u{115D9}" => "\u{11582}",
        "\u{115DA}" => "\u{11583}",
        "\u{115DB}" => "\u{11584}",
        "\u{115DC}" => "\u{115B2}",
        "\u{115DD}" => "\u{115B3}",
        "\u{0E03}" => "\u{0E02}",
        "\u{0E14}" => "\u{0E04}",
        "\u{0E15}" => "\u{0E04}",
        "\u{0E21}" => "\u{0E06}",
        "\u{0E88}" => "\u{0E08}",
        "\u{0E0B}" => "\u{0E0A}",
        "\u{0E0F}" => "\u{0E0E}",
        "\u{0E17}" => "\u{0E11}",
        "\u{0E9A}" => "\u{0E1A}",
        "\u{0E9B}" => "\u{0E1B}",
        "\u{0E9D}" => "\u{0E1D}",
        "\u{0E9E}" => "\u{0E1E}",
        "\u{0E9F}" => "\u{0E1F}",
        "\u{0E26}" => "\u{0E20}",
        "\u{0E8D}" => "\u{0E22}",
        "\u{17D4}" => "\u{0E2F}",
        "\u{0E45}" => "\u{0E32}",
        "\u{0E33}" => "\u{030A}\u{0E32}",
        "\u{17B7}" => "\u{0E34}",
        "\u{17B8}" => "\u{0E35}",
        "\u{17B9}" => "\u{0E36}",
        "\u{17BA}" => "\u{0E37}",
        "\u{0EB8}" => "\u{0E38}",
        "\u{0EB9}" => "\u{0E39}",
        "\u{0E41}" => "\u{0E40}\u{0E40}",
        "\u{0EDC}" => "\u{0EAB}\u{0E99}",
        "\u{0EDD}" => "\u{0EAB}\u{0EA1}",
        "\u{0EB3}" => "\u{030A}\u{0EB2}",
        "\u{0F02}" => "\u{0F60}\u{0F74}\u{0F82}\u{0F7F}",
        "\u{0F03}" => "\u{0F60}\u{0F74}\u{0F82}\u{0F14}",
        "\u{0F6A}" => "\u{0F62}",
        "\u{0F00}" => "\u{0F68}\u{0F7C}\u{0F7E}",
        "\u{0F77}" => "\u{0FB2}\u{0F71}\u{0F80}",
        "\u{0F79}" => "\u{0FB3}\u{0F71}\u{0F80}",
        "\u{11CB2}" => "\u{11CAA}",
        "\u{1081}" => "\u{1002}\u{103E}",
        "\u{1000}" => "\u{1002}\u{102C}",
        "\u{1070}" => "\u{1003}\u{103E}",
        "\u{1066}" => "\u{1015}\u{103E}",
        "\u{101F}" => "\u{1015}\u{102C}",
        "\u{106F}" => "\u{1015}\u{102C}\u{103E}",
        "\u{107E}" => "\u{107D}\u{103E}",
        "\u{1029}" => "\u{101E}\u{103C}",
        "\u{102A}" => "\u{101E}\u{103C}\u{1031}\u{102C}\u{103A}",
        "\u{109E}" => "\u{1083}\u{030A}",
        "\u{17A3}" => "\u{17A2}",
        "\u{19D0}" => "\u{199E}",
        "\u{19D1}" => "\u{19B1}",
        "\u{1A80}" => "\u{1A45}",
        "\u{1A90}" => "\u{1A45}",
        "\u{AA53}" => "\u{AA01}",
        "\u{AA56}" => "\u{AA23}",
        "\u{1B52}" => "\u{1B0D}",
        "\u{1B53}" => "\u{1B11}",
        "\u{1B58}" => "\u{1B28}",
        "\u{A9A3}" => "\u{A99D}",
        "\u{1896}" => "\u{185C}",
        "\u{1855}" => "\u{1835}",
        "\u{1FF6}" => "\u{13EF}",
        "\u{140D}" => "\u{1401}\u{00B7}",
        "\u{142B}" => "\u{1401}\u{1420}",
        "\u{1411}" => "\u{1404}\u{00B7}",
        "\u{1413}" => "\u{1405}\u{00B7}",
        "\u{142D}" => "\u{1405}\u{1420}",
        "\u{1415}" => "\u{1406}\u{00B7}",
        "\u{1418}" => "\u{140A}\u{00B7}",
        "\u{142E}" => "\u{140A}\u{1420}",
        "\u{141A}" => "\u{140B}\u{00B7}",
        "\u{18DD}" => "\u{141E}\u{18DF}",
        "\u{14D1}" => "\u{1421}",
        "\u{1540}" => "\u{1429}",
        "\u{143F}" => "\u{1432}\u{00B7}",
        "\u{1443}" => "\u{1434}\u{00B7}",
        "\u{2369}" => "\u{1435}",
        "\u{1447}" => "\u{1439}\u{00B7}",
        "\u{145C}" => "\u{144F}\u{00B7}",
        "\u{2E27}" => "\u{1450}",
        "\u{2283}" => "\u{1450}",
        "\u{145E}" => "\u{1450}\u{00B7}",
        "\u{1469}" => "\u{1450}\u{0027}",
        "\u{27C9}" => "\u{1450}\u{002F}",
        "\u{2AD7}" => "\u{1450}\u{1455}",
        "\u{1460}" => "\u{1451}\u{00B7}",
        "\u{2E26}" => "\u{1455}",
        "\u{2282}" => "\u{1455}",
        "\u{1462}" => "\u{1455}\u{00B7}",
        "\u{146A}" => "\u{1455}\u{0027}",
        "\u{1464}" => "\u{1456}\u{00B7}",
        "\u{1475}" => "\u{146B}\u{00B7}",
        "\u{1485}" => "\u{146B}\u{0027}",
        "\u{1479}" => "\u{146E}\u{00B7}",
        "\u{147D}" => "\u{1470}\u{00B7}",
        "\u{1603}" => "\u{1489}",
        "\u{1493}" => "\u{1489}\u{00B7}",
        "\u{1495}" => "\u{148B}\u{00B7}",
        "\u{1497}" => "\u{148C}\u{00B7}",
        "\u{149B}" => "\u{148E}\u{00B7}",
        "\u{1602}" => "\u{1490}",
        "\u{149D}" => "\u{1490}\u{00B7}",
        "\u{149F}" => "\u{1491}\u{00B7}",
        "\u{14AD}" => "\u{14A3}\u{00B7}",
        "\u{14B1}" => "\u{14A6}\u{00B7}",
        "\u{14B3}" => "\u{14A7}\u{00B7}",
        "\u{14B5}" => "\u{14A8}\u{00B7}",
        "\u{14B9}" => "\u{14AB}\u{00B7}",
        "\u{14CA}" => "\u{14C0}\u{00B7}",
        "\u{18C7}" => "\u{14C2}\u{00B7}",
        "\u{18C9}" => "\u{14C3}\u{00B7}",
        "\u{18CB}" => "\u{14C4}\u{00B7}",
        "\u{18CD}" => "\u{14C5}\u{00B7}",
        "\u{14CC}" => "\u{14C7}\u{00B7}",
        "\u{14CE}" => "\u{14C8}\u{00B7}",
        "\u{1604}" => "\u{14D3}",
        "\u{14DD}" => "\u{14D3}\u{00B7}",
        "\u{14DF}" => "\u{14D5}\u{00B7}",
        "\u{14E1}" => "\u{14D6}\u{00B7}",
        "\u{14E3}" => "\u{14D7}\u{00B7}",
        "\u{14E5}" => "\u{14D8}\u{00B7}",
        "\u{1607}" => "\u{14DA}",
        "\u{14E7}" => "\u{14DA}\u{00B7}",
        "\u{14E9}" => "\u{14DB}\u{00B7}",
        "\u{14F7}" => "\u{14ED}\u{00B7}",
        "\u{14F9}" => "\u{14EF}\u{00B7}",
        "\u{14FB}" => "\u{14F0}\u{00B7}",
        "\u{14FD}" => "\u{14F1}\u{00B7}",
        "\u{14FF}" => "\u{14F2}\u{00B7}",
        "\u{1501}" => "\u{14F4}\u{00B7}",
        "\u{1503}" => "\u{14F5}\u{00B7}",
        "\u{150C}" => "\u{150B}\u{003C}",
        "\u{150E}" => "\u{150B}\u{0062}",
        "\u{150D}" => "\u{150B}\u{1455}",
        "\u{150F}" => "\u{150B}\u{1490}",
        "\u{1518}" => "\u{1510}\u{00B7}",
        "\u{151A}" => "\u{1511}\u{00B7}",
        "\u{151C}" => "\u{1512}\u{00B7}",
        "\u{151E}" => "\u{1513}\u{00B7}",
        "\u{1520}" => "\u{1514}\u{00B7}",
        "\u{1522}" => "\u{1515}\u{00B7}",
        "\u{1524}" => "\u{1516}\u{00B7}",
        "\u{1532}" => "\u{1528}\u{00B7}",
        "\u{1534}" => "\u{1529}\u{00B7}",
        "\u{1536}" => "\u{152A}\u{00B7}",
        "\u{1538}" => "\u{152B}\u{00B7}",
        "\u{153A}" => "\u{152D}\u{00B7}",
        "\u{153C}" => "\u{152E}\u{00B7}",
        "\u{1622}" => "\u{1543}",
        "\u{18E0}" => "\u{1543}\u{00B7}",
        "\u{1623}" => "\u{1546}",
        "\u{1624}" => "\u{154A}",
        "\u{154F}" => "\u{154C}\u{00B7}",
        "\u{1583}" => "\u{1550}\u{0062}",
        "\u{1584}" => "\u{1550}\u{0062}\u{0307}",
        "\u{1581}" => "\u{1550}\u{0064}",
        "\u{157F}" => "\u{1550}\u{0050}",
        "\u{166F}" => "\u{1550}\u{146B}",
        "\u{157E}" => "\u{1550}\u{146C}",
        "\u{1580}" => "\u{1550}\u{146E}",
        "\u{1582}" => "\u{1550}\u{1470}",
        "\u{1585}" => "\u{1550}\u{1483}",
        "\u{155C}" => "\u{155A}\u{00B7}",
        "\u{18E3}" => "\u{155E}\u{00B7}",
        "\u{18E4}" => "\u{1566}\u{00B7}",
        "\u{1569}" => "\u{1567}\u{00B7}",
        "\u{18E5}" => "\u{156B}\u{00B7}",
        "\u{18E8}" => "\u{1586}\u{00B7}",
        "\u{1591}" => "\u{1595}\u{004A}",
        "\u{1670}" => "\u{1595}\u{1489}",
        "\u{158E}" => "\u{1595}\u{148A}",
        "\u{158F}" => "\u{1595}\u{148B}",
        "\u{1590}" => "\u{1595}\u{148C}",
        "\u{1592}" => "\u{1595}\u{148E}",
        "\u{1593}" => "\u{1595}\u{1490}",
        "\u{1594}" => "\u{1595}\u{1491}",
        "\u{1673}" => "\u{1596}\u{004A}",
        "\u{1671}" => "\u{1596}\u{148B}",
        "\u{1672}" => "\u{1596}\u{148C}",
        "\u{1674}" => "\u{1596}\u{148E}",
        "\u{1675}" => "\u{1596}\u{1490}",
        "\u{1676}" => "\u{1596}\u{1491}",
        "\u{18EA}" => "\u{1597}\u{00B7}",
        "\u{1677}" => "\u{15A7}\u{00B7}",
        "\u{1678}" => "\u{15A8}\u{00B7}",
        "\u{1679}" => "\u{15A9}\u{00B7}",
        "\u{167A}" => "\u{15AA}\u{00B7}",
        "\u{167B}" => "\u{15AB}\u{00B7}",
        "\u{167C}" => "\u{15AC}\u{00B7}",
        "\u{167D}" => "\u{15AD}\u{00B7}",
        "\u{2AAB}" => "\u{15D2}",
        "\u{2AAA}" => "\u{15D5}",
        "\u{A4F7}" => "\u{15E1}",
        "\u{18F0}" => "\u{15F4}\u{00B7}",
        "\u{18F2}" => "\u{161B}\u{00B7}",
        "\u{1DBB}" => "\u{1646}",
        "\u{A4ED}" => "\u{1660}",
        "\u{1DBA}" => "\u{18D4}",
        "\u{1D3E}" => "\u{18D6}",
        "\u{18DC}" => "\u{18DF}\u{141E}",
        "\u{02E1}" => "\u{18F3}",
        "\u{02B3}" => "\u{18F4}",
        "\u{02E2}" => "\u{18F5}",
        "\u{18DB}" => "\u{18F5}",
        "\u{A6B0}" => "\u{16B9}",
        "\u{16E1}" => "\u{16BC}",
        "\u{237F}" => "\u{16BD}",
        "\u{16C2}" => "\u{16BD}",
        "\u{1D23F}" => "\u{16CB}",
        "\u{2191}" => "\u{16CF}",
        "\u{21BF}" => "\u{16D0}",
        "\u{296E}" => "\u{16D0}\u{21C2}",
        "\u{2963}" => "\u{16D0}\u{16DA}",
        "\u{2D63}" => "\u{16EF}",
        "\u{21BE}" => "\u{16DA}",
        "\u{2A21}" => "\u{16DA}",
        "\u{22C4}" => "\u{16DC}",
        "\u{25C7}" => "\u{16DC}",
        "\u{25CA}" => "\u{16DC}",
        "\u{2662}" => "\u{16DC}",
        "\u{1F754}" => "\u{16DC}",
        "\u{118B7}" => "\u{16DC}",
        "\u{10294}" => "\u{16DC}",
        "\u{235A}" => "\u{16DC}\u{0332}",
        "\u{22C8}" => "\u{16DE}",
        "\u{2A1D}" => "\u{16DE}",
        "\u{104D0}" => "\u{16E6}",
        "\u{2195}" => "\u{16E8}",
        "\u{10CFC}" => "\u{10C82}",
        "\u{10CFA}" => "\u{10CA5}",
        "\u{3131}" => "\u{1100}",
        "\u{11A8}" => "\u{1100}",
        "\u{1101}" => "\u{1100}\u{1100}",
        "\u{3132}" => "\u{1100}\u{1100}",
        "\u{11A9}" => "\u{1100}\u{1100}",
        "\u{11FA}" => "\u{1100}\u{1102}",
        "\u{115A}" => "\u{1100}\u{1103}",
        "\u{11C3}" => "\u{1100}\u{1105}",
        "\u{11FB}" => "\u{1100}\u{1107}",
        "\u{11AA}" => "\u{1100}\u{1109}",
        "\u{3133}" => "\u{1100}\u{1109}",
        "\u{11C4}" => "\u{1100}\u{1109}\u{1100}",
        "\u{11FC}" => "\u{1100}\u{110E}",
        "\u{11FD}" => "\u{1100}\u{110F}",
        "\u{11FE}" => "\u{1100}\u{1112}",
        "\u{3134}" => "\u{1102}",
        "\u{11AB}" => "\u{1102}",
        "\u{1113}" => "\u{1102}\u{1100}",
        "\u{11C5}" => "\u{1102}\u{1100}",
        "\u{1114}" => "\u{1102}\u{1102}",
        "\u{3165}" => "\u{1102}\u{1102}",
        "\u{11FF}" => "\u{1102}\u{1102}",
        "\u{1115}" => "\u{1102}\u{1103}",
        "\u{3166}" => "\u{1102}\u{1103}",
        "\u{11C6}" => "\u{1102}\u{1103}",
        "\u{D7CB}" => "\u{1102}\u{1105}",
        "\u{1116}" => "\u{1102}\u{1107}",
        "\u{115B}" => "\u{1102}\u{1109}",
        "\u{11C7}" => "\u{1102}\u{1109}",
        "\u{3167}" => "\u{1102}\u{1109}",
        "\u{115C}" => "\u{1102}\u{110C}",
        "\u{11AC}" => "\u{1102}\u{110C}",
        "\u{3135}" => "\u{1102}\u{110C}",
        "\u{D7CC}" => "\u{1102}\u{110E}",
        "\u{11C9}" => "\u{1102}\u{1110}",
        "\u{115D}" => "\u{1102}\u{1112}",
        "\u{11AD}" => "\u{1102}\u{1112}",
        "\u{3136}" => "\u{1102}\u{1112}",
        "\u{11C8}" => "\u{1102}\u{1140}",
        "\u{3168}" => "\u{1102}\u{1140}",
        "\u{3137}" => "\u{1103}",
        "\u{11AE}" => "\u{1103}",
        "\u{1117}" => "\u{1103}\u{1100}",
        "\u{11CA}" => "\u{1103}\u{1100}",
        "\u{1104}" => "\u{1103}\u{1103}",
        "\u{3138}" => "\u{1103}\u{1103}",
        "\u{D7CD}" => "\u{1103}\u{1103}",
        "\u{D7CE}" => "\u{1103}\u{1103}\u{1107}",
        "\u{115E}" => "\u{1103}\u{1105}",
        "\u{11CB}" => "\u{1103}\u{1105}",
        "\u{A960}" => "\u{1103}\u{1106}",
        "\u{A961}" => "\u{1103}\u{1107}",
        "\u{D7CF}" => "\u{1103}\u{1107}",
        "\u{A962}" => "\u{1103}\u{1109}",
        "\u{D7D0}" => "\u{1103}\u{1109}",
        "\u{D7D1}" => "\u{1103}\u{1109}\u{1100}",
        "\u{A963}" => "\u{1103}\u{110C}",
        "\u{D7D2}" => "\u{1103}\u{110C}",
        "\u{D7D3}" => "\u{1103}\u{110E}",
        "\u{D7D4}" => "\u{1103}\u{1110}",
        "\u{3139}" => "\u{1105}",
        "\u{11AF}" => "\u{1105}",
        "\u{A964}" => "\u{1105}\u{1100}",
        "\u{11B0}" => "\u{1105}\u{1100}",
        "\u{313A}" => "\u{1105}\u{1100}",
        "\u{A965}" => "\u{1105}\u{1100}\u{1100}",
        "\u{D7D5}" => "\u{1105}\u{1100}\u{1100}",
        "\u{11CC}" => "\u{1105}\u{1100}\u{1109}",
        "\u{3169}" => "\u{1105}\u{1100}\u{1109}",
        "\u{D7D6}" => "\u{1105}\u{1100}\u{1112}",
        "\u{1118}" => "\u{1105}\u{1102}",
        "\u{11CD}" => "\u{1105}\u{1102}",
        "\u{A966}" => "\u{1105}\u{1103}",
        "\u{11CE}" => "\u{1105}\u{1103}",
        "\u{316A}" => "\u{1105}\u{1103}",
        "\u{A967}" => "\u{1105}\u{1103}\u{1103}",
        "\u{11CF}" => "\u{1105}\u{1103}\u{1112}",
        "\u{1119}" => "\u{1105}\u{1105}",
        "\u{11D0}" => "\u{1105}\u{1105}",
        "\u{D7D7}" => "\u{1105}\u{1105}\u{110F}",
        "\u{A968}" => "\u{1105}\u{1106}",
        "\u{11B1}" => "\u{1105}\u{1106}",
        "\u{313B}" => "\u{1105}\u{1106}",
        "\u{11D1}" => "\u{1105}\u{1106}\u{1100}",
        "\u{11D2}" => "\u{1105}\u{1106}\u{1109}",
        "\u{D7D8}" => "\u{1105}\u{1106}\u{1112}",
        "\u{A969}" => "\u{1105}\u{1107}",
        "\u{11B2}" => "\u{1105}\u{1107}",
        "\u{313C}" => "\u{1105}\u{1107}",
        "\u{D7D9}" => "\u{1105}\u{1107}\u{1103}",
        "\u{A96A}" => "\u{1105}\u{1107}\u{1107}",
        "\u{11D3}" => "\u{1105}\u{1107}\u{1109}",
        "\u{316B}" => "\u{1105}\u{1107}\u{1109}",
        "\u{A96B}" => "\u{1105}\u{1107}\u{110B}",
        "\u{11D5}" => "\u{1105}\u{1107}\u{110B}",
        "\u{D7DA}" => "\u{1105}\u{1107}\u{1111}",
        "\u{11D4}" => "\u{1105}\u{1107}\u{1112}",
        "\u{A96C}" => "\u{1105}\u{1109}",
        "\u{11B3}" => "\u{1105}\u{1109}",
        "\u{313D}" => "\u{1105}\u{1109}",
        "\u{11D6}" => "\u{1105}\u{1109}\u{1109}",
        "\u{111B}" => "\u{1105}\u{110B}",
        "\u{D7DD}" => "\u{1105}\u{110B}",
        "\u{A96D}" => "\u{1105}\u{110C}",
        "\u{A96E}" => "\u{1105}\u{110F}",
        "\u{11D8}" => "\u{1105}\u{110F}",
        "\u{11B4}" => "\u{1105}\u{1110}",
        "\u{313E}" => "\u{1105}\u{1110}",
        "\u{11B5}" => "\u{1105}\u{1111}",
        "\u{313F}" => "\u{1105}\u{1111}",
        "\u{111A}" => "\u{1105}\u{1112}",
        "\u{3140}" => "\u{1105}\u{1112}",
        "\u{113B}" => "\u{1105}\u{1112}",
        "\u{11B6}" => "\u{1105}\u{1112}",
        "\u{D7F2}" => "\u{1105}\u{1112}",
        "\u{11D7}" => "\u{1105}\u{1140}",
        "\u{316C}" => "\u{1105}\u{1140}",
        "\u{D7DB}" => "\u{1105}\u{114C}",
        "\u{11D9}" => "\u{1105}\u{1159}",
        "\u{316D}" => "\u{1105}\u{1159}",
        "\u{D7DC}" => "\u{1105}\u{1159}\u{1112}",
        "\u{3141}" => "\u{1106}",
        "\u{11B7}" => "\u{1106}",
        "\u{A96F}" => "\u{1106}\u{1100}",
        "\u{11DA}" => "\u{1106}\u{1100}",
        "\u{D7DE}" => "\u{1106}\u{1102}",
        "\u{D7DF}" => "\u{1106}\u{1102}\u{1102}",
        "\u{A970}" => "\u{1106}\u{1103}",
        "\u{11DB}" => "\u{1106}\u{1105}",
        "\u{D7E0}" => "\u{1106}\u{1106}",
        "\u{111C}" => "\u{1106}\u{1107}",
        "\u{316E}" => "\u{1106}\u{1107}",
        "\u{11DC}" => "\u{1106}\u{1107}",
        "\u{D7E1}" => "\u{1106}\u{1107}\u{1109}",
        "\u{A971}" => "\u{1106}\u{1109}",
        "\u{11DD}" => "\u{1106}\u{1109}",
        "\u{316F}" => "\u{1106}\u{1109}",
        "\u{11DE}" => "\u{1106}\u{1109}\u{1109}",
        "\u{111D}" => "\u{1106}\u{110B}",
        "\u{3171}" => "\u{1106}\u{110B}",
        "\u{11E2}" => "\u{1106}\u{110B}",
        "\u{D7E2}" => "\u{1106}\u{110C}",
        "\u{11E0}" => "\u{1106}\u{110E}",
        "\u{11E1}" => "\u{1106}\u{1112}",
        "\u{11DF}" => "\u{1106}\u{1140}",
        "\u{3170}" => "\u{1106}\u{1140}",
        "\u{3142}" => "\u{1107}",
        "\u{11B8}" => "\u{1107}",
        "\u{111E}" => "\u{1107}\u{1100}",
        "\u{3172}" => "\u{1107}\u{1100}",
        "\u{111F}" => "\u{1107}\u{1102}",
        "\u{1120}" => "\u{1107}\u{1103}",
        "\u{3173}" => "\u{1107}\u{1103}",
        "\u{D7E3}" => "\u{1107}\u{1103}",
        "\u{11E3}" => "\u{1107}\u{1105}",
        "\u{D7E4}" => "\u{1107}\u{1105}\u{1111}",
        "\u{D7E5}" => "\u{1107}\u{1106}",
        "\u{1108}" => "\u{1107}\u{1107}",
        "\u{3143}" => "\u{1107}\u{1107}",
        "\u{D7E6}" => "\u{1107}\u{1107}",
        "\u{112C}" => "\u{1107}\u{1107}\u{110B}",
        "\u{3179}" => "\u{1107}\u{1107}\u{110B}",
        "\u{1121}" => "\u{1107}\u{1109}",
        "\u{3144}" => "\u{1107}\u{1109}",
        "\u{11B9}" => "\u{1107}\u{1109}",
        "\u{1122}" => "\u{1107}\u{1109}\u{1100}",
        "\u{3174}" => "\u{1107}\u{1109}\u{1100}",
        "\u{1123}" => "\u{1107}\u{1109}\u{1103}",
        "\u{3175}" => "\u{1107}\u{1109}\u{1103}",
        "\u{D7E7}" => "\u{1107}\u{1109}\u{1103}",
        "\u{1124}" => "\u{1107}\u{1109}\u{1107}",
        "\u{1125}" => "\u{1107}\u{1109}\u{1109}",
        "\u{1126}" => "\u{1107}\u{1109}\u{110C}",
        "\u{A972}" => "\u{1107}\u{1109}\u{1110}",
        "\u{112B}" => "\u{1107}\u{110B}",
        "\u{3178}" => "\u{1107}\u{110B}",
        "\u{11E6}" => "\u{1107}\u{110B}",
        "\u{1127}" => "\u{1107}\u{110C}",
        "\u{3176}" => "\u{1107}\u{110C}",
        "\u{D7E8}" => "\u{1107}\u{110C}",
        "\u{1128}" => "\u{1107}\u{110E}",
        "\u{D7E9}" => "\u{1107}\u{110E}",
        "\u{A973}" => "\u{1107}\u{110F}",
        "\u{1129}" => "\u{1107}\u{1110}",
        "\u{3177}" => "\u{1107}\u{1110}",
        "\u{112A}" => "\u{1107}\u{1111}",
        "\u{11E4}" => "\u{1107}\u{1111}",
        "\u{A974}" => "\u{1107}\u{1112}",
        "\u{11E5}" => "\u{1107}\u{1112}",
        "\u{3145}" => "\u{1109}",
        "\u{11BA}" => "\u{1109}",
        "\u{112D}" => "\u{1109}\u{1100}",
        "\u{317A}" => "\u{1109}\u{1100}",
        "\u{11E7}" => "\u{1109}\u{1100}",
        "\u{112E}" => "\u{1109}\u{1102}",
        "\u{317B}" => "\u{1109}\u{1102}",
        "\u{112F}" => "\u{1109}\u{1103}",
        "\u{317C}" => "\u{1109}\u{1103}",
        "\u{11E8}" => "\u{1109}\u{1103}",
        "\u{1130}" => "\u{1109}\u{1105}",
        "\u{11E9}" => "\u{1109}\u{1105}",
        "\u{1131}" => "\u{1109}\u{1106}",
        "\u{D7EA}" => "\u{1109}\u{1106}",
        "\u{1132}" => "\u{1109}\u{1107}",
        "\u{317D}" => "\u{1109}\u{1107}",
        "\u{11EA}" => "\u{1109}\u{1107}",
        "\u{1133}" => "\u{1109}\u{1107}\u{1100}",
        "\u{D7EB}" => "\u{1109}\u{1107}\u{110B}",
        "\u{110A}" => "\u{1109}\u{1109}",
        "\u{3146}" => "\u{1109}\u{1109}",
        "\u{11BB}" => "\u{1109}\u{1109}",
        "\u{D7EC}" => "\u{1109}\u{1109}\u{1100}",
        "\u{D7ED}" => "\u{1109}\u{1109}\u{1103}",
        "\u{A975}" => "\u{1109}\u{1109}\u{1107}",
        "\u{1134}" => "\u{1109}\u{1109}\u{1109}",
        "\u{1135}" => "\u{1109}\u{110B}",
        "\u{1136}" => "\u{1109}\u{110C}",
        "\u{317E}" => "\u{1109}\u{110C}",
        "\u{D7EF}" => "\u{1109}\u{110C}",
        "\u{1137}" => "\u{1109}\u{110E}",
        "\u{D7F0}" => "\u{1109}\u{110E}",
        "\u{1138}" => "\u{1109}\u{110F}",
        "\u{1139}" => "\u{1109}\u{1110}",
        "\u{D7F1}" => "\u{1109}\u{1110}",
        "\u{113A}" => "\u{1109}\u{1111}",
        "\u{D7EE}" => "\u{1109}\u{1140}",
        "\u{3147}" => "\u{110B}",
        "\u{11BC}" => "\u{110B}",
        "\u{1141}" => "\u{110B}\u{1100}",
        "\u{11EC}" => "\u{110B}\u{1100}",
        "\u{11ED}" => "\u{110B}\u{1100}\u{1100}",
        "\u{1142}" => "\u{110B}\u{1103}",
        "\u{A976}" => "\u{110B}\u{1105}",
        "\u{1143}" => "\u{110B}\u{1106}",
        "\u{1144}" => "\u{110B}\u{1107}",
        "\u{1145}" => "\u{110B}\u{1109}",
        "\u{11F1}" => "\u{110B}\u{1109}",
        "\u{3182}" => "\u{110B}\u{1109}",
        "\u{1147}" => "\u{110B}\u{110B}",
        "\u{3180}" => "\u{110B}\u{110B}",
        "\u{11EE}" => "\u{110B}\u{110B}",
        "\u{1148}" => "\u{110B}\u{110C}",
        "\u{1149}" => "\u{110B}\u{110E}",
        "\u{11EF}" => "\u{110B}\u{110F}",
        "\u{114A}" => "\u{110B}\u{1110}",
        "\u{114B}" => "\u{110B}\u{1111}",
        "\u{A977}" => "\u{110B}\u{1112}",
        "\u{1146}" => "\u{110B}\u{1140}",
        "\u{11F2}" => "\u{110B}\u{1140}",
        "\u{3183}" => "\u{110B}\u{1140}",
        "\u{3148}" => "\u{110C}",
        "\u{11BD}" => "\u{110C}",
        "\u{D7F7}" => "\u{110C}\u{1107}",
        "\u{D7F8}" => "\u{110C}\u{1107}\u{1107}",
        "\u{114D}" => "\u{110C}\u{110B}",
        "\u{110D}" => "\u{110C}\u{110C}",
        "\u{3149}" => "\u{110C}\u{110C}",
        "\u{D7F9}" => "\u{110C}\u{110C}",
        "\u{A978}" => "\u{110C}\u{110C}\u{1112}",
        "\u{314A}" => "\u{110E}",
        "\u{11BE}" => "\u{110E}",
        "\u{1152}" => "\u{110E}\u{110F}",
        "\u{1153}" => "\u{110E}\u{1112}",
        "\u{314B}" => "\u{110F}",
        "\u{11BF}" => "\u{110F}",
        "\u{314C}" => "\u{1110}",
        "\u{11C0}" => "\u{1110}",
        "\u{A979}" => "\u{1110}\u{1110}",
        "\u{314D}" => "\u{1111}",
        "\u{11C1}" => "\u{1111}",
        "\u{1156}" => "\u{1111}\u{1107}",
        "\u{11F3}" => "\u{1111}\u{1107}",
        "\u{D7FA}" => "\u{1111}\u{1109}",
        "\u{1157}" => "\u{1111}\u{110B}",
        "\u{3184}" => "\u{1111}\u{110B}",
        "\u{11F4}" => "\u{1111}\u{110B}",
        "\u{D7FB}" => "\u{1111}\u{1110}",
        "\u{A97A}" => "\u{1111}\u{1112}",
        "\u{314E}" => "\u{1112}",
        "\u{11C2}" => "\u{1112}",
        "\u{11F5}" => "\u{1112}\u{1102}",
        "\u{11F6}" => "\u{1112}\u{1105}",
        "\u{11F7}" => "\u{1112}\u{1106}",
        "\u{11F8}" => "\u{1112}\u{1107}",
        "\u{A97B}" => "\u{1112}\u{1109}",
        "\u{1158}" => "\u{1112}\u{1112}",
        "\u{3185}" => "\u{1112}\u{1112}",
        "\u{113D}" => "\u{113C}\u{113C}",
        "\u{113F}" => "\u{113E}\u{113E}",
        "\u{317F}" => "\u{1140}",
        "\u{11EB}" => "\u{1140}",
        "\u{D7F3}" => "\u{1140}\u{1107}",
        "\u{D7F4}" => "\u{1140}\u{1107}\u{110B}",
        "\u{3181}" => "\u{114C}",
        "\u{11F0}" => "\u{114C}",
        "\u{D7F5}" => "\u{114C}\u{1106}",
        "\u{D7F6}" => "\u{114C}\u{1112}",
        "\u{114F}" => "\u{114E}\u{114E}",
        "\u{1151}" => "\u{1150}\u{1150}",
        "\u{3186}" => "\u{1159}",
        "\u{11F9}" => "\u{1159}",
        "\u{A97C}" => "\u{1159}\u{1159}",
        "\u{3164}" => "\u{1160}",
        "\u{314F}" => "\u{1161}",
        "\u{11A3}" => "\u{1161}\u{30FC}",
        "\u{1176}" => "\u{1161}\u{1169}",
        "\u{1177}" => "\u{1161}\u{116E}",
        "\u{1162}" => "\u{1161}\u{4E28}",
        "\u{3150}" => "\u{1161}\u{4E28}",
        "\u{3151}" => "\u{1163}",
        "\u{1178}" => "\u{1163}\u{1169}",
        "\u{1179}" => "\u{1163}\u{116D}",
        "\u{11A4}" => "\u{1163}\u{116E}",
        "\u{1164}" => "\u{1163}\u{4E28}",
        "\u{3152}" => "\u{1163}\u{4E28}",
        "\u{3153}" => "\u{1165}",
        "\u{117C}" => "\u{1165}\u{30FC}",
        "\u{117A}" => "\u{1165}\u{1169}",
        "\u{117B}" => "\u{1165}\u{116E}",
        "\u{1166}" => "\u{1165}\u{4E28}",
        "\u{3154}" => "\u{1165}\u{4E28}",
        "\u{3155}" => "\u{1167}",
        "\u{11A5}" => "\u{1167}\u{1163}",
        "\u{117D}" => "\u{1167}\u{1169}",
        "\u{117E}" => "\u{1167}\u{116E}",
        "\u{1168}" => "\u{1167}\u{4E28}",
        "\u{3156}" => "\u{1167}\u{4E28}",
        "\u{3157}" => "\u{1169}",
        "\u{116A}" => "\u{1169}\u{1161}",
        "\u{3158}" => "\u{1169}\u{1161}",
        "\u{116B}" => "\u{1169}\u{1161}\u{4E28}",
        "\u{3159}" => "\u{1169}\u{1161}\u{4E28}",
        "\u{11A6}" => "\u{1169}\u{1163}",
        "\u{11A7}" => "\u{1169}\u{1163}\u{4E28}",
        "\u{117F}" => "\u{1169}\u{1165}",
        "\u{1180}" => "\u{1169}\u{1165}\u{4E28}",
        "\u{D7B0}" => "\u{1169}\u{1167}",
        "\u{1181}" => "\u{1169}\u{1167}\u{4E28}",
        "\u{1182}" => "\u{1169}\u{1169}",
        "\u{D7B1}" => "\u{1169}\u{1169}\u{4E28}",
        "\u{1183}" => "\u{1169}\u{116E}",
        "\u{116C}" => "\u{1169}\u{4E28}",
        "\u{315A}" => "\u{1169}\u{4E28}",
        "\u{315B}" => "\u{116D}",
        "\u{D7B2}" => "\u{116D}\u{1161}",
        "\u{D7B3}" => "\u{116D}\u{1161}\u{4E28}",
        "\u{1184}" => "\u{116D}\u{1163}",
        "\u{3187}" => "\u{116D}\u{1163}",
        "\u{1186}" => "\u{116D}\u{1163}",
        "\u{1185}" => "\u{116D}\u{1163}\u{4E28}",
        "\u{3188}" => "\u{116D}\u{1163}\u{4E28}",
        "\u{D7B4}" => "\u{116D}\u{1165}",
        "\u{1187}" => "\u{116D}\u{1169}",
        "\u{1188}" => "\u{116D}\u{4E28}",
        "\u{3189}" => "\u{116D}\u{4E28}",
        "\u{315C}" => "\u{116E}",
        "\u{1189}" => "\u{116E}\u{1161}",
        "\u{118A}" => "\u{116E}\u{1161}\u{4E28}",
        "\u{116F}" => "\u{116E}\u{1165}",
        "\u{315D}" => "\u{116E}\u{1165}",
        "\u{118B}" => "\u{116E}\u{1165}\u{30FC}",
        "\u{1170}" => "\u{116E}\u{1165}\u{4E28}",
        "\u{315E}" => "\u{116E}\u{1165}\u{4E28}",
        "\u{D7B5}" => "\u{116E}\u{1167}",
        "\u{118C}" => "\u{116E}\u{1167}\u{4E28}",
        "\u{118D}" => "\u{116E}\u{116E}",
        "\u{1171}" => "\u{116E}\u{4E28}",
        "\u{315F}" => "\u{116E}\u{4E28}",
        "\u{D7B6}" => "\u{116E}\u{4E28}\u{4E28}",
        "\u{3160}" => "\u{1172}",
        "\u{118E}" => "\u{1172}\u{1161}",
        "\u{D7B7}" => "\u{1172}\u{1161}\u{4E28}",
        "\u{118F}" => "\u{1172}\u{1165}",
        "\u{1190}" => "\u{1172}\u{1165}\u{4E28}",
        "\u{1191}" => "\u{1172}\u{1167}",
        "\u{318A}" => "\u{1172}\u{1167}",
        "\u{1192}" => "\u{1172}\u{1167}\u{4E28}",
        "\u{318B}" => "\u{1172}\u{1167}\u{4E28}",
        "\u{D7B8}" => "\u{1172}\u{1169}",
        "\u{1193}" => "\u{1172}\u{116E}",
        "\u{1194}" => "\u{1172}\u{4E28}",
        "\u{318C}" => "\u{1172}\u{4E28}",
        "\u{318D}" => "\u{119E}",
        "\u{D7C5}" => "\u{119E}\u{1161}",
        "\u{119F}" => "\u{119E}\u{1165}",
        "\u{D7C6}" => "\u{119E}\u{1165}\u{4E28}",
        "\u{11A0}" => "\u{119E}\u{116E}",
        "\u{11A2}" => "\u{119E}\u{119E}",
        "\u{11A1}" => "\u{119E}\u{4E28}",
        "\u{318E}" => "\u{119E}\u{4E28}",
        "\u{30D8}" => "\u{3078}",
        "\u{2341}" => "\u{303C}",
        "\u{29C4}" => "\u{303C}",
        "\u{A49E}" => "\u{A04A}",
        "\u{A4AC}" => "\u{A050}",
        "\u{A49C}" => "\u{A0C0}",
        "\u{A4A8}" => "\u{A132}",
        "\u{A4BF}" => "\u{A259}",
        "\u{A4BE}" => "\u{A2B1}",
        "\u{A494}" => "\u{A2CD}",
        "\u{A4C0}" => "\u{A3AB}",
        "\u{A4C2}" => "\u{A3B5}",
        "\u{A4BA}" => "\u{A3BF}",
        "\u{A4B0}" => "\u{A3C2}",
        "\u{A4A7}" => "\u{A458}",
        "\u{22A5}" => "\u{A4D5}",
        "\u{27C2}" => "\u{A4D5}",
        "\u{1D21C}" => "\u{A4D5}",
        "\u{A7B1}" => "\u{A4D5}",
        "\u{A79E}" => "\u{A4E4}",
        "\u{2141}" => "\u{A4E8}",
        "\u{2142}" => "\u{A4F6}",
        "\u{1D215}" => "\u{A4F6}",
        "\u{1D22B}" => "\u{A4F6}",
        "\u{16F26}" => "\u{A4F6}",
        "\u{10411}" => "\u{A4F6}",
        "\u{2143}" => "\u{16F00}",
        "\u{11AE6}" => "\u{11AE5}\u{11AEF}",
        "\u{11AE8}" => "\u{11AE5}\u{11AE5}",
        "\u{11AE9}" => "\u{11AE5}\u{11AE5}\u{11AEF}",
        "\u{11AEA}" => "\u{11AE5}\u{11AE5}\u{11AF0}",
        "\u{11AE7}" => "\u{11AE5}\u{11AF0}",
        "\u{11AF4}" => "\u{11AF3}\u{11AEF}",
        "\u{11AF6}" => "\u{11AF3}\u{11AF3}",
        "\u{11AF7}" => "\u{11AF3}\u{11AF3}\u{11AEF}",
        "\u{11AF8}" => "\u{11AF3}\u{11AF3}\u{11AF0}",
        "\u{11AF5}" => "\u{11AF3}\u{11AF0}",
        "\u{11AEC}" => "\u{11AEB}\u{11AEF}",
        "\u{11AED}" => "\u{11AEB}\u{11AEB}",
        "\u{11AEE}" => "\u{11AEB}\u{11AEB}\u{11AEF}",
        "\u{2295}" => "\u{102A8}",
        "\u{2A01}" => "\u{102A8}",
        "\u{1F728}" => "\u{102A8}",
        "\u{A69A}" => "\u{102A8}",
        "\u{25BD}" => "\u{102BC}",
        "\u{1D214}" => "\u{102BC}",
        "\u{1F704}" => "\u{102BC}",
        "\u{29D6}" => "\u{102C0}",
        "\u{A79B}" => "\u{1043A}",
        "\u{A79A}" => "\u{10412}",
        "\u{104A0}" => "\u{10486}",
        "\u{103D1}" => "\u{10382}",
        "\u{103D3}" => "\u{10393}",
        "\u{12038}" => "\u{1039A}",
        "\u{2625}" => "\u{1099E}",
        "\u{132F9}" => "\u{1099E}",
        "\u{3039}" => "\u{5344}",
        "\u{F967}" => "\u{4E0D}",
        "\u{2F800}" => "\u{4E3D}",
        "\u{FA70}" => "\u{4E26}",
        "\u{239C}" => "\u{4E28}",
        "\u{239F}" => "\u{4E28}",
        "\u{23A2}" => "\u{4E28}",
        "\u{23A5}" => "\u{4E28}",
        "\u{23AA}" => "\u{4E28}",
        "\u{23AE}" => "\u{4E28}",
        "\u{31D1}" => "\u{4E28}",
        "\u{1175}" => "\u{4E28}",
        "\u{3163}" => "\u{4E28}",
        "\u{2F01}" => "\u{4E28}",
        "\u{119C}" => "\u{4E28}\u{30FC}",
        "\u{1198}" => "\u{4E28}\u{1161}",
        "\u{1199}" => "\u{4E28}\u{1163}",
        "\u{D7BD}" => "\u{4E28}\u{1163}\u{1169}",
        "\u{D7BE}" => "\u{4E28}\u{1163}\u{4E28}",
        "\u{D7BF}" => "\u{4E28}\u{1167}",
        "\u{D7C0}" => "\u{4E28}\u{1167}\u{4E28}",
        "\u{119A}" => "\u{4E28}\u{1169}",
        "\u{D7C1}" => "\u{4E28}\u{1169}\u{4E28}",
        "\u{D7C2}" => "\u{4E28}\u{116D}",
        "\u{119B}" => "\u{4E28}\u{116E}",
        "\u{D7C3}" => "\u{4E28}\u{1172}",
        "\u{119D}" => "\u{4E28}\u{119E}",
        "\u{D7C4}" => "\u{4E28}\u{4E28}",
        "\u{F905}" => "\u{4E32}",
        "\u{2F801}" => "\u{4E38}",
        "\u{F95E}" => "\u{4E39}",
        "\u{2F802}" => "\u{4E41}",
        "\u{31E0}" => "\u{4E59}",
        "\u{2F04}" => "\u{4E59}",
        "\u{31DF}" => "\u{4E5A}",
        "\u{2E83}" => "\u{4E5A}",
        "\u{31D6}" => "\u{4E5B}",
        "\u{2E82}" => "\u{4E5B}",
        "\u{2EF2}" => "\u{4E80}",
        "\u{F91B}" => "\u{4E82}",
        "\u{31DA}" => "\u{4E85}",
        "\u{2F05}" => "\u{4E85}",
        "\u{F9BA}" => "\u{4E86}",
        "\u{30CB}" => "\u{4E8C}",
        "\u{2F06}" => "\u{4E8C}",
        "\u{2F803}" => "\u{20122}",
        "\u{2F07}" => "\u{4EA0}",
        "\u{F977}" => "\u{4EAE}",
        "\u{2F08}" => "\u{4EBA}",
        "\u{30A4}" => "\u{4EBB}",
        "\u{2E85}" => "\u{4EBB}",
        "\u{F9FD}" => "\u{4EC0}",
        "\u{2F819}" => "\u{4ECC}",
        "\u{F9A8}" => "\u{4EE4}",
        "\u{2F804}" => "\u{4F60}",
        "\u{5002}" => "\u{4F75}",
        "\u{2F807}" => "\u{4F75}",
        "\u{FA73}" => "\u{4F80}",
        "\u{F92D}" => "\u{4F86}",
        "\u{F9B5}" => "\u{4F8B}",
        "\u{FA30}" => "\u{4FAE}",
        "\u{2F805}" => "\u{4FAE}",
        "\u{2F806}" => "\u{4FBB}",
        "\u{F965}" => "\u{4FBF}",
        "\u{503C}" => "\u{5024}",
        "\u{F9D4}" => "\u{502B}",
        "\u{2F808}" => "\u{507A}",
        "\u{2F809}" => "\u{5099}",
        "\u{2F80B}" => "\u{50CF}",
        "\u{F9BB}" => "\u{50DA}",
        "\u{FA31}" => "\u{50E7}",
        "\u{2F80A}" => "\u{50E7}",
        "\u{2F80C}" => "\u{349E}",
        "\u{2F09}" => "\u{513F}",
        "\u{FA0C}" => "\u{5140}",
        "\u{2E8E}" => "\u{5140}",
        "\u{FA74}" => "\u{5145}",
        "\u{FA32}" => "\u{514D}",
        "\u{2F80E}" => "\u{514D}",
        "\u{2F80F}" => "\u{5154}",
        "\u{2F810}" => "\u{5164}",
        "\u{2F0A}" => "\u{5165}",
        "\u{2F814}" => "\u{5167}",
        "\u{FA72}" => "\u{5168}",
        "\u{F978}" => "\u{5169}",
        "\u{30CF}" => "\u{516B}",
        "\u{2F0B}" => "\u{516B}",
        "\u{F9D1}" => "\u{516D}",
        "\u{2F811}" => "\u{5177}",
        "\u{2F812}" => "\u{2051C}",
        "\u{2F91B}" => "\u{20525}",
        "\u{FA75}" => "\u{5180}",
        "\u{2F813}" => "\u{34B9}",
        "\u{2F0C}" => "\u{5182}",
        "\u{2F815}" => "\u{518D}",
        "\u{2F816}" => "\u{2054B}",
        "\u{2F8D2}" => "\u{5192}",
        "\u{2F8D3}" => "\u{5195}",
        "\u{2F9CA}" => "\u{34BB}",
        "\u{2F8D4}" => "\u{6700}",
        "\u{2F0D}" => "\u{5196}",
        "\u{2F817}" => "\u{5197}",
        "\u{2F818}" => "\u{51A4}",
        "\u{2F0E}" => "\u{51AB}",
        "\u{2F81A}" => "\u{51AC}",
        "\u{FA71}" => "\u{51B5}",
        "\u{2F81B}" => "\u{51B5}",
        "\u{F92E}" => "\u{51B7}",
        "\u{F979}" => "\u{51C9}",
        "\u{F955}" => "\u{51CC}",
        "\u{F954}" => "\u{51DC}",
        "\u{FA15}" => "\u{51DE}",
        "\u{2F0F}" => "\u{51E0}",
        "\u{2F80D}" => "\u{2063A}",
        "\u{2F81D}" => "\u{51F5}",
        "\u{2F10}" => "\u{51F5}",
        "\u{2F11}" => "\u{5200}",
        "\u{2E89}" => "\u{5202}",
        "\u{2F81E}" => "\u{5203}",
        "\u{FA00}" => "\u{5207}",
        "\u{2F850}" => "\u{5207}",
        "\u{F99C}" => "\u{5217}",
        "\u{F9DD}" => "\u{5229}",
        "\u{2F81F}" => "\u{34DF}",
        "\u{F9FF}" => "\u{523A}",
        "\u{2F820}" => "\u{523B}",
        "\u{2F821}" => "\u{5246}",
        "\u{2F822}" => "\u{5272}",
        "\u{2F823}" => "\u{5277}",
        "\u{F9C7}" => "\u{5289}",
        "\u{2F9D9}" => "\u{20804}",
        "\u{30AB}" => "\u{529B}",
        "\u{F98A}" => "\u{529B}",
        "\u{2F12}" => "\u{529B}",
        "\u{F99D}" => "\u{52A3}",
        "\u{2F824}" => "\u{3515}",
        "\u{2F992}" => "\u{52B3}",
        "\u{FA76}" => "\u{52C7}",
        "\u{2F825}" => "\u{52C7}",
        "\u{FA33}" => "\u{52C9}",
        "\u{2F826}" => "\u{52C9}",
        "\u{F952}" => "\u{52D2}",
        "\u{F92F}" => "\u{52DE}",
        "\u{FA34}" => "\u{52E4}",
        "\u{2F827}" => "\u{52E4}",
        "\u{F97F}" => "\u{52F5}",
        "\u{2F13}" => "\u{52F9}",
        "\u{FA77}" => "\u{52FA}",
        "\u{2F828}" => "\u{52FA}",
        "\u{2F829}" => "\u{5305}",
        "\u{2F82A}" => "\u{5306}",
        "\u{2F9DD}" => "\u{208DE}",
        "\u{2F14}" => "\u{5315}",
        "\u{F963}" => "\u{5317}",
        "\u{2F82B}" => "\u{5317}",
        "\u{2F15}" => "\u{531A}",
        "\u{2F16}" => "\u{5338}",
        "\u{F9EB}" => "\u{533F}",
        "\u{2F17}" => "\u{5341}",
        "\u{3038}" => "\u{5341}",
        "\u{303A}" => "\u{5345}",
        "\u{2F82C}" => "\u{5349}",
        "\u{0FD6}" => "\u{534D}",
        "\u{0FD5}" => "\u{5350}",
        "\u{FA35}" => "\u{5351}",
        "\u{2F82D}" => "\u{5351}",
        "\u{2F82E}" => "\u{535A}",
        "\u{30C8}" => "\u{535C}",
        "\u{2F18}" => "\u{535C}",
        "\u{2F19}" => "\u{5369}",
        "\u{2E8B}" => "\u{353E}",
        "\u{2F82F}" => "\u{5373}",
        "\u{F91C}" => "\u{5375}",
        "\u{2F830}" => "\u{537D}",
        "\u{2F831}" => "\u{537F}",
        "\u{2F832}" => "\u{537F}",
        "\u{2F833}" => "\u{537F}",
        "\u{2F1A}" => "\u{5382}",
        "\u{2F834}" => "\u{20A2C}",
        "\u{2F1B}" => "\u{53B6}",
        "\u{F96B}" => "\u{53C3}",
        "\u{2F1C}" => "\u{53C8}",
        "\u{2F836}" => "\u{53CA}",
        "\u{2F837}" => "\u{53DF}",
        "\u{2F838}" => "\u{20B63}",
        "\u{30ED}" => "\u{53E3}",
        "\u{2F1D}" => "\u{53E3}",
        "\u{56D7}" => "\u{53E3}",
        "\u{2F1E}" => "\u{53E3}",
        "\u{F906}" => "\u{53E5}",
        "\u{2F839}" => "\u{53EB}",
        "\u{2F83A}" => "\u{53F1}",
        "\u{2F83B}" => "\u{5406}",
        "\u{F9DE}" => "\u{540F}",
        "\u{F9ED}" => "\u{541D}",
        "\u{2F83D}" => "\u{5438}",
        "\u{F980}" => "\u{5442}",
        "\u{2F83E}" => "\u{5448}",
        "\u{2F83F}" => "\u{5468}",
        "\u{2F83C}" => "\u{549E}",
        "\u{2F840}" => "\u{54A2}",
        "\u{F99E}" => "\u{54BD}",
        "\u{439B}" => "\u{3588}",
        "\u{2F841}" => "\u{54F6}",
        "\u{2F842}" => "\u{5510}",
        "\u{2F843}" => "\u{5553}",
        "\u{555F}" => "\u{5553}",
        "\u{FA79}" => "\u{5555}",
        "\u{2F844}" => "\u{5563}",
        "\u{2F845}" => "\u{5584}",
        "\u{2F846}" => "\u{5584}",
        "\u{F90B}" => "\u{5587}",
        "\u{FA7A}" => "\u{5599}",
        "\u{2F847}" => "\u{5599}",
        "\u{FA36}" => "\u{559D}",
        "\u{FA78}" => "\u{559D}",
        "\u{2F848}" => "\u{55AB}",
        "\u{2F849}" => "\u{55B3}",
        "\u{FA0D}" => "\u{55C0}",
        "\u{2F84A}" => "\u{55C2}",
        "\u{FA7B}" => "\u{55E2}",
        "\u{FA37}" => "\u{5606}",
        "\u{2F84C}" => "\u{5606}",
        "\u{2F84E}" => "\u{5651}",
        "\u{2F84F}" => "\u{5674}",
        "\u{FA38}" => "\u{5668}",
        "\u{F9A9}" => "\u{56F9}",
        "\u{2F84B}" => "\u{5716}",
        "\u{2F84D}" => "\u{5717}",
        "\u{2F1F}" => "\u{571F}",
        "\u{58EB}" => "\u{571F}",
        "\u{2F20}" => "\u{571F}",
        "\u{2F855}" => "\u{578B}",
        "\u{2F852}" => "\u{57CE}",
        "\u{39B3}" => "\u{363D}",
        "\u{2F853}" => "\u{57F4}",
        "\u{2F854}" => "\u{580D}",
        "\u{2F857}" => "\u{5831}",
        "\u{2F856}" => "\u{5832}",
        "\u{FA39}" => "\u{5840}",
        "\u{FA10}" => "\u{585A}",
        "\u{FA7C}" => "\u{585A}",
        "\u{F96C}" => "\u{585E}",
        "\u{586B}" => "\u{5861}",
        "\u{58FF}" => "\u{58AB}",
        "\u{2F858}" => "\u{58AC}",
        "\u{FA7D}" => "\u{58B3}",
        "\u{F94A}" => "\u{58D8}",
        "\u{F942}" => "\u{58DF}",
        "\u{2F859}" => "\u{214E4}",
        "\u{2F851}" => "\u{58EE}",
        "\u{2F85A}" => "\u{58F2}",
        "\u{2F85B}" => "\u{58F7}",
        "\u{2F21}" => "\u{5902}",
        "\u{2F85C}" => "\u{5906}",
        "\u{2F22}" => "\u{590A}",
        "\u{30BF}" => "\u{5915}",
        "\u{2F23}" => "\u{5915}",
        "\u{2F85D}" => "\u{591A}",
        "\u{2F85E}" => "\u{5922}",
        "\u{2F24}" => "\u{5927}",
        "\u{FA7E}" => "\u{5944}",
        "\u{F90C}" => "\u{5948}",
        "\u{F909}" => "\u{5951}",
        "\u{FA7F}" => "\u{5954}",
        "\u{2F85F}" => "\u{5962}",
        "\u{F981}" => "\u{5973}",
        "\u{2F25}" => "\u{5973}",
        "\u{2F860}" => "\u{216A8}",
        "\u{2F861}" => "\u{216EA}",
        "\u{2F865}" => "\u{59D8}",
        "\u{2F862}" => "\u{59EC}",
        "\u{2F863}" => "\u{5A1B}",
        "\u{2F864}" => "\u{5A27}",
        "\u{FA80}" => "\u{5A62}",
        "\u{2F866}" => "\u{5A66}",
        "\u{5B00}" => "\u{5AAF}",
        "\u{2F867}" => "\u{36EE}",
        "\u{2F868}" => "\u{36FC}",
        "\u{2F986}" => "\u{5AB5}",
        "\u{2F869}" => "\u{5B08}",
        "\u{FA81}" => "\u{5B28}",
        "\u{2F86A}" => "\u{5B3E}",
        "\u{2F86B}" => "\u{5B3E}",
        "\u{2F26}" => "\u{5B50}",
        "\u{2F27}" => "\u{5B80}",
        "\u{FA04}" => "\u{5B85}",
        "\u{2F86C}" => "\u{219C8}",
        "\u{2F86D}" => "\u{5BC3}",
        "\u{2F86E}" => "\u{5BD8}",
        "\u{F95F}" => "\u{5BE7}",
        "\u{F9AA}" => "\u{5BE7}",
        "\u{2F86F}" => "\u{5BE7}",
        "\u{F9BC}" => "\u{5BEE}",
        "\u{2F870}" => "\u{5BF3}",
        "\u{2F871}" => "\u{21B18}",
        "\u{2F28}" => "\u{5BF8}",
        "\u{2F872}" => "\u{5BFF}",
        "\u{2F873}" => "\u{5C06}",
        "\u{2F29}" => "\u{5C0F}",
        "\u{2F875}" => "\u{5C22}",
        "\u{2E90}" => "\u{5C22}",
        "\u{2F2A}" => "\u{5C22}",
        "\u{2E8F}" => "\u{5C23}",
        "\u{2F876}" => "\u{3781}",
        "\u{2F2B}" => "\u{5C38}",
        "\u{F9BD}" => "\u{5C3F}",
        "\u{2F877}" => "\u{5C60}",
        "\u{F94B}" => "\u{5C62}",
        "\u{FA3B}" => "\u{5C64}",
        "\u{F9DF}" => "\u{5C65}",
        "\u{FA3C}" => "\u{5C6E}",
        "\u{2F878}" => "\u{5C6E}",
        "\u{2F2C}" => "\u{5C6E}",
        "\u{2F8F8}" => "\u{21D0B}",
        "\u{2F2D}" => "\u{5C71}",
        "\u{2F879}" => "\u{5CC0}",
        "\u{2F87A}" => "\u{5C8D}",
        "\u{2F87B}" => "\u{21DE4}",
        "\u{2F87D}" => "\u{21DE6}",
        "\u{F9D5}" => "\u{5D19}",
        "\u{2F87C}" => "\u{5D43}",
        "\u{F921}" => "\u{5D50}",
        "\u{2F87F}" => "\u{5D6B}",
        "\u{2F87E}" => "\u{5D6E}",
        "\u{2F880}" => "\u{5D7C}",
        "\u{2F9F4}" => "\u{5DB2}",
        "\u{F9AB}" => "\u{5DBA}",
        "\u{2F2E}" => "\u{5DDB}",
        "\u{2F882}" => "\u{5DE2}",
        "\u{30A8}" => "\u{5DE5}",
        "\u{2F2F}" => "\u{5DE5}",
        "\u{2F30}" => "\u{5DF1}",
        "\u{2E92}" => "\u{5DF3}",
        "\u{2F883}" => "\u{382F}",
        "\u{2F884}" => "\u{5DFD}",
        "\u{2F31}" => "\u{5DFE}",
        "\u{5E32}" => "\u{5E21}",
        "\u{2F885}" => "\u{5E28}",
        "\u{2F886}" => "\u{5E3D}",
        "\u{2F887}" => "\u{5E69}",
        "\u{2F888}" => "\u{3862}",
        "\u{2F889}" => "\u{22183}",
        "\u{2F32}" => "\u{5E72}",
        "\u{F98E}" => "\u{5E74}",
        "\u{2F939}" => "\u{2219F}",
        "\u{2E93}" => "\u{5E7A}",
        "\u{2F33}" => "\u{5E7A}",
        "\u{2F34}" => "\u{5E7F}",
        "\u{FA01}" => "\u{5EA6}",
        "\u{2F88A}" => "\u{387C}",
        "\u{2F88B}" => "\u{5EB0}",
        "\u{2F88C}" => "\u{5EB3}",
        "\u{2F88D}" => "\u{5EB6}",
        "\u{F928}" => "\u{5ECA}",
        "\u{2F88E}" => "\u{5ECA}",
        "\u{F9A2}" => "\u{5EC9}",
        "\u{FA82}" => "\u{5ED2}",
        "\u{FA0B}" => "\u{5ED3}",
        "\u{FA83}" => "\u{5ED9}",
        "\u{F982}" => "\u{5EEC}",
        "\u{2F35}" => "\u{5EF4}",
        "\u{2F890}" => "\u{5EFE}",
        "\u{2F36}" => "\u{5EFE}",
        "\u{2F891}" => "\u{22331}",
        "\u{2F892}" => "\u{22331}",
        "\u{F943}" => "\u{5F04}",
        "\u{2F37}" => "\u{5F0B}",
        "\u{2F38}" => "\u{5F13}",
        "\u{2F894}" => "\u{5F22}",
        "\u{2F895}" => "\u{5F22}",
        "\u{2F39}" => "\u{5F50}",
        "\u{2E94}" => "\u{5F51}",
        "\u{2F874}" => "\u{5F53}",
        "\u{2F896}" => "\u{38C7}",
        "\u{2F3A}" => "\u{5F61}",
        "\u{2F899}" => "\u{5F62}",
        "\u{FA84}" => "\u{5F69}",
        "\u{2F89A}" => "\u{5F6B}",
        "\u{2F3B}" => "\u{5F73}",
        "\u{F9D8}" => "\u{5F8B}",
        "\u{2F89B}" => "\u{38E3}",
        "\u{2F89C}" => "\u{5F9A}",
        "\u{F966}" => "\u{5FA9}",
        "\u{FA85}" => "\u{5FAD}",
        "\u{2F3C}" => "\u{5FC3}",
        "\u{2E96}" => "\u{5FC4}",
        "\u{2E97}" => "\u{38FA}",
        "\u{2F89D}" => "\u{5FCD}",
        "\u{2F89E}" => "\u{5FD7}",
        "\u{F9A3}" => "\u{5FF5}",
        "\u{2F89F}" => "\u{5FF9}",
        "\u{F960}" => "\u{6012}",
        "\u{F9AC}" => "\u{601C}",
        "\u{FA6B}" => "\u{6075}",
        "\u{2F8A2}" => "\u{391C}",
        "\u{2F8A1}" => "\u{393A}",
        "\u{2F8A0}" => "\u{6081}",
        "\u{FA3D}" => "\u{6094}",
        "\u{2F8A3}" => "\u{6094}",
        "\u{2F8A5}" => "\u{60C7}",
        "\u{FA86}" => "\u{60D8}",
        "\u{F9B9}" => "\u{60E1}",
        "\u{2F8A4}" => "\u{226D4}",
        "\u{FA88}" => "\u{6108}",
        "\u{FA3E}" => "\u{6168}",
        "\u{F9D9}" => "\u{6144}",
        "\u{2F8A6}" => "\u{6148}",
        "\u{2F8A7}" => "\u{614C}",
        "\u{2F8A9}" => "\u{614C}",
        "\u{FA87}" => "\u{614E}",
        "\u{2F8A8}" => "\u{614E}",
        "\u{FA8A}" => "\u{6160}",
        "\u{2F8AA}" => "\u{617A}",
        "\u{FA3F}" => "\u{618E}",
        "\u{FA89}" => "\u{618E}",
        "\u{2F8AB}" => "\u{618E}",
        "\u{F98F}" => "\u{6190}",
        "\u{2F8AD}" => "\u{61A4}",
        "\u{2F8AE}" => "\u{61AF}",
        "\u{2F8AC}" => "\u{61B2}",
        "\u{FAD0}" => "\u{22844}",
        "\u{FACF}" => "\u{2284A}",
        "\u{2F8AF}" => "\u{61DE}",
        "\u{FA40}" => "\u{61F2}",
        "\u{FA8B}" => "\u{61F2}",
        "\u{2F8B0}" => "\u{61F2}",
        "\u{F90D}" => "\u{61F6}",
        "\u{2F8B1}" => "\u{61F6}",
        "\u{F990}" => "\u{6200}",
        "\u{2F3D}" => "\u{6208}",
        "\u{2F8B2}" => "\u{6210}",
        "\u{2F8B3}" => "\u{621B}",
        "\u{F9D2}" => "\u{622E}",
        "\u{FA8C}" => "\u{6234}",
        "\u{2F3E}" => "\u{6236}",
        "\u{6238}" => "\u{6236}",
        "\u{2F3F}" => "\u{624B}",
        "\u{2E98}" => "\u{624C}",
        "\u{2F8B4}" => "\u{625D}",
        "\u{2F8B5}" => "\u{62B1}",
        "\u{F925}" => "\u{62C9}",
        "\u{F95B}" => "\u{62CF}",
        "\u{FA02}" => "\u{62D3}",
        "\u{2F8B6}" => "\u{62D4}",
        "\u{2F8BA}" => "\u{62FC}",
        "\u{F973}" => "\u{62FE}",
        "\u{2F8B8}" => "\u{22B0C}",
        "\u{2F8B9}" => "\u{633D}",
        "\u{2F8B7}" => "\u{6350}",
        "\u{2F8BB}" => "\u{6368}",
        "\u{F9A4}" => "\u{637B}",
        "\u{2F8BC}" => "\u{6383}",
        "\u{F975}" => "\u{63A0}",
        "\u{2F8C1}" => "\u{63A9}",
        "\u{FA8D}" => "\u{63C4}",
        "\u{2F8BD}" => "\u{63E4}",
        "\u{FA8F}" => "\u{6452}",
        "\u{2F8BE}" => "\u{22BF1}",
        "\u{FA8E}" => "\u{641C}",
        "\u{2F8BF}" => "\u{6422}",
        "\u{2F8C0}" => "\u{63C5}",
        "\u{2F8C3}" => "\u{6469}",
        "\u{2F8C6}" => "\u{6477}",
        "\u{2F8C4}" => "\u{647E}",
        "\u{2F8C2}" => "\u{3A2E}",
        "\u{6409}" => "\u{3A41}",
        "\u{F991}" => "\u{649A}",
        "\u{2F8C5}" => "\u{649D}",
        "\u{F930}" => "\u{64C4}",
        "\u{2F8C7}" => "\u{3A6C}",
        "\u{2F40}" => "\u{652F}",
        "\u{2F41}" => "\u{6534}",
        "\u{2E99}" => "\u{6535}",
        "\u{FA41}" => "\u{654F}",
        "\u{2F8C8}" => "\u{654F}",
        "\u{FA90}" => "\u{6556}",
        "\u{2F8C9}" => "\u{656C}",
        "\u{F969}" => "\u{6578}",
        "\u{2F8CA}" => "\u{2300A}",
        "\u{2F42}" => "\u{6587}",
        "\u{2EEB}" => "\u{6589}",
        "\u{2F43}" => "\u{6597}",
        "\u{F9BE}" => "\u{6599}",
        "\u{2F44}" => "\u{65A4}",
        "\u{2F45}" => "\u{65B9}",
        "\u{F983}" => "\u{65C5}",
        "\u{2F46}" => "\u{65E0}",
        "\u{2E9B}" => "\u{65E1}",
        "\u{FA42}" => "\u{65E2}",
        "\u{2F8CB}" => "\u{65E3}",
        "\u{2F47}" => "\u{65E5}",
        "\u{F9E0}" => "\u{6613}",
        "\u{66F6}" => "\u{3ADA}",
        "\u{2F8D1}" => "\u{3AE4}",
        "\u{2F8CD}" => "\u{6649}",
        "\u{6669}" => "\u{665A}",
        "\u{FA12}" => "\u{6674}",
        "\u{FA91}" => "\u{6674}",
        "\u{FA43}" => "\u{6691}",
        "\u{2F8CF}" => "\u{6691}",
        "\u{F9C5}" => "\u{6688}",
        "\u{2F8D0}" => "\u{3B08}",
        "\u{2F8D5}" => "\u{669C}",
        "\u{FA06}" => "\u{66B4}",
        "\u{F98B}" => "\u{66C6}",
        "\u{2F8CE}" => "\u{3B19}",
        "\u{2F897}" => "\u{232B8}",
        "\u{2F48}" => "\u{66F0}",
        "\u{F901}" => "\u{66F4}",
        "\u{2F8CC}" => "\u{66F8}",
        "\u{2F49}" => "\u{6708}",
        "\u{2F980}" => "\u{2335F}",
        "\u{80A6}" => "\u{670C}",
        "\u{80D0}" => "\u{670F}",
        "\u{80CA}" => "\u{6710}",
        "\u{8101}" => "\u{6713}",
        "\u{80F6}" => "\u{3B35}",
        "\u{F929}" => "\u{6717}",
        "\u{FA92}" => "\u{6717}",
        "\u{2F8D8}" => "\u{6717}",
        "\u{8127}" => "\u{6718}",
        "\u{FA93}" => "\u{671B}",
        "\u{2F8D9}" => "\u{671B}",
        "\u{5E50}" => "\u{3B3A}",
        "\u{4420}" => "\u{3B3B}",
        "\u{2F989}" => "\u{23393}",
        "\u{81A7}" => "\u{6723}",
        "\u{2F98A}" => "\u{2339C}",
        "\u{2F4A}" => "\u{6728}",
        "\u{F9E1}" => "\u{674E}",
        "\u{2F8DC}" => "\u{6753}",
        "\u{FA94}" => "\u{6756}",
        "\u{2F8DB}" => "\u{675E}",
        "\u{2F8DD}" => "\u{233C3}",
        "\u{67FF}" => "\u{676E}",
        "\u{F9C8}" => "\u{677B}",
        "\u{2F8E0}" => "\u{6785}",
        "\u{F9F4}" => "\u{6797}",
        "\u{2F8DE}" => "\u{3B49}",
        "\u{FAD1}" => "\u{233D5}",
        "\u{F9C9}" => "\u{67F3}",
        "\u{2F8DF}" => "\u{67FA}",
        "\u{F9DA}" => "\u{6817}",
        "\u{2F8E5}" => "\u{681F}",
        "\u{2F8E1}" => "\u{6852}",
        "\u{2F8E3}" => "\u{2346D}",
        "\u{F97A}" => "\u{6881}",
        "\u{FA44}" => "\u{6885}",
        "\u{2F8E2}" => "\u{6885}",
        "\u{2F8E4}" => "\u{688E}",
        "\u{F9E2}" => "\u{68A8}",
        "\u{2F8E6}" => "\u{6914}",
        "\u{2F8E8}" => "\u{6942}",
        "\u{FAD2}" => "\u{3B9D}",
        "\u{2F8E7}" => "\u{3B9D}",
        "\u{69E9}" => "\u{3BA3}",
        "\u{6A27}" => "\u{699D}",
        "\u{2F8E9}" => "\u{69A3}",
        "\u{2F8EA}" => "\u{69EA}",
        "\u{F914}" => "\u{6A02}",
        "\u{F95C}" => "\u{6A02}",
        "\u{F9BF}" => "\u{6A02}",
        "\u{F94C}" => "\u{6A13}",
        "\u{2F8EC}" => "\u{236A3}",
        "\u{2F8EB}" => "\u{6AA8}",
        "\u{F931}" => "\u{6AD3}",
        "\u{2F8ED}" => "\u{6ADB}",
        "\u{F91D}" => "\u{6B04}",
        "\u{2F8EE}" => "\u{3C18}",
        "\u{2F4B}" => "\u{6B20}",
        "\u{2F8EF}" => "\u{6B21}",
        "\u{2F8F0}" => "\u{238A7}",
        "\u{2F8F1}" => "\u{6B54}",
        "\u{2F8F2}" => "\u{3C4E}",
        "\u{2F4C}" => "\u{6B62}",
        "\u{2EED}" => "\u{6B6F}",
        "\u{2F8F3}" => "\u{6B72}",
        "\u{F98C}" => "\u{6B77}",
        "\u{FA95}" => "\u{6B79}",
        "\u{2F4D}" => "\u{6B79}",
        "\u{2E9E}" => "\u{6B7A}",
        "\u{2F8F4}" => "\u{6B9F}",
        "\u{F9A5}" => "\u{6BAE}",
        "\u{2F4E}" => "\u{6BB3}",
        "\u{F970}" => "\u{6BBA}",
        "\u{FA96}" => "\u{6BBA}",
        "\u{2F8F5}" => "\u{6BBA}",
        "\u{2F8F6}" => "\u{6BBB}",
        "\u{2F8F7}" => "\u{23A8D}",
        "\u{2F4F}" => "\u{6BCB}",
        "\u{2E9F}" => "\u{6BCD}",
        "\u{2F8F9}" => "\u{23AFA}",
        "\u{2F50}" => "\u{6BD4}",
        "\u{2F51}" => "\u{6BDB}",
        "\u{2F52}" => "\u{6C0F}",
        "\u{2EA0}" => "\u{6C11}",
        "\u{2F53}" => "\u{6C14}",
        "\u{2F54}" => "\u{6C34}",
        "\u{2EA1}" => "\u{6C35}",
        "\u{2EA2}" => "\u{6C3A}",
        "\u{2F8FA}" => "\u{6C4E}",
        "\u{2F8FE}" => "\u{6C67}",
        "\u{F972}" => "\u{6C88}",
        "\u{2F8FC}" => "\u{6CBF}",
        "\u{F968}" => "\u{6CCC}",
        "\u{2F8FD}" => "\u{6CCD}",
        "\u{F9E3}" => "\u{6CE5}",
        "\u{2F8FB}" => "\u{23CBC}",
        "\u{F915}" => "\u{6D1B}",
        "\u{FA05}" => "\u{6D1E}",
        "\u{2F907}" => "\u{6D34}",
        "\u{2F900}" => "\u{6D3E}",
        "\u{F9CA}" => "\u{6D41}",
        "\u{FA97}" => "\u{6D41}",
        "\u{2F902}" => "\u{6D41}",
        "\u{2F8FF}" => "\u{6D16}",
        "\u{2F903}" => "\u{6D69}",
        "\u{F92A}" => "\u{6D6A}",
        "\u{FA45}" => "\u{6D77}",
        "\u{2F901}" => "\u{6D77}",
        "\u{2F904}" => "\u{6D78}",
        "\u{2F905}" => "\u{6D85}",
        "\u{2F906}" => "\u{23D1E}",
        "\u{F9F5}" => "\u{6DCB}",
        "\u{F94D}" => "\u{6DDA}",
        "\u{F9D6}" => "\u{6DEA}",
        "\u{2F90E}" => "\u{6DF9}",
        "\u{FA46}" => "\u{6E1A}",
        "\u{2F908}" => "\u{6E2F}",
        "\u{2F909}" => "\u{6E6E}",
        "\u{6F59}" => "\u{6E88}",
        "\u{FA99}" => "\u{6ECB}",
        "\u{2F90B}" => "\u{6ECB}",
        "\u{F9CB}" => "\u{6E9C}",
        "\u{F9EC}" => "\u{6EBA}",
        "\u{2F90C}" => "\u{6EC7}",
        "\u{F904}" => "\u{6ED1}",
        "\u{FA98}" => "\u{6EDB}",
        "\u{2F90A}" => "\u{3D33}",
        "\u{F94E}" => "\u{6F0F}",
        "\u{FA47}" => "\u{6F22}",
        "\u{FA9A}" => "\u{6F22}",
        "\u{F992}" => "\u{6F23}",
        "\u{2F90D}" => "\u{23ED1}",
        "\u{2F90F}" => "\u{6F6E}",
        "\u{2F910}" => "\u{23F5E}",
        "\u{2F911}" => "\u{23F8E}",
        "\u{2F912}" => "\u{6FC6}",
        "\u{F922}" => "\u{6FEB}",
        "\u{F984}" => "\u{6FFE}",
        "\u{2F915}" => "\u{701B}",
        "\u{FA9B}" => "\u{701E}",
        "\u{2F914}" => "\u{701E}",
        "\u{2F913}" => "\u{7039}",
        "\u{2F917}" => "\u{704A}",
        "\u{2F916}" => "\u{3D96}",
        "\u{2F55}" => "\u{706B}",
        "\u{2EA3}" => "\u{706C}",
        "\u{2F835}" => "\u{7070}",
        "\u{2F919}" => "\u{7077}",
        "\u{2F918}" => "\u{707D}",
        "\u{F9FB}" => "\u{7099}",
        "\u{2F91A}" => "\u{70AD}",
        "\u{F99F}" => "\u{70C8}",
        "\u{F916}" => "\u{70D9}",
        "\u{FA48}" => "\u{716E}",
        "\u{FA9C}" => "\u{716E}",
        "\u{2F91D}" => "\u{24263}",
        "\u{2F91C}" => "\u{7145}",
        "\u{F993}" => "\u{7149}",
        "\u{FA6C}" => "\u{242EE}",
        "\u{2F91E}" => "\u{719C}",
        "\u{F9C0}" => "\u{71CE}",
        "\u{F9EE}" => "\u{71D0}",
        "\u{2F91F}" => "\u{243AB}",
        "\u{F932}" => "\u{7210}",
        "\u{F91E}" => "\u{721B}",
        "\u{2F920}" => "\u{7228}",
        "\u{2F56}" => "\u{722A}",
        "\u{FA49}" => "\u{722B}",
        "\u{2EA4}" => "\u{722B}",
        "\u{FA9E}" => "\u{7235}",
        "\u{2F921}" => "\u{7235}",
        "\u{2F57}" => "\u{7236}",
        "\u{2F58}" => "\u{723B}",
        "\u{2EA6}" => "\u{4E2C}",
        "\u{2F59}" => "\u{723F}",
        "\u{2F5A}" => "\u{7247}",
        "\u{2F922}" => "\u{7250}",
        "\u{2F5B}" => "\u{7259}",
        "\u{2F923}" => "\u{24608}",
        "\u{2F5C}" => "\u{725B}",
        "\u{F946}" => "\u{7262}",
        "\u{2F924}" => "\u{7280}",
        "\u{2F925}" => "\u{7295}",
        "\u{2F5D}" => "\u{72AC}",
        "\u{2EA8}" => "\u{72AD}",
        "\u{FA9F}" => "\u{72AF}",
        "\u{F9FA}" => "\u{72C0}",
        "\u{2F926}" => "\u{24735}",
        "\u{F92B}" => "\u{72FC}",
        "\u{FA16}" => "\u{732A}",
        "\u{FAA0}" => "\u{732A}",
        "\u{2F927}" => "\u{24814}",
        "\u{F9A7}" => "\u{7375}",
        "\u{2F928}" => "\u{737A}",
        "\u{2F5E}" => "\u{7384}",
        "\u{F961}" => "\u{7387}",
        "\u{F9DB}" => "\u{7387}",
        "\u{2F5F}" => "\u{7389}",
        "\u{2F929}" => "\u{738B}",
        "\u{2F92A}" => "\u{3EAC}",
        "\u{2F92B}" => "\u{73A5}",
        "\u{F9AD}" => "\u{73B2}",
        "\u{2F92C}" => "\u{3EB8}",
        "\u{2F92D}" => "\u{3EB8}",
        "\u{F917}" => "\u{73DE}",
        "\u{F9CC}" => "\u{7409}",
        "\u{F9E4}" => "\u{7406}",
        "\u{FA4A}" => "\u{7422}",
        "\u{2F92E}" => "\u{7447}",
        "\u{2F92F}" => "\u{745C}",
        "\u{F9AE}" => "\u{7469}",
        "\u{FAA1}" => "\u{7471}",
        "\u{2F930}" => "\u{7471}",
        "\u{2F931}" => "\u{7485}",
        "\u{F994}" => "\u{7489}",
        "\u{F9EF}" => "\u{7498}",
        "\u{2F932}" => "\u{74CA}",
        "\u{2F60}" => "\u{74DC}",
        "\u{2F61}" => "\u{74E6}",
        "\u{2F933}" => "\u{3F1B}",
        "\u{FAA2}" => "\u{7506}",
        "\u{2F62}" => "\u{7518}",
        "\u{2F63}" => "\u{751F}",
        "\u{2F934}" => "\u{7524}",
        "\u{2F64}" => "\u{7528}",
        "\u{2F65}" => "\u{7530}",
        "\u{FAA3}" => "\u{753B}",
        "\u{2F936}" => "\u{753E}",
        "\u{2F935}" => "\u{24C36}",
        "\u{F9CD}" => "\u{7559}",
        "\u{F976}" => "\u{7565}",
        "\u{F962}" => "\u{7570}",
        "\u{2F938}" => "\u{7570}",
        "\u{2F937}" => "\u{24C92}",
        "\u{2F66}" => "\u{758B}",
        "\u{2F67}" => "\u{7592}",
        "\u{F9E5}" => "\u{75E2}",
        "\u{2F93A}" => "\u{7610}",
        "\u{FAA5}" => "\u{761F}",
        "\u{FAA4}" => "\u{761D}",
        "\u{F9C1}" => "\u{7642}",
        "\u{F90E}" => "\u{7669}",
        "\u{2F68}" => "\u{7676}",
        "\u{2F69}" => "\u{767D}",
        "\u{2F93B}" => "\u{24FA1}",
        "\u{2F93C}" => "\u{24FB8}",
        "\u{2F6A}" => "\u{76AE}",
        "\u{2F6B}" => "\u{76BF}",
        "\u{2F93D}" => "\u{25044}",
        "\u{2F93E}" => "\u{3FFC}",
        "\u{FA17}" => "\u{76CA}",
        "\u{FAA6}" => "\u{76CA}",
        "\u{FAA7}" => "\u{76DB}",
        "\u{F933}" => "\u{76E7}",
        "\u{2F93F}" => "\u{4008}",
        "\u{2F6C}" => "\u{76EE}",
        "\u{FAA8}" => "\u{76F4}",
        "\u{2F940}" => "\u{76F4}",
        "\u{2F942}" => "\u{250F2}",
        "\u{2F941}" => "\u{250F3}",
        "\u{F96D}" => "\u{7701}",
        "\u{FAD3}" => "\u{4018}",
        "\u{2F943}" => "\u{25119}",
        "\u{2F945}" => "\u{771E}",
        "\u{2F946}" => "\u{771F}",
        "\u{2F947}" => "\u{771F}",
        "\u{2F944}" => "\u{25133}",
        "\u{FAAA}" => "\u{7740}",
        "\u{FAA9}" => "\u{774A}",
        "\u{2F948}" => "\u{774A}",
        "\u{9FC3}" => "\u{4039}",
        "\u{FAD4}" => "\u{4039}",
        "\u{2F949}" => "\u{4039}",
        "\u{6663}" => "\u{403F}",
        "\u{2F94B}" => "\u{4046}",
        "\u{2F94A}" => "\u{778B}",
        "\u{FAD5}" => "\u{25249}",
        "\u{FA9D}" => "\u{77A7}",
        "\u{2F6D}" => "\u{77DB}",
        "\u{2F6E}" => "\u{77E2}",
        "\u{2F6F}" => "\u{77F3}",
        "\u{2F94C}" => "\u{4096}",
        "\u{2F94D}" => "\u{2541D}",
        "\u{784F}" => "\u{7814}",
        "\u{2F94E}" => "\u{784E}",
        "\u{F9CE}" => "\u{786B}",
        "\u{F93B}" => "\u{788C}",
        "\u{2F94F}" => "\u{788C}",
        "\u{FA4B}" => "\u{7891}",
        "\u{F947}" => "\u{78CA}",
        "\u{FAAB}" => "\u{78CC}",
        "\u{2F950}" => "\u{78CC}",
        "\u{F964}" => "\u{78FB}",
        "\u{2F951}" => "\u{40E3}",
        "\u{F985}" => "\u{792A}",
        "\u{2F70}" => "\u{793A}",
        "\u{2EAD}" => "\u{793B}",
        "\u{FA18}" => "\u{793C}",
        "\u{FA4C}" => "\u{793E}",
        "\u{FA4E}" => "\u{7948}",
        "\u{FA4D}" => "\u{7949}",
        "\u{2F952}" => "\u{25626}",
        "\u{FA4F}" => "\u{7950}",
        "\u{FA50}" => "\u{7956}",
        "\u{2F953}" => "\u{7956}",
        "\u{FA51}" => "\u{795D}",
        "\u{FA19}" => "\u{795E}",
        "\u{FA1A}" => "\u{7965}",
        "\u{FA61}" => "\u{8996}",
        "\u{FAB8}" => "\u{8996}",
        "\u{F93C}" => "\u{797F}",
        "\u{2F954}" => "\u{2569A}",
        "\u{FA52}" => "\u{798D}",
        "\u{FA53}" => "\u{798E}",
        "\u{FA1B}" => "\u{798F}",
        "\u{2F956}" => "\u{798F}",
        "\u{2F955}" => "\u{256C5}",
        "\u{F9B6}" => "\u{79AE}",
        "\u{2F71}" => "\u{79B8}",
        "\u{2F72}" => "\u{79BE}",
        "\u{F995}" => "\u{79CA}",
        "\u{2F958}" => "\u{412F}",
        "\u{2F957}" => "\u{79EB}",
        "\u{F956}" => "\u{7A1C}",
        "\u{2F95A}" => "\u{7A4A}",
        "\u{FA54}" => "\u{7A40}",
        "\u{2F959}" => "\u{7A40}",
        "\u{2F95B}" => "\u{7A4F}",
        "\u{2F73}" => "\u{7A74}",
        "\u{FA55}" => "\u{7A81}",
        "\u{2F95C}" => "\u{2597C}",
        "\u{FAAC}" => "\u{7AB1}",
        "\u{F9F7}" => "\u{7ACB}",
        "\u{2F74}" => "\u{7ACB}",
        "\u{2EEF}" => "\u{7ADC}",
        "\u{2F95D}" => "\u{25AA7}",
        "\u{2F95E}" => "\u{25AA7}",
        "\u{2F95F}" => "\u{7AEE}",
        "\u{2F75}" => "\u{7AF9}",
        "\u{F9F8}" => "\u{7B20}",
        "\u{FA56}" => "\u{7BC0}",
        "\u{FAAD}" => "\u{7BC0}",
        "\u{2F960}" => "\u{4202}",
        "\u{2F961}" => "\u{25BAB}",
        "\u{2F962}" => "\u{7BC6}",
        "\u{2F964}" => "\u{4227}",
        "\u{2F963}" => "\u{7BC9}",
        "\u{2F965}" => "\u{25C80}",
        "\u{FAD6}" => "\u{25CD0}",
        "\u{F9A6}" => "\u{7C3E}",
        "\u{F944}" => "\u{7C60}",
        "\u{2F76}" => "\u{7C73}",
        "\u{FAAE}" => "\u{7C7B}",
        "\u{F9F9}" => "\u{7C92}",
        "\u{FA1D}" => "\u{7CBE}",
        "\u{2F966}" => "\u{7CD2}",
        "\u{FA03}" => "\u{7CD6}",
        "\u{2F968}" => "\u{7CE8}",
        "\u{2F967}" => "\u{42A0}",
        "\u{2F969}" => "\u{7CE3}",
        "\u{F97B}" => "\u{7CE7}",
        "\u{2F77}" => "\u{7CF8}",
        "\u{2EAF}" => "\u{7CF9}",
        "\u{2F96B}" => "\u{25F86}",
        "\u{2F96A}" => "\u{7D00}",
        "\u{F9CF}" => "\u{7D10}",
        "\u{F96A}" => "\u{7D22}",
        "\u{F94F}" => "\u{7D2F}",
        "\u{7D76}" => "\u{7D55}",
        "\u{2F96C}" => "\u{7D63}",
        "\u{FAAF}" => "\u{7D5B}",
        "\u{F93D}" => "\u{7DA0}",
        "\u{F957}" => "\u{7DBE}",
        "\u{2F96E}" => "\u{7DC7}",
        "\u{F996}" => "\u{7DF4}",
        "\u{FA57}" => "\u{7DF4}",
        "\u{FAB0}" => "\u{7DF4}",
        "\u{2F96F}" => "\u{7E02}",
        "\u{2F96D}" => "\u{4301}",
        "\u{FA58}" => "\u{7E09}",
        "\u{F950}" => "\u{7E37}",
        "\u{FA59}" => "\u{7E41}",
        "\u{2F970}" => "\u{7E45}",
        "\u{2F898}" => "\u{261DA}",
        "\u{2F971}" => "\u{4334}",
        "\u{2F78}" => "\u{7F36}",
        "\u{2F972}" => "\u{26228}",
        "\u{FAB1}" => "\u{7F3E}",
        "\u{2F973}" => "\u{26247}",
        "\u{2F79}" => "\u{7F51}",
        "\u{2EAB}" => "\u{7F52}",
        "\u{2EB2}" => "\u{7F52}",
        "\u{2EB1}" => "\u{7F53}",
        "\u{2F974}" => "\u{4359}",
        "\u{FA5A}" => "\u{7F72}",
        "\u{2F975}" => "\u{262D9}",
        "\u{F9E6}" => "\u{7F79}",
        "\u{2F976}" => "\u{7F7A}",
        "\u{F90F}" => "\u{7F85}",
        "\u{2F977}" => "\u{2633E}",
        "\u{2F7A}" => "\u{7F8A}",
        "\u{2F978}" => "\u{7F95}",
        "\u{F9AF}" => "\u{7F9A}",
        "\u{FA1E}" => "\u{7FBD}",
        "\u{2F7B}" => "\u{7FBD}",
        "\u{2F979}" => "\u{7FFA}",
        "\u{F934}" => "\u{8001}",
        "\u{2F7C}" => "\u{8001}",
        "\u{2EB9}" => "\u{8002}",
        "\u{FA5B}" => "\u{8005}",
        "\u{FAB2}" => "\u{8005}",
        "\u{2F97A}" => "\u{8005}",
        "\u{2F7D}" => "\u{800C}",
        "\u{2F97B}" => "\u{264DA}",
        "\u{2F7E}" => "\u{8012}",
        "\u{2F97C}" => "\u{26523}",
        "\u{2F7F}" => "\u{8033}",
        "\u{F9B0}" => "\u{8046}",
        "\u{2F97D}" => "\u{8060}",
        "\u{2F97E}" => "\u{265A8}",
        "\u{F997}" => "\u{806F}",
        "\u{2F97F}" => "\u{8070}",
        "\u{F945}" => "\u{807E}",
        "\u{2F80}" => "\u{807F}",
        "\u{2EBA}" => "\u{8080}",
        "\u{2F81}" => "\u{8089}",
        "\u{F953}" => "\u{808B}",
        "\u{2F8D6}" => "\u{80AD}",
        "\u{2F982}" => "\u{80B2}",
        "\u{2F981}" => "\u{43D5}",
        "\u{2F8D7}" => "\u{43D9}",
        "\u{8141}" => "\u{80FC}",
        "\u{2F983}" => "\u{8103}",
        "\u{2F985}" => "\u{813E}",
        "\u{2F984}" => "\u{440B}",
        "\u{2F8DA}" => "\u{6721}",
        "\u{2F987}" => "\u{267A7}",
        "\u{2F988}" => "\u{267B5}",
        "\u{6726}" => "\u{4443}",
        "\u{F926}" => "\u{81D8}",
        "\u{2F82}" => "\u{81E3}",
        "\u{F9F6}" => "\u{81E8}",
        "\u{2F83}" => "\u{81EA}",
        "\u{FA5C}" => "\u{81ED}",
        "\u{2F84}" => "\u{81F3}",
        "\u{2F85}" => "\u{81FC}",
        "\u{2F893}" => "\u{8201}",
        "\u{2F98B}" => "\u{8201}",
        "\u{2F98C}" => "\u{8204}",
        "\u{2F86}" => "\u{820C}",
        "\u{FA6D}" => "\u{8218}",
        "\u{2F87}" => "\u{821B}",
        "\u{2F88}" => "\u{821F}",
        "\u{2F98E}" => "\u{446B}",
        "\u{2F89}" => "\u{826E}",
        "\u{F97C}" => "\u{826F}",
        "\u{2F8A}" => "\u{8272}",
        "\u{2F8B}" => "\u{8278}",
        "\u{FA5D}" => "\u{8279}",
        "\u{FA5E}" => "\u{8279}",
        "\u{2EBE}" => "\u{8279}",
        "\u{2EBF}" => "\u{8279}",
        "\u{2EC0}" => "\u{8279}",
        "\u{2F990}" => "\u{828B}",
        "\u{2F98F}" => "\u{8291}",
        "\u{2F991}" => "\u{829D}",
        "\u{2F993}" => "\u{82B1}",
        "\u{2F994}" => "\u{82B3}",
        "\u{2F995}" => "\u{82BD}",
        "\u{F974}" => "\u{82E5}",
        "\u{2F998}" => "\u{82E5}",
        "\u{2F996}" => "\u{82E6}",
        "\u{2F997}" => "\u{26B3C}",
        "\u{F9FE}" => "\u{8336}",
        "\u{FAB3}" => "\u{8352}",
        "\u{2F99A}" => "\u{8363}",
        "\u{2F999}" => "\u{831D}",
        "\u{2F99C}" => "\u{8323}",
        "\u{2F99D}" => "\u{83BD}",
        "\u{2F9A0}" => "\u{8353}",
        "\u{F93E}" => "\u{83C9}",
        "\u{2F9A1}" => "\u{83CA}",
        "\u{2F9A2}" => "\u{83CC}",
        "\u{2F9A3}" => "\u{83DC}",
        "\u{2F99E}" => "\u{83E7}",
        "\u{FAB4}" => "\u{83EF}",
        "\u{F958}" => "\u{83F1}",
        "\u{FA5F}" => "\u{8457}",
        "\u{2F99F}" => "\u{8457}",
        "\u{2F9A4}" => "\u{26C36}",
        "\u{2F99B}" => "\u{83AD}",
        "\u{F918}" => "\u{843D}",
        "\u{F96E}" => "\u{8449}",
        "\u{853F}" => "\u{848D}",
        "\u{2F9A6}" => "\u{26CD5}",
        "\u{2F9A5}" => "\u{26D6B}",
        "\u{F999}" => "\u{84EE}",
        "\u{2F9A8}" => "\u{84F1}",
        "\u{2F9A9}" => "\u{84F3}",
        "\u{F9C2}" => "\u{84FC}",
        "\u{2F9AA}" => "\u{8516}",
        "\u{2F9A7}" => "\u{452B}",
        "\u{2F9AC}" => "\u{8564}",
        "\u{2F9AD}" => "\u{26F2C}",
        "\u{F923}" => "\u{85CD}",
        "\u{2F9AE}" => "\u{455D}",
        "\u{2F9B0}" => "\u{26FB1}",
        "\u{2F9AF}" => "\u{4561}",
        "\u{F9F0}" => "\u{85FA}",
        "\u{F935}" => "\u{8606}",
        "\u{2F9B2}" => "\u{456B}",
        "\u{FA20}" => "\u{8612}",
        "\u{F91F}" => "\u{862D}",
        "\u{2F9B1}" => "\u{270D2}",
        "\u{8641}" => "\u{8637}",
        "\u{F910}" => "\u{863F}",
        "\u{2F8C}" => "\u{864D}",
        "\u{2EC1}" => "\u{864E}",
        "\u{2F9B3}" => "\u{8650}",
        "\u{F936}" => "\u{865C}",
        "\u{2F9B4}" => "\u{865C}",
        "\u{2F9B5}" => "\u{8667}",
        "\u{2F9B6}" => "\u{8669}",
        "\u{2F8D}" => "\u{866B}",
        "\u{2F9B7}" => "\u{86A9}",
        "\u{2F9B8}" => "\u{8688}",
        "\u{2F9BA}" => "\u{86E2}",
        "\u{2F9B9}" => "\u{870E}",
        "\u{2F9BC}" => "\u{8728}",
        "\u{2F9BD}" => "\u{876B}",
        "\u{2F9C0}" => "\u{87E1}",
        "\u{FAB5}" => "\u{8779}",
        "\u{2F9BB}" => "\u{8779}",
        "\u{2F9BE}" => "\u{8786}",
        "\u{2F9BF}" => "\u{45D7}",
        "\u{2F9AB}" => "\u{273CA}",
        "\u{F911}" => "\u{87BA}",
        "\u{2F9C1}" => "\u{8801}",
        "\u{2F9C2}" => "\u{45F9}",
        "\u{F927}" => "\u{881F}",
        "\u{2F8E}" => "\u{8840}",
        "\u{FA08}" => "\u{884C}",
        "\u{2F8F}" => "\u{884C}",
        "\u{2F9C3}" => "\u{8860}",
        "\u{2F9C4}" => "\u{8863}",
        "\u{2F90}" => "\u{8863}",
        "\u{2EC2}" => "\u{8864}",
        "\u{F9A0}" => "\u{88C2}",
        "\u{2F9C5}" => "\u{27667}",
        "\u{F9E7}" => "\u{88CF}",
        "\u{2F9C6}" => "\u{88D7}",
        "\u{2F9C7}" => "\u{88DE}",
        "\u{F9E8}" => "\u{88E1}",
        "\u{F912}" => "\u{88F8}",
        "\u{2F9C9}" => "\u{88FA}",
        "\u{2F9C8}" => "\u{4635}",
        "\u{FA60}" => "\u{8910}",
        "\u{FAB6}" => "\u{8941}",
        "\u{F924}" => "\u{8964}",
        "\u{2F91}" => "\u{897E}",
        "\u{2EC4}" => "\u{897F}",
        "\u{2EC3}" => "\u{8980}",
        "\u{FAB7}" => "\u{8986}",
        "\u{FA0A}" => "\u{898B}",
        "\u{2F92}" => "\u{898B}",
        "\u{2F9CB}" => "\u{278AE}",
        "\u{2EC5}" => "\u{89C1}",
        "\u{2F93}" => "\u{89D2}",
        "\u{2F94}" => "\u{8A00}",
        "\u{2F9CC}" => "\u{27966}",
        "\u{8A7D}" => "\u{8A2E}",
        "\u{8A1E}" => "\u{46B6}",
        "\u{2F9CD}" => "\u{46BE}",
        "\u{2F9CE}" => "\u{46C7}",
        "\u{2F9CF}" => "\u{8AA0}",
        "\u{F96F}" => "\u{8AAA}",
        "\u{F9A1}" => "\u{8AAA}",
        "\u{FAB9}" => "\u{8ABF}",
        "\u{FABB}" => "\u{8ACB}",
        "\u{F97D}" => "\u{8AD2}",
        "\u{F941}" => "\u{8AD6}",
        "\u{FABE}" => "\u{8AED}",
        "\u{2F9D0}" => "\u{8AED}",
        "\u{FA22}" => "\u{8AF8}",
        "\u{FABA}" => "\u{8AF8}",
        "\u{F95D}" => "\u{8AFE}",
        "\u{FABD}" => "\u{8AFE}",
        "\u{FA62}" => "\u{8B01}",
        "\u{FABC}" => "\u{8B01}",
        "\u{FA63}" => "\u{8B39}",
        "\u{FABF}" => "\u{8B39}",
        "\u{F9FC}" => "\u{8B58}",
        "\u{F95A}" => "\u{8B80}",
        "\u{8B8F}" => "\u{8B86}",
        "\u{FAC0}" => "\u{8B8A}",
        "\u{2F9D1}" => "\u{8B8A}",
        "\u{2EC8}" => "\u{8BA0}",
        "\u{2F95}" => "\u{8C37}",
        "\u{2F96}" => "\u{8C46}",
        "\u{F900}" => "\u{8C48}",
        "\u{2F9D2}" => "\u{8C55}",
        "\u{2F97}" => "\u{8C55}",
        "\u{8C63}" => "\u{8C5C}",
        "\u{2F98}" => "\u{8C78}",
        "\u{2F9D3}" => "\u{27CA8}",
        "\u{2F99}" => "\u{8C9D}",
        "\u{2F9D4}" => "\u{8CAB}",
        "\u{2F9D5}" => "\u{8CC1}",
        "\u{F948}" => "\u{8CC2}",
        "\u{F903}" => "\u{8CC8}",
        "\u{FA64}" => "\u{8CD3}",
        "\u{FA65}" => "\u{8D08}",
        "\u{FAC1}" => "\u{8D08}",
        "\u{2F9D6}" => "\u{8D1B}",
        "\u{2EC9}" => "\u{8D1D}",
        "\u{2F9A}" => "\u{8D64}",
        "\u{2F9B}" => "\u{8D70}",
        "\u{2F9D7}" => "\u{8D77}",
        "\u{8D86}" => "\u{8D7F}",
        "\u{FAD7}" => "\u{27ED3}",
        "\u{2F9D8}" => "\u{27F2F}",
        "\u{2F9C}" => "\u{8DB3}",
        "\u{2F9DA}" => "\u{8DCB}",
        "\u{2F9DB}" => "\u{8DBC}",
        "\u{8DFA}" => "\u{8DE5}",
        "\u{F937}" => "\u{8DEF}",
        "\u{2F9DC}" => "\u{8DF0}",
        "\u{8E9B}" => "\u{8E97}",
        "\u{2F9D}" => "\u{8EAB}",
        "\u{F902}" => "\u{8ECA}",
        "\u{2F9E}" => "\u{8ECA}",
        "\u{2F9DE}" => "\u{8ED4}",
        "\u{8F27}" => "\u{8EFF}",
        "\u{F998}" => "\u{8F26}",
        "\u{F9D7}" => "\u{8F2A}",
        "\u{FAC2}" => "\u{8F38}",
        "\u{2F9DF}" => "\u{8F38}",
        "\u{FA07}" => "\u{8F3B}",
        "\u{F98D}" => "\u{8F62}",
        "\u{2ECB}" => "\u{8F66}",
        "\u{2F9F}" => "\u{8F9B}",
        "\u{2F98D}" => "\u{8F9E}",
        "\u{F971}" => "\u{8FB0}",
        "\u{2FA0}" => "\u{8FB0}",
        "\u{2FA1}" => "\u{8FB5}",
        "\u{FA66}" => "\u{8FB6}",
        "\u{2ECC}" => "\u{8FB6}",
        "\u{2ECD}" => "\u{8FB6}",
        "\u{2F881}" => "\u{5DE1}",
        "\u{F99A}" => "\u{9023}",
        "\u{FA25}" => "\u{9038}",
        "\u{FA67}" => "\u{9038}",
        "\u{FAC3}" => "\u{9072}",
        "\u{F9C3}" => "\u{907C}",
        "\u{2F9E0}" => "\u{285D2}",
        "\u{2F9E1}" => "\u{285ED}",
        "\u{F913}" => "\u{908F}",
        "\u{2FA2}" => "\u{9091}",
        "\u{2F9E2}" => "\u{9094}",
        "\u{F92C}" => "\u{90CE}",
        "\u{90DE}" => "\u{90CE}",
        "\u{FA2E}" => "\u{90CE}",
        "\u{2F9E3}" => "\u{90F1}",
        "\u{FA26}" => "\u{90FD}",
        "\u{2F9E5}" => "\u{2872E}",
        "\u{2F9E4}" => "\u{9111}",
        "\u{2F9E6}" => "\u{911B}",
        "\u{2FA3}" => "\u{9149}",
        "\u{F919}" => "\u{916A}",
        "\u{FAC4}" => "\u{9199}",
        "\u{F9B7}" => "\u{91B4}",
        "\u{2FA4}" => "\u{91C6}",
        "\u{F9E9}" => "\u{91CC}",
        "\u{2FA5}" => "\u{91CC}",
        "\u{F97E}" => "\u{91CF}",
        "\u{F90A}" => "\u{91D1}",
        "\u{2FA6}" => "\u{91D1}",
        "\u{F9B1}" => "\u{9234}",
        "\u{2F9E7}" => "\u{9238}",
        "\u{FAC5}" => "\u{9276}",
        "\u{2F9E8}" => "\u{92D7}",
        "\u{2F9E9}" => "\u{92D8}",
        "\u{2F9EA}" => "\u{927C}",
        "\u{F93F}" => "\u{9304}",
        "\u{F99B}" => "\u{934A}",
        "\u{93AE}" => "\u{93AD}",
        "\u{2F9EB}" => "\u{93F9}",
        "\u{2F9EC}" => "\u{9415}",
        "\u{2F9ED}" => "\u{28BFA}",
        "\u{2ED0}" => "\u{9485}",
        "\u{2ED1}" => "\u{9577}",
        "\u{2FA7}" => "\u{9577}",
        "\u{2ED2}" => "\u{9578}",
        "\u{2ED3}" => "\u{957F}",
        "\u{2FA8}" => "\u{9580}",
        "\u{2F9EE}" => "\u{958B}",
        "\u{2F9EF}" => "\u{4995}",
        "\u{F986}" => "\u{95AD}",
        "\u{2F9F0}" => "\u{95B7}",
        "\u{2F9F1}" => "\u{28D77}",
        "\u{2ED4}" => "\u{95E8}",
        "\u{2FA9}" => "\u{961C}",
        "\u{2ECF}" => "\u{961D}",
        "\u{2ED6}" => "\u{961D}",
        "\u{F9C6}" => "\u{962E}",
        "\u{F951}" => "\u{964B}",
        "\u{FA09}" => "\u{964D}",
        "\u{F959}" => "\u{9675}",
        "\u{F9D3}" => "\u{9678}",
        "\u{FAC6}" => "\u{967C}",
        "\u{F9DC}" => "\u{9686}",
        "\u{F9F1}" => "\u{96A3}",
        "\u{2F9F2}" => "\u{49E6}",
        "\u{2FAA}" => "\u{96B6}",
        "\u{FA2F}" => "\u{96B7}",
        "\u{96B8}" => "\u{96B7}",
        "\u{F9B8}" => "\u{96B7}",
        "\u{2FAB}" => "\u{96B9}",
        "\u{2F9F3}" => "\u{96C3}",
        "\u{F9EA}" => "\u{96E2}",
        "\u{FA68}" => "\u{96E3}",
        "\u{FAC7}" => "\u{96E3}",
        "\u{2FAC}" => "\u{96E8}",
        "\u{F9B2}" => "\u{96F6}",
        "\u{F949}" => "\u{96F7}",
        "\u{2F9F5}" => "\u{9723}",
        "\u{2F9F6}" => "\u{29145}",
        "\u{F938}" => "\u{9732}",
        "\u{F9B3}" => "\u{9748}",
        "\u{2FAD}" => "\u{9751}",
        "\u{2ED8}" => "\u{9752}",
        "\u{FA1C}" => "\u{9756}",
        "\u{FAC8}" => "\u{9756}",
        "\u{2F81C}" => "\u{291DF}",
        "\u{2FAE}" => "\u{975E}",
        "\u{2FAF}" => "\u{9762}",
        "\u{2F9F7}" => "\u{2921A}",
        "\u{2FB0}" => "\u{9769}",
        "\u{2F9F8}" => "\u{4A6E}",
        "\u{2F9F9}" => "\u{4A76}",
        "\u{2FB1}" => "\u{97CB}",
        "\u{FAC9}" => "\u{97DB}",
        "\u{2F9FA}" => "\u{97E0}",
        "\u{2ED9}" => "\u{97E6}",
        "\u{2FB2}" => "\u{97ED}",
        "\u{2F9FB}" => "\u{2940A}",
        "\u{2FB3}" => "\u{97F3}",
        "\u{FA69}" => "\u{97FF}",
        "\u{FACA}" => "\u{97FF}",
        "\u{2FB4}" => "\u{9801}",
        "\u{2F9FC}" => "\u{4AB2}",
        "\u{FACB}" => "\u{980B}",
        "\u{2F9FE}" => "\u{980B}",
        "\u{2F9FF}" => "\u{980B}",
        "\u{F9B4}" => "\u{9818}",
        "\u{2FA00}" => "\u{9829}",
        "\u{2F9FD}" => "\u{29496}",
        "\u{FA6A}" => "\u{983B}",
        "\u{FACC}" => "\u{983B}",
        "\u{F9D0}" => "\u{985E}",
        "\u{2EDA}" => "\u{9875}",
        "\u{2FB5}" => "\u{98A8}",
        "\u{2FA01}" => "\u{295B6}",
        "\u{2EDB}" => "\u{98CE}",
        "\u{2FB6}" => "\u{98DB}",
        "\u{2EDC}" => "\u{98DE}",
        "\u{2EDD}" => "\u{98DF}",
        "\u{2FB7}" => "\u{98DF}",
        "\u{2EDF}" => "\u{98E0}",
        "\u{2FA02}" => "\u{98E2}",
        "\u{FA2A}" => "\u{98EF}",
        "\u{FA2B}" => "\u{98FC}",
        "\u{2FA03}" => "\u{4B33}",
        "\u{FA2C}" => "\u{9928}",
        "\u{2FA04}" => "\u{9929}",
        "\u{2EE0}" => "\u{9963}",
        "\u{2FB8}" => "\u{9996}",
        "\u{2FB9}" => "\u{9999}",
        "\u{2FA05}" => "\u{99A7}",
        "\u{2FBA}" => "\u{99AC}",
        "\u{2FA06}" => "\u{99C2}",
        "\u{F91A}" => "\u{99F1}",
        "\u{2FA07}" => "\u{99FE}",
        "\u{F987}" => "\u{9A6A}",
        "\u{2EE2}" => "\u{9A6C}",
        "\u{2FBB}" => "\u{9AA8}",
        "\u{2FA08}" => "\u{4BCE}",
        "\u{2FBC}" => "\u{9AD8}",
        "\u{2FBD}" => "\u{9ADF}",
        "\u{2FA09}" => "\u{29B30}",
        "\u{FACD}" => "\u{9B12}",
        "\u{2FA0A}" => "\u{9B12}",
        "\u{2FBE}" => "\u{9B25}",
        "\u{2FBF}" => "\u{9B2F}",
        "\u{2FC0}" => "\u{9B32}",
        "\u{2FC1}" => "\u{9B3C}",
        "\u{2EE4}" => "\u{9B3C}",
        "\u{2FC2}" => "\u{9B5A}",
        "\u{F939}" => "\u{9B6F}",
        "\u{2FA0B}" => "\u{9C40}",
        "\u{F9F2}" => "\u{9C57}",
        "\u{2EE5}" => "\u{9C7C}",
        "\u{2FC3}" => "\u{9CE5}",
        "\u{2FA0C}" => "\u{9CFD}",
        "\u{2FA0D}" => "\u{4CCE}",
        "\u{2FA0F}" => "\u{9D67}",
        "\u{2FA0E}" => "\u{4CED}",
        "\u{2FA10}" => "\u{2A0CE}",
        "\u{FA2D}" => "\u{9DB4}",
        "\u{2FA12}" => "\u{2A105}",
        "\u{2FA11}" => "\u{4CF8}",
        "\u{F93A}" => "\u{9DFA}",
        "\u{2FA13}" => "\u{2A20E}",
        "\u{F920}" => "\u{9E1E}",
        "\u{9E43}" => "\u{9E42}",
        "\u{2FC4}" => "\u{9E75}",
        "\u{F940}" => "\u{9E7F}",
        "\u{2FC5}" => "\u{9E7F}",
        "\u{2FA14}" => "\u{2A291}",
        "\u{F988}" => "\u{9E97}",
        "\u{F9F3}" => "\u{9E9F}",
        "\u{2FC6}" => "\u{9EA5}",
        "\u{2EE8}" => "\u{9EA6}",
        "\u{2FA15}" => "\u{9EBB}",
        "\u{2FC7}" => "\u{9EBB}",
        "\u{2F88F}" => "\u{2A392}",
        "\u{2FC8}" => "\u{9EC3}",
        "\u{2EE9}" => "\u{9EC4}",
        "\u{2FC9}" => "\u{9ECD}",
        "\u{F989}" => "\u{9ECE}",
        "\u{2FA16}" => "\u{4D56}",
        "\u{2FCA}" => "\u{9ED1}",
        "\u{9ED2}" => "\u{9ED1}",
        "\u{FA3A}" => "\u{58A8}",
        "\u{2FA17}" => "\u{9EF9}",
        "\u{2FCB}" => "\u{9EF9}",
        "\u{2FCC}" => "\u{9EFD}",
        "\u{2FA19}" => "\u{9F05}",
        "\u{2FA18}" => "\u{9EFE}",
        "\u{2FCD}" => "\u{9F0E}",
        "\u{2FA1A}" => "\u{9F0F}",
        "\u{2FCE}" => "\u{9F13}",
        "\u{2FA1B}" => "\u{9F16}",
        "\u{2FCF}" => "\u{9F20}",
        "\u{2FA1C}" => "\u{9F3B}",
        "\u{2FD0}" => "\u{9F3B}",
        "\u{FAD8}" => "\u{9F43}",
        "\u{2FD1}" => "\u{9F4A}",
        "\u{2EEC}" => "\u{9F50}",
        "\u{2FD2}" => "\u{9F52}",
        "\u{2FA1D}" => "\u{2A600}",
        "\u{2EEE}" => "\u{9F7F}",
        "\u{F9C4}" => "\u{9F8D}",
        "\u{2FD3}" => "\u{9F8D}",
        "\u{FAD9}" => "\u{9F8E}",
        "\u{2EF0}" => "\u{9F99}",
        "\u{F907}" => "\u{9F9C}",
        "\u{F908}" => "\u{9F9C}",
        "\u{FACE}" => "\u{9F9C}",
        "\u{2FD4}" => "\u{9F9C}",
        "\u{2EF3}" => "\u{9F9F}",
        "\u{2FD5}" => "\u{9FA0}",
        ];

    /**
     * Метод для нормализации имени пользователя
     */
    public function normUsername(string $str): string
    {
        $str = \strtr($str, $this->confusables);
        return \preg_replace('%[^\p{L}\p{N}]+%u', '', $str);
    }
}
